%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% radix4.m
%
% Test with radix 4 algorithm
%
% By Magnus Nilsson 2000-12-23
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
N = 64;
time = 1:N;
t = log(N)/log(4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Chebychev windowing 
win = chebwin(N,60);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Signal
x = win'.*(sin(2*pi*0.25*time)+sin(2*pi*0.38*time));
xr = real(x);
xi = imag(x);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Radix-4 Algorithm

ss = N/4;
count = 0;
for q = 1:t
    L = 4^q;
    r = N/L;
    Lx = L/4;
    rx = 4*r;
    yr = xr;
    yi = xi;
    for j = 0:Lx-1
        for k = 0:r-1
            in0r = yr(j*rx + k + 1);
            in0i = yi(j*rx + k + 1);
            in1r = yr(j*rx + r + k + 1);
            in1i = yi(j*rx + r + k + 1);
            in2r = yr(j*rx + 2*r + k + 1);
            in2i = yi(j*rx + 2*r + k + 1);
            in3r = yr(j*rx + 3*r + k + 1);
            in3i = yi(j*rx + 3*r + k + 1);

            ar = in0r;
            ai = in0i;
            br = in1r*cos(2*pi*j/L) + in1i*sin(2*pi*j/L);
            bi = in1i*cos(2*pi*j/L) - in1r*sin(2*pi*j/L);
            cr = in2r*cos(2*pi*2*j/L) + in2i*sin(2*pi*2*j/L);
            ci = in2i*cos(2*pi*2*j/L) - in2r*sin(2*pi*2*j/L);
            dr = in3r*cos(2*pi*3*j/L) + in3i*sin(2*pi*3*j/L);
            di = in3i*cos(2*pi*3*j/L) - in3r*sin(2*pi*3*j/L);


      

            to = 10+(q-1)*2;

           
            aa(1,:) = dec2bin(round(cos(2*pi*1*j/L)*2^to));
            aa(2,:) = dec2bin(round(sin(2*pi*1*j/L)*2^to));
            aa(3,:) = dec2bin(round(cos(2*pi*2*j/L)*2^to));
            aa(4,:) = dec2bin(round(sin(2*pi*2*j/L)*2^to));
            aa(5,:) = dec2bin(round(cos(2*pi*3*j/L)*2^to));
            aa(6,:) = dec2bin(round(sin(2*pi*3*j/L)*2^to));

            sprintf('Twiddlefactor for step %d block %d is:', q, count)
            Twiddlefactor = aa
            count = count + 1;
            clear aa;

            temp1r = ar + cr;
            temp1i = ai + ci;
            
            temp2r = ar - cr;
            temp2i = ai - ci;

            Ar = temp1r + br + dr;
            Ai = temp1i + bi + di;
	    Br = temp2r + bi - di;
	    Bi = temp2i - br + dr;
	    Cr = temp1r - br - dr;
	    Ci = temp1i - bi - di;
	    Dr = temp2r - bi + di;
	    Di = temp2i + br - dr;
	    
            xr(j*r + k + 1) = Ar;
            xi(j*r + k + 1) = Ai;
            xr((j + Lx)*r + k + 1) = Br;
            xi((j + Lx)*r + k + 1) = Bi; 
            xr((j + 2*Lx)*r + k + 1) = Cr;
            xi((j + 2*Lx)*r + k + 1) = Ci; 
            xr((j + 3*Lx)*r + k + 1) = Dr;
            xi((j + 3*Lx)*r + k + 1) = Di;
        end
    end
end 

x_complex = xr+i*xi;
	    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);
figure;
hold on;
plot(l,20*log10(abs(x_complex)),'r+');
plot(l,20*log10(abs(fft(x))));
title('Radix-4 Complex algorithm and MATLAB FFT');
legend('Radix-4','Matlab FFT',2);
axis([0 N-1 20*log10(min(abs(x_complex))) 20*log10(max(abs(x_complex)))]);
xlabel('n');
ylabel('dB');
zoom on;
grid on;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--  
%-------------------------------------------------------------------------
%--           Copyright ERICSSON MICROWAVE SYSTEMS AB 1998              --
%-------------------------------------------------------------------------
%-- 
%-- 
%-- 
%-- The copyright to the source program herein is the
%-- property of Ericsson Microwave Systems AB, Sweden.
%-- The program may be used and/or copied only with the
%-- written permission from Ericsson Microwave Systems
%-- AB or in accordance with the terms and conditions 
%-- stipulated in the agreement/contract under which 
%-- the program have been supplied.
%-- 
%--  
%-------------------------------------------------------------------------
%-------------------------------------------------------------------------
%--  
%-- Project     : NORA_NCRg1
%-- Title       : Generate sinus to test_pattern
%-- File name   : sinus_gen.m
%-- Document no : -
%-- Revision    : pa1
%-- Date        : 00-06-07
%-- Prepared    : FX/D Daniel Wallstrm
%-- Responsible : FX/DC Hkan Enskog
%-- Description : Utdata: 64 st 14bit data, sinus.
%--
%--
%-- ------------------------------------------------------------------------
%-- Revision history: (all revisions included)                            --
%-- ------------------------------------------------------------------------
%-- Rev:   Initials:     Date:    Modification:                           --
%-- ------------------------------------------------------------------------
%-- pa1     emwdawa     00-06-07   First revision                          --
%-- ------------------------------------------------------------------------
%--                                                                       --
%-- ------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------- Invariabler ---------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
%indatafile='../modelsim/env_utfil.txt';                     %%%%%%%%% Test data

no_of_data=16;
no_of_bits=17;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------- Utdata----- ---------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

utdatafile_real='dwc_mem_16_17_re.mif';
utdatafile_imag='dwc_mem_16_17_im.mif';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------------------------ Generera data ---------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = (1:no_of_data)';

DATA = ((2^(no_of_bits -1))-1) *( cos((2*pi*3*N)/16) + i*sin((2*pi*3*N)/16) );

DATA = round(DATA);


DATA_real_plot = real(DATA);
DATA_imag_plot = imag(DATA);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--------------------- OMRKNING TILL TV-KOMPLEMENT --------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for k=1:length(DATA)

   dec_real=real(DATA(k));       % Dela upp DATA i en realdel och en imaginrdel.
%   dec_real=-      
   dec_imag=imag(DATA(k));

   
        if (dec_real>2^(no_of_bits-0)-1)|(dec_real<-(2^(no_of_bits-0)))      
                                                      % ndrat frn (no_of_bits-1) till (no_of_bits-0).
                error('too big decimal input')
        end

        dummy_real=blanks(no_of_bits);         % Skapar en tom strng.
        dummy_imag=blanks(no_of_bits);         % Skapar en tom strng.


        dummy_real(1:no_of_bits)='0';
        dummy_imag(1:no_of_bits)='0';

        if (dec_real<0);                      % Letar upp alla tal mindre n 0.
           dec_real=2^no_of_bits+dec_real;    % Tv-komp. omv.
        end

        if (dec_imag<0);                      % Letar upp alla tal mindre n 0.
           dec_imag=2^no_of_bits+dec_imag;    % Tv-komp. omv.
        end

        bin_real=dec2bin(dec_real);

        bin_imag=dec2bin(dec_imag);

        dummy_real(no_of_bits-length(bin_real)+1:no_of_bits)=bin_real;

        dummy_imag(no_of_bits-length(bin_imag)+1:no_of_bits)=bin_imag;


%%%%
        test_dec = bin2dec(dummy_real)
        if (test_dec >  65535);                      % Letar upp alla tal mindre n 0.
           test_real=65535-test_dec                  % Tv-komp. omv.
        end

%        test_dec_real=bin2dec(bin_real)
%%%%



        DATA_real(k,:)=dummy_real;

        DATA_imag(k,:)=dummy_imag;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------------------------ Skriv till fil --------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fp_re=fopen(utdatafile_real,'w');              % ; = att inte skriva "ut".
fp_im=fopen(utdatafile_imag,'w');              % ; = att inte skriva "ut".

[a,b]=size(DATA_real);

for r=1:a

 fprintf(fp_re, '%s\n',DATA_real(r,[1:b]));
 fprintf(fp_im, '%s\n',DATA_imag(r,[1:b]));

end;

fclose(fp_re);
fclose(fp_im);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------------------------ Ls in fil ------------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------- Plottning ---------------------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DQ=DQ(1:296);
%DI=DI(1:296);

%plot_data=DATA(1,1:12); %no_of_data-1);
plot_data_re=DATA_real_plot(1:no_of_data);
plot_data_im=DATA_imag_plot(1:no_of_data);


%DI=dat(1,1:no_of_bytes-1);
%ENV=dat(2,1:no_of_bytes-1);
%DQ=dat(3,1:no_of_bytes-0);

figure(1)
plot(plot_data_re,'b')
hold on
plot(plot_data_im,'r')
title('DOWNCONVERT')
hold off
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(2)
xx=1:length(DATA);
zz=(xx-length(xx)/2-1)/length(xx)*37.5;
yfft=fftshift(fft(randn(length(DATA),1)+DATA)/length(DATA));
plot(zz(xx),20*log10(abs(yfft(xx))));grid on;
xlabel('FREQUENCY [MHz]');
ylabel('AMPLITUDE [dB]');
title('');
