%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% radix2_2.m
%
% Radix 2 algorithm
%
% By Magnus Nilsson 2000-11-27
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.  
t = 1:1:1024;
x = sin(2*pi*0.35*t)+sin(2*pi*0.25*t);
N = length(x);
b = bin2dec(fliplr(dec2bin(0:1:length(x)-1)))+1;
MC = x(b); % Make in bit reversed order
alfa = N/2;
beta = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate Twiddle factor
for n = 1:N/2
   W(n) = exp(-j*2*pi*(n-1)/N);
   W_r(n) = cos(2*pi*(n-1)/N);
   W_i(n) = -sin(2*pi*(n-1)/N);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate FFT using inplace non-recursive DIT FFT, radix-2
for h = 1:(log(N)/log(2))
    b = 2^(h-1);
    a = 1;
    aO = 1; 
    for d = 1:alfa
        c = 1;
        for e = 1:beta
            a+b;
            temp1 = W(c)*MC(a+b);
            temp2 = MC(a);
            MC(a) = MC(a) + temp1;
            MC(a+b) = temp2 - temp1;
            a = a + 1;
            c = c + alfa;            
        end
        a = aO + 2^(h);
        aO = a;
    end
    alfa = alfa/2;
    beta = beta*2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);
figure;
subplot(2,1,1), plot(l,abs(MC));
title('Radix-2 FFT algorithm');
xlabel('N');
legend('y(x) = sin(2*pi*0.35*t)+sin(2*pi*0.25*t)',2)
axis([0 N/2 0 1.2*max(MC)]);
subplot(2,1,2), plot(l,abs(fft(x)));
title('MATLAB FFT algorithm');
xlabel('N');
legend('y(x) = sin(2*pi*0.35*t)+sin(2*pi*0.25*t)',2)
axis([0 N/2 0 1.2*max(MC)]);

