%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--  
%-------------------------------------------------------------------------
%--           Copyright ERICSSON MICROWAVE SYSTEMS AB 2001              --
%-------------------------------------------------------------------------
%-- 
%-- 
%-- 
%-- The copyright to the source program herein is the
%-- property of Ericsson Microwave Systems AB, Sweden.
%-- The program may be used and/or copied only with the
%-- written permission from Ericsson Microwave Systems
%-- AB or in accordance with the terms and conditions 
%-- stipulated in the agreement/contract under which 
%-- the program have been supplied.
%-- 
%--  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--  
%-- Project     : FFT
%-- Title       : Radix 2
%-- File name   : radix2_ImRe.m
%-- Document no : -
%-- Revision    : pa1
%-- Date        : 00-11-28
%-- Prepared    : FX/D Magnus Nilsson
%-- Responsible : FX/DC Anders Wanner
%-- Description : Test with radix 2 algorithm
%--
%-- ------------------------------------------------------------------------
%-- Revision history: (all revisions included)                            --
%-- ------------------------------------------------------------------------
%-- Rev:   Initials:     Date:    Modification:                           --
%-- ------------------------------------------------------------------------
%-- pa1     emwmsnn     00-11-28   First revision                         --
%-- ------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
x_r = [2 1 4 5 7 9 6 3];  
x_i = [0 0 0 0 0 0 0 0];
%t = 1:1:1024;
%x = sin(2*pi*0.35*t)+sin(2*pi*0.25*t);
N = length(x_r);
b = bin2dec(fliplr(dec2bin(0:1:length(x_r)-1)))+1;
MC_r = x_r(b); % Make in bit reversed order real-part
MC_i = x_i(b); % Make in bit reversed order im-part
alfa = N/2;
beta = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate Twiddle factor
for n = 1:N/2
   W(n) = exp(-j*2*pi*(n-1)/N);
   W_r(n) = cos(2*pi*(n-1)/N);
   W_i(n) = -sin(2*pi*(n-1)/N);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate FFT using inplace non-recursive DIT FFT, radix-2
for h = 1:(log(N)/log(2))
    b = 2^(h-1);
    a = 1;
    aO = 1; 
    for d = 1:alfa
        c = 1;
        for e = 1:beta
            a+b;
 
            temp1_r = W_r(c)*MC_r(a+b)-W_i(c)*MC_i(a+b);
            temp1_i = W_r(c)*MC_i(a+b)+W_i(c)*MC_r(a+b);

            temp2_r = MC_r(a);
            temp2_i = MC_i(a);

            MC_r(a) = MC_r(a) + temp1_r;
            MC_i(a) = MC_i(a) + temp1_i;

            MC_r(a+b) = temp2_r - temp1_r;
            MC_i(a+b) = temp2_i - temp1_i;

            a = a + 1;
            c = c + alfa;            
        end
        a = aO + 2^(h);
        aO = a;
    end
    alfa = alfa/2;
    beta = beta*2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);
figure;
subplot(2,1,1), plot(l,abs(MC_r+MC_i*j));
%axis([0 N/2 0 1.2*max(MC_r)]);
subplot(2,1,2), plot(l,abs(fft(x_r)));
%axis([0 N/2 0 1.2*max(MC_r)]);
