%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% radix4.m
%
% Test with radix 4 algorithm
%
% By Magnus Nilsson 2000-11-30
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
t = 1:1:256;
x = sin(2*pi*0.35*t)+sin(2*pi*0.38*t);
x1 = x;
n = length(x);
t = log(n)/log(4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Radix-4 Algorithm

for q = 1:t
    L = 4^q;
    r = n/L;
    Lx = L/4;
    rx = 4*r;
    y = x;
    for j = 0:Lx-1
        for k = 0:r-1
            a = y(j*rx + k + 1);
            b = exp(-i*2*pi*j/L)*y(j*rx + r + k + 1);
            c = exp(-i*2*pi*2*j/L)*y(j*rx + 2*r + k + 1);
            d = exp(-i*2*pi*3*j/L)*y(j*rx + 3*r + k + 1);
            t0 = a + c;
            t1 = a - c;
            t2 = b + d;
            t3 = b - d;
            x(j*r + k + 1) = t0 + t2;
            x((j + Lx)*r + k + 1) = t1 - i*t3; 
            x((j + 2*Lx)*r + k + 1) = t0 - t2; 
            x((j + 3*Lx)*r + k + 1) = t1 + i*t3;

            round = q
            Twiddlefactor_j = j
            Twiddlefactor_2j = 2*j
            Twiddlefactor_3j = 3*j

            in0 = j*rx + k
            out0 = j*r + k

            in1 = j*rx + r + k
            out1 = (j + Lx)*r + k

            in2 = j*rx + 2*r + k
            out2 = (j + 2*Lx)*r + k

            in3 = j*rx + 3*r + k
            out3 = (j + 3*Lx)*r + k

        end
    end
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,n-1,n);
figure;
subplot(2,1,1);plot(l,abs(x));
axis tight;
xlabel('n')
legend('Radix-4 FFT algorithm')
subplot(2,1,2);plot(l,abs(fft(x1)));
axis tight;
xlabel('n')
legend('Matlab FFT algorithm')
