%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% radix4.m
%
% Test with radix 4 algorithm
%
% By Magnus Nilsson 2000-11-30
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
N = 256;
bits = 18;
maxl = 15;
minl = -15;
f1 = 1.0;
f2 = 2.3;
Fs = 8;
var = 0.1;
t = log(N)/log(4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Chebychev windowing of the signal
win = chebwin(N,60);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Signal generation
u = (randn(1,N))*(var/2);
p = 1:N;
signal(p) = 0.5*(sin(2*pi*f1*p/Fs)+sin(2*pi*f2*p/Fs)+ u(p));
x = midquant(bits,minl,maxl,win.*signal');
%x = midquant(bits,minl,maxl,signal);
x1 = signal;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Radix-4 Algorithm

for q = 1:t
    L = 4^q;
    r = N/L;
    Lx = L/4;
    rx = 4*r;
    y = x;
    for j = 0:Lx-1
        for k = 0:r-1
            a = y(j*rx + k + 1);
            b = exp(-i*2*pi*j/L)*y(j*rx + r + k + 1);
            c = exp(-i*2*pi*2*j/L)*y(j*rx + 2*r + k + 1);
            d = exp(-i*2*pi*3*j/L)*y(j*rx + 3*r + k + 1);
            t0 = a + c;
            t1 = a - c;
            t2 = b + d;
            t3 = b - d;
            x(j*r + k + 1) = midquant(bits,minl,maxl,(t0 + t2));
            x((j + Lx)*r + k + 1) = midquant(bits,minl,maxl,(t1 - i*t3));
            x((j + 2*Lx)*r + k + 1) = midquant(bits,minl,maxl,(t0 - t2));
            x((j + 3*Lx)*r + k + 1) = midquant(bits,minl,maxl,(t1 + i*t3));
        end
	    
    end

end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);
figure;
subplot(2,1,1);plot(l,20*log10(abs(x)));
title('Radix-4 FFT 18-bits, 0.5*(sin(2*pi*f1*p/Fs)+sin(2*pi*f2*p/Fs)+ u(p)');
axis([0 N/2-1 -30 40]);
xlabel('n');
ylabel('dB');
subplot(2,1,2);plot(l,20*log10(abs(fft(x1))));
title('MATLAB FFT, 0.5*(sin(2*pi*f1*p/Fs)+sin(2*pi*f2*p/Fs)+ u(p)');
axis([0 N/2-1 -30 40]);
xlabel('n');
ylabel('dB');
