%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--  
%-------------------------------------------------------------------------
%--           Copyright ERICSSON MICROWAVE SYSTEMS AB 2001              --
%-------------------------------------------------------------------------
%-- 
%-- 
%-- 
%-- The copyright to the source program herein is the
%-- property of Ericsson Microwave Systems AB, Sweden.
%-- The program may be used and/or copied only with the
%-- written permission from Ericsson Microwave Systems
%-- AB or in accordance with the terms and conditions 
%-- stipulated in the agreement/contract under which 
%-- the program have been supplied.
%-- 
%--  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--  
%-- Project     : FFT
%-- Title       : Radix 4
%-- File name   : radix4_20.m
%-- Document no : -
%-- Revision    : pa1
%-- Date        : 01-01-24
%-- Prepared    : FX/D Magnus Nilsson
%-- Responsible : FX/DC Anders Wanner
%-- Description : Radix algorithm, amplitude and phase error.
%--
%-- ------------------------------------------------------------------------
%-- Revision history: (all revisions included)                            --
%-- ------------------------------------------------------------------------
%-- Rev:   Initials:     Date:    Modification:                           --
%-- ------------------------------------------------------------------------
%-- pa1     emwmsnn     01-01-15   First revision                         --
%-- ------------------------------------------------------------------------
%-- pa2     emwmsnn     01-01-24   Second revision                        --
%-- ------------------------------------------------------------------------
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
N = 256;
f1 = 35.5*8/256;
f2 = 2.3;
Fs = 8;
var = 0.1;
v = linspace(0,N-1,N);
Bits = 16;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Chebychev windowing of the signal
win = chebwin(N,60);
%win = blackman(N);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sinal generation
u = (randn(1,N))*(var/2);
n = 1:N;
%signal = 0.5*(sin(2*pi*f1*n/Fs)+sin(2*pi*f2*n/Fs)+ u(n));
signal = cos(2*pi*f1*n/Fs)+i*sin(2*pi*f1*n/Fs);
%signal = sin(2*pi*f1*n/Fs);
%signal = randn(1,N);
x = win'.*signal;
x = round(x*2^Bits);
x2 = x;
signal = win'.*signal;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate FFT using inplace DIT FFT, radix-4
q = 1;
L = 4^q;
r = N/L;
Lx = L/4;
rx = 4*r;
y = x;
for j = 0:Lx-1
    W1 = 1*2^Bits;
    W2 = round((exp(-i*2*pi*j/L))*2^Bits);
    W3 = round((exp(-i*2*pi*2*j/L))*2^Bits);
    W4 = round((exp(-i*2*pi*3*j/L))*2^Bits);
    for k = 0:r-1
        a = round((W1 * y(j*rx + k + 1))/2^Bits);
        b = round((W2 * y(j*rx + r + k + 1))/2^Bits);
        c = round((W3 * y(j*rx + 2*r + k + 1))/2^Bits);
        d = round((W4 * y(j*rx + 3*r + k + 1))/2^Bits);
        x(j*r + k + 1) = a+b+c+d;
        x((j + Lx)*r + k + 1) = a-i*b-c+i*d; 
        x((j + 2*Lx)*r + k + 1) = a-b+c-d; 
        x((j + 3*Lx)*r + k + 1) = a+i*b-c-i*d;
    end        
end

q = 2;
L = 4^q;
r = N/L;
Lx = L/4;
rx = 4*r;
y = x;
for j = 0:Lx-1
    W1 = 1*2^Bits;
    W2 = round((exp(-i*2*pi*j/L))*2^Bits);
    W3 = round((exp(-i*2*pi*2*j/L))*2^Bits);
    W4 = round((exp(-i*2*pi*3*j/L))*2^Bits);
    for k = 0:r-1
        a = round((W1 * y(j*rx + k + 1))/2^Bits);
        b = round((W2 * y(j*rx + r + k + 1))/2^Bits);
        c = round((W3 * y(j*rx + 2*r + k + 1))/2^Bits);
        d = round((W4 * y(j*rx + 3*r + k + 1))/2^Bits);
        x(j*r + k + 1) = a+b+c+d;
        x((j + Lx)*r + k + 1) = a-i*b-c+i*d; 
        x((j + 2*Lx)*r + k + 1) = a-b+c-d; 
        x((j + 3*Lx)*r + k + 1) = a+i*b-c-i*d;
    end        
end

q = 3;
L = 4^q;
r = N/L;
Lx = L/4;
rx = 4*r;
y = x;
for j = 0:Lx-1
    W1 = 1*2^Bits;
    W2 = round((exp(-i*2*pi*j/L))*2^Bits);
    W3 = round((exp(-i*2*pi*2*j/L))*2^Bits);
    W4 = round((exp(-i*2*pi*3*j/L))*2^Bits);
    for k = 0:r-1
        a = round((W1 * y(j*rx + k + 1))/2^Bits);
        b = round((W2 * y(j*rx + r + k + 1))/2^Bits);
        c = round((W3 * y(j*rx + 2*r + k + 1))/2^Bits);
        d = round((W4 * y(j*rx + 3*r + k + 1))/2^Bits);
        x(j*r + k + 1) = a+b+c+d;
        x((j + Lx)*r + k + 1) = a-i*b-c+i*d; 
        x((j + 2*Lx)*r + k + 1) = a-b+c-d; 
        x((j + 3*Lx)*r + k + 1) = a+i*b-c-i*d;
    end        
end

q = 4;
L = 4^q;
r = N/L;
Lx = L/4;
rx = 4*r;
y = x;
for j = 0:Lx-1
    W1 = 1*2^Bits;
    W2 = round((exp(-i*2*pi*j/L))*2^Bits);
    W3 = round((exp(-i*2*pi*2*j/L))*2^Bits);
    W4 = round((exp(-i*2*pi*3*j/L))*2^Bits);
    for k = 0:r-1
        a = round((W1 * y(j*rx + k + 1))/2^Bits);
        b = round((W2 * y(j*rx + r + k + 1))/2^Bits);
        c = round((W3 * y(j*rx + 2*r + k + 1))/2^Bits);
        d = round((W4 * y(j*rx + 3*r + k + 1))/2^Bits);
        x(j*r + k + 1) = a+b+c+d;
        x((j + Lx)*r + k + 1) = a-i*b-c+i*d; 
        x((j + 2*Lx)*r + k + 1) = a-b+c-d; 
        x((j + 3*Lx)*r + k + 1) = a+i*b-c-i*d;
    end        
end
x = x/2^Bits+10e-20;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);

figure;
subplot(2,2,1);
plot(l,20*log10(abs(x)));
title(['Radix-4 FFT, Bits = ',num2str(Bits)]);
axis tight;
legend('sin(2*pi*f1*p/Fs)');
xlabel('n');
ylabel('dB');

x2 = fft(signal);
subplot(2,2,2);
plot(l,20*log10(abs(x2)));
title('MATLAB FFT');
axis tight;
legend('sin(2*pi*f1*p/Fs)');
xlabel('n');
ylabel('dB');

x3 = x2./x;
subplot(2,2,3);
plot(20*log10(abs(x3)));
title('Amplitude error factor Radix-4 FFT/MATLAB FFT');
axis tight;
xlabel('n');
ylabel('dB');

x4 = angle(x3)*180/pi;
subplot(2,2,4);
plot(x4);
title('Phase error');
axis tight;
xlabel('n');
ylabel('Phase error');

%maxamp = 2^Bits;
%figure;
%y2 = 20*log10(abs(fft(x2))/(maxamp*sqrt(N*sum(win).^2/sum(win.^2))));
%y2(find(y2<-150))=-150;
%plot(20*log10(abs(fft(x2))/(maxamp*sqrt(N*sum(win).^2/sum(win.^2)))));
%plot(20*log10(abs(fft(signal))/(maxamp*N)));
