%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% radix4.m
%
% Test with radix 4 algorithm
%
% By Magnus Nilsson 2000-11-30
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
N = 4;
f1 = 1.3;
f2 = 2.7;
Fs = 8;
var = 0.03;
v = linspace(0,N-1,N);
Bits = 8;
t = log(N)/log(4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Chebychev windowing of the signal
win = chebwin(N,60);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sinal generation
u = (randn(1,N))*(var/2);
n = 1:N;
signal(n) = 0.5*(sin(2*pi*f1*n/Fs)+0.2*sin(2*pi*f2*n/Fs));%+ u(n));
x = win.*signal';
x = round(x.*(2^Bits));
N = length(x);
signal =win.*signal';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate FFT using inplace DIT FFT, radix-4

for q = 1:t
    L = 4^q;
    r = N/L;
    Lx = L/4;
    rx = 4*r;
    y = x;
    for j = 0:Lx-1
        for k = 0:r-1
            a = y(j*rx + k + 1);
            b = exp(-i*2*pi*j/L)*y(j*rx + r + k + 1);
            c = exp(-i*2*pi*2*j/L)*y(j*rx + 2*r + k + 1);
            d = exp(-i*2*pi*3*j/L)*y(j*rx + 3*r + k + 1);
            t0 = a + c;
            t1 = a - c;
            t2 = b + d;
            t3 = b - d;
            x(j*r + k + 1) = t0 + t2;
            x((j + Lx)*r + k + 1) = t1 - i*t3; 
            x((j + 2*Lx)*r + k + 1) = t0 - t2; 
            x((j + 3*Lx)*r + k + 1) = t1 + i*t3;
            
            exp(-i*2*pi*j/L)
            exp(-i*2*pi*2*j/L)
            exp(-i*2*pi*3*j/L)
        end        
    end
end 

%x = round(x/2^(Bits))+1e-14;
bin_sig = dec2bin(round(signal*2^12));
x_r = dec2bin(real(x));
x_i = dec2bin(imag(x));
x = x/2^Bits;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);
figure;
subplot(2,1,1);
plot(l,20*log10(abs(x)));
title('Radix-4 FFT 16-bits, sin(2*pi*f1*p/Fs)+sin(2*pi*f2*p/Fs)+ u(p)');
%axis([0 N/2-1 -40 40]);
xlabel('n');
ylabel('dB');
subplot(2,1,2);
plot(l,20*log10(abs(fft(signal))));
title('MATLAB FFT, sin(2*pi*f1*p/Fs)+sin(2*pi*f2*p/Fs)+ u(p)');
%axis([0 N/2-1 -40 40]);
xlabel('n');
ylabel('dB');
