%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% radix4.m
%
% Test with radix 4 algorithm
%
% By Magnus Nilsson 2000-11-30
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
N = 256;
t = 1:N;
x = sin(2*pi*0.35*t)+sin(2*pi*0.38*t);
xr = real(x);
xi = imag(x);
t = log(N)/log(4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Radix-4 Algorithm

for q = 1:t
    L = 4^q;
    r = N/L;
    Lx = L/4;
    rx = 4*r;
    yr = xr;
    yi = xi;
    for j = 0:Lx-1
        for k = 0:r-1
%            in0r = yr(j*rx + k + 1);
%            in0i = yi(j*rx + k + 1);
%            in1r = yr(j*rx + r + k + 1);
%            in1i = yi(j*rx + r + k + 1);
%            in2r = yr(j*rx + 2*r + k + 1);
%            in2i = yi(j*rx + 2*r + k + 1);
%            in3r = yr(j*rx + 3*r + k + 1);
%            in3i = yi(j*rx + 3*r + k + 1);

%            ar = in0r;
%            ai = in0i;
%            br = in1r*cos(2*pi*j/L) - in1i*sin(2*pi*j/L);
%            bi = in1r*sin(2*pi*j/L) + in1i*cos(2*pi*j/L);
%            cr = in2r*cos(2*pi*2*j/L) - in2i*sin(2*pi*2*j/L);
%            ci = in2r*sin(2*pi*2*j/L) + in2i*cos(2*pi*2*j/L);
%            dr = in3r*cos(2*pi*3*j/L) - in3i*sin(2*pi*3*j/L);
%            di = in3r*sin(2*pi*3*j/L) + in3i*cos(2*pi*3*j/L);

            y = complex(yr,yi);

            ar = real(y(j*rx + k + 1));
            br = real(exp(-i*2*pi*j/L)*y(j*rx + r + k + 1));
            cr = real(exp(-i*2*pi*2*j/L)*y(j*rx + 2*r + k + 1));
            dr = real(exp(-i*2*pi*3*j/L)*y(j*rx + 3*r + k + 1));

            ai = imag(y(j*rx + k + 1));
            bi = imag(exp(-i*2*pi*j/L)*y(j*rx + r + k + 1));
            ci = imag(exp(-i*2*pi*2*j/L)*y(j*rx + 2*r + k + 1));
            di = imag(exp(-i*2*pi*3*j/L)*y(j*rx + 3*r + k + 1));

            Ar = ar + br + cr + dr;
            Ai = ai + bi + ci + di;
	    Br = ar + bi - cr - di;
	    Bi = ai - br - ci + dr;
	    Cr = ar - br + cr - dr;
	    Ci = ai - bi + ci - di;
	    Dr = ar - bi - cr + di;
	    Di = ai + br - ci - di;
	    
            xr(j*r + k + 1) = Ar;
            xi(j*r + k + 1) = Ai;
            xr((j + Lx)*r + k + 1) = Br;
            xi((j + Lx)*r + k + 1) = Bi; 
            xr((j + 2*Lx)*r + k + 1) = Cr;
            xi((j + 2*Lx)*r + k + 1) = Ci; 
            xr((j + 3*Lx)*r + k + 1) = Dr;
            xi((j + 3*Lx)*r + k + 1) = Di;
        end
    end
end 

x_complex = complex(xr,xi);
	    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);
figure;
subplot(2,1,1);plot(l,abs(x_complex));
subplot(2,1,2);plot(l,abs(fft(x)));

