%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--  
%-------------------------------------------------------------------------
%--           Copyright ERICSSON MICROWAVE SYSTEMS AB 2001              --
%-------------------------------------------------------------------------
%-- 
%-- 
%-- 
%-- The copyright to the source program herein is the
%-- property of Ericsson Microwave Systems AB, Sweden.
%-- The program may be used and/or copied only with the
%-- written permission from Ericsson Microwave Systems
%-- AB or in accordance with the terms and conditions 
%-- stipulated in the agreement/contract under which 
%-- the program have been supplied.
%-- 
%--  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--  
%-- Project     : FFT
%-- Title       : Radix-4
%-- File name   : radix4_complex2.m
%-- Document no : -
%-- Revision    : pa2
%-- Date        : 00-12-23
%-- Prepared    : FX/D Magnus Nilsson
%-- Responsible : FX/DC Hkan Enskog
%-- Description : Complex Radix-4 algorithm with structure and binary
%--               twiddle factor
%--
%-- ------------------------------------------------------------------------
%-- Revision history: (all revisions included)                            --
%-- ------------------------------------------------------------------------
%-- Rev:   Initials:     Date:    Modification:                           --
%-- ------------------------------------------------------------------------
%-- pa1     emwmsnn     00-12-23   First revision                         --
%-- ------------------------------------------------------------------------
%-- pa2     emwmsnn     01-01-05   Second revision                        --
%-- ------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------- Innitialize variables ----------------------%
clear all;
close all;
twiddlefile='twiddle_factor.mif';
DATA_file=fopen(twiddlefile,'w');
count = 0;
N = 16;
time = 1:N;
t = log(N)/log(4);
structure = zeros(N,2*t);
m = 0;
h =0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------- Chebychev windowing -----------------------%
win = chebwin(N,60);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--------------------- Signal -----------------------------%
x = win'.*(sin(2*pi*0.25*time)+sin(2*pi*0.38*time));
xr = real(x);
xi = imag(x);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------- Radix-4 Algorithm -----------------------%

for q = 1:t
    L = 4^q;
    r = N/L;
    Lx = L/4;
    rx = 4*r;
    yr = xr;
    yi = xi;
    for j = 0:Lx-1
        for k = 0:r-1
            in0r = yr(j*rx + k + 1);
            in0i = yi(j*rx + k + 1);
            in1r = yr(j*rx + r + k + 1);
            in1i = yi(j*rx + r + k + 1);
            in2r = yr(j*rx + 2*r + k + 1);
            in2i = yi(j*rx + 2*r + k + 1);
            in3r = yr(j*rx + 3*r + k + 1);
            in3i = yi(j*rx + 3*r + k + 1);

            ar = in0r;
            ai = in0i;
            br = in1r*cos(2*pi*j/L) + in1i*sin(2*pi*j/L);
            bi = in1i*cos(2*pi*j/L) - in1r*sin(2*pi*j/L);
            cr = in2r*cos(2*pi*2*j/L) + in2i*sin(2*pi*2*j/L);
            ci = in2i*cos(2*pi*2*j/L) - in2r*sin(2*pi*2*j/L);
            dr = in3r*cos(2*pi*3*j/L) + in3i*sin(2*pi*3*j/L);
            di = in3i*cos(2*pi*3*j/L) - in3r*sin(2*pi*3*j/L);

            no_of_bits = 16;%+(q-1)*2;

            DATA(1) = round(1*2^(no_of_bits-2));
            DATA(2) = round(1*2^(no_of_bits-2));
            DATA(3) = round(cos(2*pi*1*j/L)*2^(no_of_bits-2));
            DATA(4) = round(sin(2*pi*1*j/L)*2^(no_of_bits-2));
            DATA(5) = round(cos(2*pi*2*j/L)*2^(no_of_bits-2));
            DATA(6) = round(sin(2*pi*2*j/L)*2^(no_of_bits-2));
            DATA(7) = round(cos(2*pi*3*j/L)*2^(no_of_bits-2));
            DATA(8) = round(sin(2*pi*3*j/L)*2^(no_of_bits-2));
        
            %Tow's complement
            for s=1:length(DATA)
                dec_value=DATA(s);       % Place the k:th DATA value in a new variable.
      

                if (dec_value>2^(no_of_bits-0)-1)|(dec_value<-(2^(no_of_bits-0)))                                             
                    error('Too big decimal input')
                end

                dummy_value=blanks(no_of_bits);         % Skapar en tom strng.
                dummy_value(1:no_of_bits)='0';

                if (dec_value<0);                      % Letar upp alla tal mindre n 0.
                    dec_value=2^no_of_bits+dec_value;    % Tv-komp. omv.
                end

                bin_value=dec2bin(dec_value);
                dummy_value(no_of_bits-length(bin_value)+1:no_of_bits)=bin_value;        
                DATA_value(s,:)=dummy_value;     
            end

            vhdl_round=blanks(no_of_bits+3);
            vhdl_round(1:no_of_bits+2)='0';
            vhdl_round(no_of_bits+3)='1';

            %To file

            [a,b]=size(DATA_value);
            fprintf(DATA_file, 'Twiddlefactor for step %d block %d is:\n', q, count);                               
            fprintf(DATA_file, 'constant cos_0j  : std_logic_vector((N-1) downto 0) :="');
            fprintf(DATA_file, '%s";\n',DATA_value(1,[1:b]));
            fprintf(DATA_file, 'constant sin_0j  : std_logic_vector((N-1) downto 0) :="');
            fprintf(DATA_file, '%s";\n',DATA_value(2,[1:b]));
            fprintf(DATA_file, 'constant cos_1j  : std_logic_vector((N-1) downto 0) :="');
            fprintf(DATA_file, '%s";\n',DATA_value(3,[1:b]));
            fprintf(DATA_file, 'constant sin_1j  : std_logic_vector((N-1) downto 0) :="');
            fprintf(DATA_file, '%s";\n',DATA_value(4,[1:b]));
            fprintf(DATA_file, 'constant cos_2j  : std_logic_vector((N-1) downto 0) :="');
            fprintf(DATA_file, '%s";\n',DATA_value(5,[1:b]));
            fprintf(DATA_file, 'constant sin_2j  : std_logic_vector((N-1) downto 0) :="');
            fprintf(DATA_file, '%s";\n',DATA_value(6,[1:b]));
            fprintf(DATA_file, 'constant cos_3j  : std_logic_vector((N-1) downto 0) :="');
            fprintf(DATA_file, '%s";\n',DATA_value(7,[1:b]));
            fprintf(DATA_file, 'constant sin_3j  : std_logic_vector((N-1) downto 0) :="');
            fprintf(DATA_file, '%s";\n',DATA_value(8,[1:b]));
            fprintf(DATA_file, 'constant round   : std_logic_vector((N+2) downto 0) :="');
            fprintf(DATA_file, '%s";\n',vhdl_round([1:no_of_bits+3]));
            
            fprintf(DATA_file, '\n'); 		                  

            count = count + 1;
            clear DATA_value;

            structure(1+m*4+h*N) = j*rx + k;
            structure(1+N+m*4+h*N) = j*r + k;
            structure(2+m*4+h*N) = j*rx + r + k;
            structure(2+N+m*4+h*N) = (j + Lx)*r + k;
            structure(3+m*4+h*N) = j*rx + 2*r + k;
            structure(3+N+m*4+h*N) = (j + 2*Lx)*r + k;
            structure(4+m*4+h*N) = j*rx + 3*r + k;
            structure(4+N+m*4+h*N) = (j + 3*Lx)*r + k;            

            m = m+1;

            temp1r = ar + cr;
            temp1i = ai + ci;            
            temp2r = ar - cr;
            temp2i = ai - ci;

            Ar = temp1r + br + dr;
            Ai = temp1i + bi + di;
	    Br = temp2r + bi - di;
	    Bi = temp2i - br + dr;
	    Cr = temp1r - br - dr;
	    Ci = temp1i - bi - di;
	    Dr = temp2r - bi + di;
	    Di = temp2i + br - dr;
	    
            xr(j*r + k + 1) = Ar;
            xi(j*r + k + 1) = Ai;
            xr((j + Lx)*r + k + 1) = Br;
            xi((j + Lx)*r + k + 1) = Bi; 
            xr((j + 2*Lx)*r + k + 1) = Cr;
            xi((j + 2*Lx)*r + k + 1) = Ci; 
            xr((j + 3*Lx)*r + k + 1) = Dr;
            xi((j + 3*Lx)*r + k + 1) = Di;
        end
    end
    h = h+1;	    
end
fclose(DATA_file);	
diary('structure.mif');  
structure
diary off;
x_complex = xr+i*xi;
	    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);
figure;
hold on;
plot(l,20*log10(abs(x_complex)),'r+');
plot(l,20*log10(abs(fft(x))));
title('Radix-4 Complex algorithm and MATLAB FFT');
legend('Radix-4','Matlab FFT',2);
axis([0 N-1 20*log10(min(abs(x_complex))) 20*log10(max(abs(x_complex)))]);
xlabel('n');
ylabel('dB');
zoom on;
grid on;
