%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% radix4.m
%
% Test with radix 4 algorithm
%
% By Magnus Nilsson 2000-11-30
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
N = 256;
f1 = 35.5*8/256;
f2 = 2.3;
Fs = 8;
var = 0.1;
v = linspace(0,N-1,N);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Chebychev windowing of the signal
win = chebwin(N,60);
%win = blackman(N);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sinal generation
%signal = 0;
%Bits = 0;
u = (randn(1,N))*(var/2);
n = 1:N;
signal1 = 0.5*(sin(2*pi*f1*n/Fs)+sin(2*pi*f2*n/Fs)+ u(n));
signal2 = cos(2*pi*f1*n/Fs)+i*sin(2*pi*f1*n/Fs);
signal3 = sin(2*pi*f1*n/Fs);
signal4 = randn(1,N);
x = win'.*signal;
x = round(x*2^Bits);
x2 = x;
signal = win'.*signal;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate FFT using inplace DIT FFT, radix-4
q = 1;
L = 4^q;
r = N/L;
Lx = L/4;
rx = 4*r;
y = x;
for j = 0:Lx-1
    W1 = 1*2^Bits;
    W2 = round((exp(-i*2*pi*j/L))*2^Bits);
    W3 = round((exp(-i*2*pi*2*j/L))*2^Bits);
    W4 = round((exp(-i*2*pi*3*j/L))*2^Bits);
    for k = 0:r-1
        a = round((W1 * y(j*rx + k + 1))/2^Bits);
        b = round((W2 * y(j*rx + r + k + 1))/2^Bits);
        c = round((W3 * y(j*rx + 2*r + k + 1))/2^Bits);
        d = round((W4 * y(j*rx + 3*r + k + 1))/2^Bits);
        x(j*r + k + 1) = a+b+c+d;
        x((j + Lx)*r + k + 1) = a-i*b-c+i*d; 
        x((j + 2*Lx)*r + k + 1) = a-b+c-d; 
        x((j + 3*Lx)*r + k + 1) = a+i*b-c-i*d;
    end        
end

q = 2;
L = 4^q;
r = N/L;
Lx = L/4;
rx = 4*r;
y = x;
for j = 0:Lx-1
    W1 = 1*2^Bits;
    W2 = round((exp(-i*2*pi*j/L))*2^Bits);
    W3 = round((exp(-i*2*pi*2*j/L))*2^Bits);
    W4 = round((exp(-i*2*pi*3*j/L))*2^Bits);
    for k = 0:r-1
        a = round((W1 * y(j*rx + k + 1))/2^Bits);
        b = round((W2 * y(j*rx + r + k + 1))/2^Bits);
        c = round((W3 * y(j*rx + 2*r + k + 1))/2^Bits);
        d = round((W4 * y(j*rx + 3*r + k + 1))/2^Bits);
        x(j*r + k + 1) = a+b+c+d;
        x((j + Lx)*r + k + 1) = a-i*b-c+i*d; 
        x((j + 2*Lx)*r + k + 1) = a-b+c-d; 
        x((j + 3*Lx)*r + k + 1) = a+i*b-c-i*d;
    end        
end

q = 3;
L = 4^q;
r = N/L;
Lx = L/4;
rx = 4*r;
y = x;
for j = 0:Lx-1
    W1 = 1*2^Bits;
    W2 = round((exp(-i*2*pi*j/L))*2^Bits);
    W3 = round((exp(-i*2*pi*2*j/L))*2^Bits);
    W4 = round((exp(-i*2*pi*3*j/L))*2^Bits);
    for k = 0:r-1
        a = round((W1 * y(j*rx + k + 1))/2^Bits);
        b = round((W2 * y(j*rx + r + k + 1))/2^Bits);
        c = round((W3 * y(j*rx + 2*r + k + 1))/2^Bits);
        d = round((W4 * y(j*rx + 3*r + k + 1))/2^Bits);
        x(j*r + k + 1) = a+b+c+d;
        x((j + Lx)*r + k + 1) = a-i*b-c+i*d; 
        x((j + 2*Lx)*r + k + 1) = a-b+c-d; 
        x((j + 3*Lx)*r + k + 1) = a+i*b-c-i*d;
    end        
end

q = 4;
L = 4^q;
r = N/L;
Lx = L/4;
rx = 4*r;
y = x;
for j = 0:Lx-1
    W1 = 1*2^Bits;
    W2 = round((exp(-i*2*pi*j/L))*2^Bits);
    W3 = round((exp(-i*2*pi*2*j/L))*2^Bits);
    W4 = round((exp(-i*2*pi*3*j/L))*2^Bits);
    for k = 0:r-1
        a = round((W1 * y(j*rx + k + 1))/2^Bits);
        b = round((W2 * y(j*rx + r + k + 1))/2^Bits);
        c = round((W3 * y(j*rx + 2*r + k + 1))/2^Bits);
        d = round((W4 * y(j*rx + 3*r + k + 1))/2^Bits);
        x(j*r + k + 1) = a+b+c+d;
        x((j + Lx)*r + k + 1) = a-i*b-c+i*d; 
        x((j + 2*Lx)*r + k + 1) = a-b+c-d; 
        x((j + 3*Lx)*r + k + 1) = a+i*b-c-i*d;
    end        
end
x = x/2^Bits+10e-20;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);


figure(2);
subplot(2,2,1);
plot(l,20*log10(abs(x)));
title(['Radix-4 FFT, Bits = ',num2str(Bits)]);
axis tight;
xlabel('n');
ylabel('dB');

x2 = fft(signal);
subplot(2,2,2);
plot(l,20*log10(abs(x2)));
title('MATLAB FFT');
axis tight;
xlabel('n');
ylabel('dB');

x3 = x2./x;
subplot(2,2,3);
plot(20*log10(abs(x3)));
title('Amplitude error factor Radix-4 FFT/MATLAB FFT');
axis tight;
xlabel('n');
ylabel('dB');
%ylabel('Amplitude error factor');

x4 = angle(x3)*180/pi;
subplot(2,2,4);
plot(x4);
title('Angle error');
axis tight;
xlabel('n');
ylabel('Phase error');


