%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% radix4.m
%
% Test with radix 4 algorithm
%
% By Magnus Nilsson 2000-12-23
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
N = 4;
time = 1:N;
t = log(N)/log(4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Chebychev windowing 
win = chebwin(N,60);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Signal
x = win'.*(sin(2*pi*0.25*time)+sin(2*pi*0.38*time));
xr = real(x);
xi = imag(x);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Radix-4 Algorithm

Bits = 10;
L = 4;
r = 1;
Lx = 1;
rx = 4;
yr = xr;
yi = xi;
j = 0;
k = 0;

cos_j =  round(cos(2*pi*j/L)*2^Bits);
sin_j =  round(sin(2*pi*j/L)*2^Bits);
cos_2j = round(cos(2*pi*2*j/L)*2^Bits);
sin_2j = round(sin(2*pi*2*j/L)*2^Bits);
cos_3j = round(cos(2*pi*3*j/L)*2^Bits);
sin_3j = round(sin(2*pi*3*j/L)*2^Bits);

in0r = round(yr(j*rx + k + 1)*2^Bits);
in0i = round(yi(j*rx + k + 1)*2^Bits);
in1r = round(yr(j*rx + r + k + 1)*2^Bits);
in1i = round(yi(j*rx + r + k + 1)*2^Bits);
in2r = round(yr(j*rx + 2*r + k + 1)*2^Bits);
in2i = round(yi(j*rx + 2*r + k + 1)*2^Bits);
in3r = round(yr(j*rx + 3*r + k + 1)*2^Bits);
in3i = round(yi(j*rx + 3*r + k + 1)*2^Bits);

ar = round((2^Bits*in0r)/2^Bits);
ai = round((2^Bits*in0i)/2^Bits);
br = round((in1r*cos_j + in1i*sin_j)/2^Bits);
bi = round((in1i*cos_j - in1r*sin_j)/2^Bits);
cr = round((in2r*cos_2j + in2i*sin_2j)/2^Bits);
ci = round((in2i*cos_2j - in2r*sin_2j)/2^Bits);
dr = round((in3r*cos_3j + in3i*sin_3j)/2^Bits);
di = round((in3i*cos_3j - in3r*sin_3j)/2^Bits);



q(1,:) = dec2bin(cos(2*pi*j/L)*2^10);
q(2,:) = dec2bin(sin(2*pi*j/L)*2^10);

q(3,:) = dec2bin(cos(2*pi*2*j/L)*2^10);
q(4,:) = dec2bin(sin(2*pi*2*j/L)*2^10);

q(5,:) = dec2bin(cos(2*pi*3*j/L)*2^10);
q(6,:) = dec2bin(sin(2*pi*3*j/L)*2^10);

temp1r = ar + cr;
temp1i = ai + ci;

temp2r = ar - cr;
temp2i = ai - ci;

Ar = temp1r + br + dr;
Ai = temp1i + bi + di;
Br = temp2r + bi - di;
Bi = temp2i - br + dr;
Cr = temp1r - br - dr;
Ci = temp1i - bi - di;
Dr = temp2r - bi + di;
Di = temp2i + br - dr;

xr(j*r + k + 1) = Ar;
xi(j*r + k + 1) = Ai;
xr((j + Lx)*r + k + 1) = Br;
xi((j + Lx)*r + k + 1) = Bi;
xr((j + 2*Lx)*r + k + 1) = Cr;
xi((j + 2*Lx)*r + k + 1) = Ci;
xr((j + 3*Lx)*r + k + 1) = Dr;
xi((j + 3*Lx)*r + k + 1) = Di;

x_complex = xr+i*xi;
x = x_complex/2^Bits;
	    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);
figure;
hold on;
plot(l,(abs(x_complex)),'r+');
plot(l,(abs(fft(x))));
title('Radix-4 Complex algorithm and MATLAB FFT');
legend('Radix-4','Matlab FFT',2);
%axis([0 N/2-1 20*log10(min(abs(x_complex))) 20*log10(max(abs(x_complex)))]);
xlabel('n');
ylabel('dB');
zoom on;
grid on;
