%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--  
%-------------------------------------------------------------------------
%--           Copyright ERICSSON MICROWAVE SYSTEMS AB 2001              --
%-------------------------------------------------------------------------
%-- 
%-- 
%-- 
%-- The copyright to the source program herein is the
%-- property of Ericsson Microwave Systems AB, Sweden.
%-- The program may be used and/or copied only with the
%-- written permission from Ericsson Microwave Systems
%-- AB or in accordance with the terms and conditions 
%-- stipulated in the agreement/contract under which 
%-- the program have been supplied.
%-- 
%--  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--  
%-- Project     : FFT
%-- Title       : VHDL structure
%-- File name   : radix4_vhdl2.m
%-- Document no : -
%-- Revision    : pa1
%-- Date        : 01-01-03
%-- Prepared    : FX/D Magnus Nilsson
%-- Responsible : FX/DC Anders Wanner
%-- Description : Radix-4 VHDL
%--
%-- ------------------------------------------------------------------------
%-- Revision history: (all revisions included)                            --
%-- ------------------------------------------------------------------------
%-- Rev:   Initials:     Date:    Modification:                           --
%-- ------------------------------------------------------------------------
%-- pa1     emwmsnn     01-01-03   First revision                         --
%-- ------------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
N = 4;
time = 1:N;
t = log(N)/log(4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Signal
x = sin(2*pi*0.25*time);
xr = real(x);
xi = imag(x);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Radix-4 Algorithm

L = 4;
r = 1;
Lx = 1;
rx = 4;
j = 0;
k = 0;

in0r = xr(j*rx + k + 1)*2^10;
in0i = xi(j*rx + k + 1)*2^10;
in1r = xr(j*rx + r + k + 1)*2^10;
in1i = xi(j*rx + r + k + 1)*2^10;
in2r = xr(j*rx + 2*r + k + 1)*2^10;
in2i = xi(j*rx + 2*r + k + 1)*2^10;
in3r = xr(j*rx + 3*r + k + 1)*2^10;
in3i = xi(j*rx + 3*r + k + 1)*2^10;

ar = round((round(in0r)*round(1*2^10))/2^20);
ai = round((round(in0i)*round(1*2^10))/2^20);
br = round((round(in1r)*round(cos(2*pi*j/L)*2^10) + round(in1i)*round(sin(2*pi*j/L)*2^10))/2^20);
bi = round((round(in1i)*round(cos(2*pi*j/L)*2^10) - round(in1r)*round(sin(2*pi*j/L)*2^10))/2^20);
cr = round((round(in2r)*round(cos(2*pi*2*j/L)*2^10) + round(in2i)*round(sin(2*pi*2*j/L)*2^10))/2^20);
ci = round((round(in2i)*round(cos(2*pi*2*j/L)*2^10) - round(in2r)*round(sin(2*pi*2*j/L)*2^10))/2^20);
dr = round((round(in3r)*round(cos(2*pi*3*j/L)*2^10) + round(in3i)*round(sin(2*pi*3*j/L)*2^10))/2^20);
di = round((round(in3i)*round(cos(2*pi*3*j/L)*2^10) - round(in3r)*round(sin(2*pi*3*j/L)*2^10))/2^20);

q(1,:) = dec2bin(round(cos(2*pi*j/L)*2^10));
q(2,:) = dec2bin(round(sin(2*pi*j/L)*2^10));

q(3,:) = dec2bin(round(cos(2*pi*2*j/L)*2^10));
q(4,:) = dec2bin(round(sin(2*pi*2*j/L)*2^10));
     
q(5,:) = dec2bin(round(cos(2*pi*3*j/L)*2^10));
q(6,:) = dec2bin(round(sin(2*pi*3*j/L)*2^10));
     
temp1r = ar + cr;
temp1i = ai + ci;
temp2r = ar - cr;
temp2i = ai - ci;

Ar = temp1r + br + dr;
Ai = temp1i + bi + di;
Br = temp2r + bi - di;
Bi = temp2i - br + dr;
Cr = temp1r - br - dr;
Ci = temp1i - bi - di;
Dr = temp2r - bi + di;
Di = temp2i + br - dr;
	    
xr(j*r + k + 1) = Ar;
xi(j*r + k + 1) = Ai;
xr((j + Lx)*r + k + 1) = Br;
xi((j + Lx)*r + k + 1) = Bi; 
xr((j + 2*Lx)*r + k + 1) = Cr;
xi((j + 2*Lx)*r + k + 1) = Ci; 
xr((j + 3*Lx)*r + k + 1) = Dr;
xi((j + 3*Lx)*r + k + 1) = Di;

x_complex = round((xr+i*xi));;
	    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,N-1,N);
figure;
hold on;
plot(l,abs(x_complex),'r+');
plot(l,abs(fft(x)));
title('Radix-4 Complex algorithm and MATLAB FFT');
legend('Radix-4','Matlab FFT',2);
%axis([0 N/2-1 20*log10(min(abs(x_complex))) 20*log10(max(abs(x_complex)))]);
xlabel('n');
ylabel('dB');
zoom on;
grid on;
