%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--  
%-------------------------------------------------------------------------
%--           Copyright ERICSSON MICROWAVE SYSTEMS AB 2001              --
%-------------------------------------------------------------------------
%-- 
%-- 
%-- 
%-- The copyright to the source program herein is the
%-- property of Ericsson Microwave Systems AB, Sweden.
%-- The program may be used and/or copied only with the
%-- written permission from Ericsson Microwave Systems
%-- AB or in accordance with the terms and conditions 
%-- stipulated in the agreement/contract under which 
%-- the program have been supplied.
%-- 
%--  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--  
%-- Project     : FFT
%-- Title       : Radix-2 vs DFT Flops
%-- File name   : radix_flops.m
%-- Document no : -
%-- Revision    : pa1
%-- Date        : 00-11-30
%-- Prepared    : FX/D Magnus Nilsson
%-- Responsible : FX/DC Anders Wanner
%-- Description : Radix-2 vs. Discrete Fourier transfor
%--
%-- ------------------------------------------------------------------------
%-- Revision history: (all revisions included)                            --
%-- ------------------------------------------------------------------------
%-- Rev:   Initials:     Date:    Modification:                           --
%-- ------------------------------------------------------------------------
%-- pa1     emwmsnn     00-11-30   First revision                         --
%-- ------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
N_length = [0 4 16 64 256 1024];
for floopys = 1:length(N_length)
     p = 1:1:N_length(floopys);
     x = sin(2*pi*0.35*p)+sin(2*pi*0.38*p);
     floopys	   

     % Radix-2 Algorithm
     flops(0);
     X2 = fft(x);
     radix2(floopys) = flops;

     % Discret Fourier transform
     X = zeros(1,N_length(floopys));
     flops(0);
     for n = 1:N_length(floopys)
         for k = 1:N_length(floopys)
             X(n) = X(n) + (x(k)*exp((-i*2*pi*(n-1)*(k-1))/N_length(floopys)));
	 end
     end    
     dft(floopys) = flops;
end    	    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
figure;
hold on;
plot(N_length,30*radix2,'r--');
plot(N_length,dft);
hold off;
axis tight
xlabel('N');
ylabel('Number of flops');
legend('Radix-2','Direct Calculation',2);
title('Radix-2 vs. Direct Calculation in flops')
