%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% radix4.m
%
% Test with radix 4 algorithm
%
% By Magnus Nilsson 2000-11-30
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Innitialize variables.
t = 1:1:2048;
x_test = sin(2*pi*0.35*t)+sin(2*pi*0.40*t);
ar = real(x_test);
ai = imag(x_test);
n = length(x_test);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate FFT using inplace DIT FFT, split-radix

% L shaped butterflies
theta = -8 * atan(1.0) / n;

m = n;
while m > 2
    mq = bitshift(m,-2);
    i = 0;
    while i < mq
        w1r = cos(theta * i);
        w1i = sin(theta * i);
        w3r = cos(theta * 3 * i);
        w3i = sin(theta * 3 * i);
        k = m;
        while k <= n
            j = k - m + i + 1;
            while j < n
                j1 = j + mq;
                j2 = j1 + mq;
                j3 = j2 + mq;
                x1r = ar(j) - ar(j2);
                x1i = ai(j) - ai(j2);
                ar(j) =  ar(j) + ar(j2);
                ai(j) =  ai(j) + ai(j2);
                x3r = ar(j3) - ar(j1);
                x3i = ai(j3) - ai(j1);
                ar(j1) = ar(j1) + ar(j3);
                ai(j1) = ai(j1) + ai(j3);
                x0r = x1r - x3i;
                x0i = x1i + x3r;
                ar(j2) = w1r * x0r - w1i * x0i;
                ai(j2) = w1r * x0i + w1i * x0r;
                x0r = x1r + x3i;
                x0i = x1i - x3r;
                ar(j3) = w3r * x0r - w3i * x0i;
                ai(j3) = w3r * x0i + w3i * x0r;
                j = j + 2*k;
            end
            k = bitshift(k,2);
        end
        i = i + 1;
    end
    theta = theta + 2*theta;
    m = bitshift(m,-1);
end

% Radix 2 butterflies
k = 2;
while k <=n
    j = k - 2 + 1;
    while j < n
        x0r = ar(j) - ar(j + 1);
        x0i = ai(j) - ai(j + 1);
        ar(j) = ar(j) + ar(j + 1);
        ai(j) = ai(j) + ai(j + 1);
        ar(j + 1) = x0r;
        ai(j + 1) = x0i;
        j = j + 2*k;
    end
    k = bitshift(k,2);
end

% Unscrambler
i = 0;
j = 1;
while j < (n-1)
    k = bitshift(n,-1);
    while k > bitxor(i,k)
        k = bitshift(k,-1);
    end    
    if (j < i)
        x0r = ar(j);
        x0i = ai(j);
        ar(j) = ar(i);
        ai(j) = ai(i);
        ar(i) = x0r;
        ai(i) = x0i;
    end
    j = j + 1;
end

clear j;

a = ar + j*ai;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Put in bitreverse order
b = bin2dec(fliplr(dec2bin(0:1:n-1)))+1;
c = linspace(1,n,n);
d = a(b);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot results
l = linspace(0,n-1,n);
figure;
subplot(2,1,1);plot(l,abs(a));
subplot(2,1,2);plot(l,abs(fft(x_test)));
%axis([0 n/2-1 0 1.2*max(abs(fft(x_test)))]);
