%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--  
%-------------------------------------------------------------------------
%--           Copyright ERICSSON MICROWAVE SYSTEMS AB 2001              --
%-------------------------------------------------------------------------
%-- 
%-- 
%-- 
%-- The copyright to the source program herein is the
%-- property of Ericsson Microwave Systems AB, Sweden.
%-- The program may be used and/or copied only with the
%-- written permission from Ericsson Microwave Systems
%-- AB or in accordance with the terms and conditions 
%-- stipulated in the agreement/contract under which 
%-- the program have been supplied.
%-- 
%--  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--  
%-- Project     : FFT
%-- Title       : VHDL stimuli
%-- File name   : stimuli.m
%-- Document no : -
%-- Revision    : pa1
%-- Date        : 01-01-15
%-- Prepared    : FX/D Magnus Nilsson
%-- Responsible : FX/DC Anders Wanner
%-- Description : Stimuli for VHDL-code
%--
%-- ------------------------------------------------------------------------
%-- Revision history: (all revisions included)                            --
%-- ------------------------------------------------------------------------
%-- Rev:   Initials:     Date:    Modification:                           --
%-- ------------------------------------------------------------------------
%-- pa1     emwmsnn     01-01-15   First revision                         --
%-- ------------------------------------------------------------------------
%-- pa2     emwmsnn     01-01-24   Second revision                        --
%-- ------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------- Innitialize variables ----------------------%
clear all;
close all;
N=16;
simbits = 16;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------- Outdata----------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vhdl_input_r='/proj/fmd/fft/vhdl/vhdl_real_input.mif';
vhdl_input_i='/proj/fmd/fft/vhdl/vhdl_imag_input.mif';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------- Chebychev windowing -----------------------%
win = chebwin(N,60);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--------------------- Signal -----------------------------%
t = 1:N;
x1 = sin(2*pi*4/16*t);
x2 = sin(2*pi*4.5/16*t);
x3 = cos(2*pi*4/16*t)+i*sin(2*pi*4/16*t);
x4 = cos(2*pi*4.5/16*t)+i*sin(2*pi*4.5/16*t);
x = round(2^8*x4);
x_test = 2^4*x4; %For Matlab calcutation

xr = real(x);
xi = imag(x);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------- Make data file for VHDL  -------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
input_i = towscomp(simbits,xi);
input_r = towscomp(simbits,xr);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------- To file ------------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fp_in_re=fopen(vhdl_input_r,'w');
fp_in_im=fopen(vhdl_input_i,'w');

[a,b]=size(input_r);

for r=1:a
 fprintf(fp_in_re, '%s\n',input_r(r,[1:b]));
 fprintf(fp_in_im, '%s\n',input_i(r,[1:b]));
end;

fclose(fp_in_re);	
fclose(fp_in_im);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------- From file ---------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

a = textread('/proj/fmd/fft/vhdl/vhdl_real_output.mif','%d');
b = textread('/proj/fmd/fft/vhdl/vhdl_imag_output.mif','%d');
l= linspace(0,N-1,N);
hold on
plot(20*log10(abs(fft(x_test))),'r--')
plot(20*log10(abs(a+i*b)))%+0.0000000000001))     
legend('Matlab','Modelsim');
axis tight;
xlabel('n');
ylabel('dB');
if var(b) == 0
   title('Radix-4 and Matlab FFT, ABS calculated by FPGA, signal = x3');       
else
   title('Radix-4 and Matlab FFT, ABS calculated by Matlab, signal = x3');
end;
hold;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-Error between Matlab calculated FFT and FPGA calculation-%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure;
o=abs(fft(x_test))'-abs(a+i*b);
qq=o./abs(fft(x_test'));
plot(100*qq);
%axis([1 N -10 10]);
xlabel('n');
ylabel('%');

if var(b) == 0
   title('Radix-4 and Matlab FFT, Error in % between Matlab calculated ABS and Modelsim ABS');  
else
   title('Radix-4 and Matlab FFT, Error in % between Matlab FFT  and Modelsim FFT');
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------Four types of Input signals------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure
subplot(2,2,1); plot(x1)
axis tight;
xlabel('n');
title('Sequence = "00", sin(2*pi*4/16*t)');
subplot(2,2,2); plot(20*log10(abs(fft(x1))))
axis tight;
xlabel('n');
ylabel('dB');
title('FFT of "00"');

subplot(2,2,3); plot(x2)
axis tight;
xlabel('n');
title('Sequence = "01",sin(2*pi*4.5/16*t)'); 
subplot(2,2,4); plot(20*log10(abs(fft(x2))))
axis tight;
xlabel('n');
ylabel('dB')
title('FFT of "01"');    
     
figure
subplot(2,2,1); plot(x3)
axis tight;
xlabel('n');
title('Sequence = "10", cos(2*pi*4/16*t)+i*sin(2*pi*4/16*t)');
subplot(2,2,2); plot(20*log10(abs(fft(x3))))
axis tight;
xlabel('n');
ylabel('dB');
title('FFT of "10"');

subplot(2,2,3); plot(x4)
axis tight;
xlabel('n');
title('Sequence = "11", cos(2*pi*4.5/16*t)+i*sin(2*pi*4.5/16*t)');
subplot(2,2,4); plot(20*log10(abs(fft(x4))))
axis tight;
xlabel('n');
ylabel('dB');
title('FFT of "11"');
