%-------------------------------------------------------------------------
%--           Copyright ERICSSON MICROWAVE SYSTEMS AB 2001              --
%-------------------------------------------------------------------------
%-- 
%-- 
%-- 
%-- The copyright to the source program herein is the
%-- property of Ericsson Microwave Systems AB, Sweden.
%-- The program may be used and/or copied only with the
%-- written permission from Ericsson Microwave Systems
%-- AB or in accordance with the terms and conditions 
%-- stipulated in the agreement/contract under which 
%-- the program have been supplied.
%-- 
%--  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--  
%-- Project     : FFT
%-- Title       : Tow's complement
%-- File name   : towscomp.m
%-- Document no : -
%-- Revision    : pa1
%-- Date        : 01-01-15
%-- Prepared    : FX/D Magnus Nilsson
%-- Responsible : FX/DC Anders Wanner
%-- Description : Makes binary tow's complement of integer value
%--
%-- ------------------------------------------------------------------------
%-- Revision history: (all revisions included)                            --
%-- ------------------------------------------------------------------------
%-- Rev:   Initials:     Date:    Modification:                           --
%-- ------------------------------------------------------------------------
%-- pa1     emwmsnn     01-01-15   First revision                         --
%-- ------------------------------------------------------------------------
%function output = towscomp(no_of_bits,DATA)
%     
%output =     Binary tow's complement value
%no_of bits = The number of bits that you want the integer value to be represented in
%DATA =       Vector of integer value

function output = towscomp(no_of_bits,DATA)
for s=1:length(DATA)
    dec_value=DATA(s);                      % Place the k:th DATA value in a new variable.      

    if (dec_value>2^(no_of_bits-0)-1)|(dec_value<-(2^(no_of_bits-0)))                                             
       error('Too big decimal input')
    end

    dummy_value=blanks(no_of_bits);         % Makes empty string.
    dummy_value(1:no_of_bits)='0';

    if (dec_value<0);                       % Looks for values less then  0.
       dec_value=2^no_of_bits+dec_value;    % Tow's complement.
    end

    bin_value=dec2bin(dec_value);
    dummy_value(no_of_bits-length(bin_value)+1:no_of_bits)=bin_value;        
    output(s,:)=dummy_value;     
end


