%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
--  
-------------------------------------------------------------------------
--           Copyright ERICSSON MICROWAVE SYSTEMS AB 2001              --
-------------------------------------------------------------------------
-- 
-- 
-- 
-- The copyright to the source program herein is the
-- property of Ericsson Microwave Systems AB, Sweden.
-- The program may be used and/or copied only with the
-- written permission from Ericsson Microwave Systems
-- AB or in accordance with the terms and conditions 
-- stipulated in the agreement/contract under which 
-- the program have been supplied.
-- 
--  
--------------------------------------------------------------------------
--------------------------------------------------------------------------
--  
-- Project     : FFT
-- Title       : Radix-4
-- File name   : Top_Level.vhd
-- Document no : -
-- Revision    : pa2
-- Date        : 00-12-23
-- Prepared    : FX/D Magnus Nilsson
-- Responsible : FX/DC Hkan Enskog
-- Description : Complex Radix-4 algorithm with structure and binary
--               twiddle factor
--
-- ------------------------------------------------------------------------
-- Revision history: (all revisions included)                            --
-- ------------------------------------------------------------------------
-- Rev:   Initials:     Date:    Modification:                           --
-- ------------------------------------------------------------------------
-- pa1     emwmsnn     00-12-23   First revision                         --
-- ------------------------------------------------------------------------
-- pa2     emwmsnn     01-01-05   Second revision                        --
-- ------------------------------------------------------------------------


---------------------------------------------------------------------------
-- Entity declaration of 'Valid'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Valid is
  generic(
    delay : Integer := 15 ) ;
  port(
    done_shift_in : in     std_logic ;
    clk           : in     std_logic ;
    reset         : in     std_logic ;
    data_valid    : out    std_logic ) ;
end  Valid ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Valid'
---------------------------------------------------------------------------

architecture a0 of Valid is

signal count          : integer;    
signal valid          : std_logic;

begin 

-------------------------------------Valid-------------------------------------
-- purpose : valid_fft
valid_fft : process (clk, reset)
    
begin  -- process valid_fft
    if reset = '0' then
        data_valid  <= '0';
	count <= 0;          		
	valid <= '0';
    elsif clk'event and clk = '1' then
        if done_shift_in = '1' then
            valid <= '1';
        end if;
        if valid = '1' then        	
			if count = (delay-1) then
				data_valid  <= '1';
	    		count <= 0;
	    		valid <= '0';           		   
			else
		    	count <= count +1;
	    		data_valid  <= '0'; 
			end if;	    		
	    else
	    	data_valid <= '0';		 	    	
		end if;	
    end if;
end process valid_fft;
end  a0 ; -- of Valid

---------------------------------------------------------------------------
-- Entity declaration of 'Shift_out'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Shift_out is
  generic(
    N            : Integer := 18 ;
    no_of_blocks : Integer := 64 ) ;
  port(
    Shift_out_r_0  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_1  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_2  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_3  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_4  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_5  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_6  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_7  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_8  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_9  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_10 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_11 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_12 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_13 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_14 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_15 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_16 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_17 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_18 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_19 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_20 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_21 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_22 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_23 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_24 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_25 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_26 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_27 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_28 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_29 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_30 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_31 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_32 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_33 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_34 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_35 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_36 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_37 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_38 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_39 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_40 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_41 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_42 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_43 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_44 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_45 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_46 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_47 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_48 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_49 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_50 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_51 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_52 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_53 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_54 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_55 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_56 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_57 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_58 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_59 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_60 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_61 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_62 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_63 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_0  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_1  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_2  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_3  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_4  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_5  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_6  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_7  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_8  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_9  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_10 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_11 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_12 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_13 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_14 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_15 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_16 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_17 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_18 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_19 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_20 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_21 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_22 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_23 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_24 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_25 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_26 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_27 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_28 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_29 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_30 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_31 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_32 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_33 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_34 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_35 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_36 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_37 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_38 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_39 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_40 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_41 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_42 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_43 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_44 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_45 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_46 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_47 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_48 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_49 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_50 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_51 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_52 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_53 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_54 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_55 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_56 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_57 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_58 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_59 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_60 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_61 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_62 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_63 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r    : out    std_logic_vector(17 downto 0) ;
    Shift_out_i    : out    std_logic_vector(17 downto 0) ;
    clk            : in     std_logic ;
    valid_data     : in     std_logic ;
    Data_valid_out : out    std_logic ;
    reset          : in     std_logic ) ;
end  Shift_out ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Shift_out'
---------------------------------------------------------------------------

architecture a0 of Shift_out is

signal count           : integer;
-- Real part
signal out_vector_r_0 : std_logic_vector((N-1) downto 0);
signal out_vector_r_1 : std_logic_vector((N-1) downto 0);
signal out_vector_r_2 : std_logic_vector((N-1) downto 0); 
signal out_vector_r_3 : std_logic_vector((N-1) downto 0);
signal out_vector_r_4 : std_logic_vector((N-1) downto 0);
signal out_vector_r_5 : std_logic_vector((N-1) downto 0);
signal out_vector_r_6 : std_logic_vector((N-1) downto 0);
signal out_vector_r_7 : std_logic_vector((N-1) downto 0);
signal out_vector_r_8 : std_logic_vector((N-1) downto 0);
signal out_vector_r_9 : std_logic_vector((N-1) downto 0);
signal out_vector_r_10 : std_logic_vector((N-1) downto 0);
signal out_vector_r_11 : std_logic_vector((N-1) downto 0);
signal out_vector_r_12 : std_logic_vector((N-1) downto 0);
signal out_vector_r_13 : std_logic_vector((N-1) downto 0);
signal out_vector_r_14 : std_logic_vector((N-1) downto 0);
signal out_vector_r_15 : std_logic_vector((N-1) downto 0);
signal out_vector_r_16 : std_logic_vector((N-1) downto 0);
signal out_vector_r_17 : std_logic_vector((N-1) downto 0);
signal out_vector_r_18 : std_logic_vector((N-1) downto 0);
signal out_vector_r_19 : std_logic_vector((N-1) downto 0);
signal out_vector_r_20 : std_logic_vector((N-1) downto 0);
signal out_vector_r_21 : std_logic_vector((N-1) downto 0);
signal out_vector_r_22 : std_logic_vector((N-1) downto 0);
signal out_vector_r_23 : std_logic_vector((N-1) downto 0);
signal out_vector_r_24 : std_logic_vector((N-1) downto 0);
signal out_vector_r_25 : std_logic_vector((N-1) downto 0);
signal out_vector_r_26 : std_logic_vector((N-1) downto 0);
signal out_vector_r_27 : std_logic_vector((N-1) downto 0);
signal out_vector_r_28 : std_logic_vector((N-1) downto 0);
signal out_vector_r_29 : std_logic_vector((N-1) downto 0);
signal out_vector_r_30 : std_logic_vector((N-1) downto 0);
signal out_vector_r_31 : std_logic_vector((N-1) downto 0);
signal out_vector_r_32 : std_logic_vector((N-1) downto 0);
signal out_vector_r_33 : std_logic_vector((N-1) downto 0);
signal out_vector_r_34 : std_logic_vector((N-1) downto 0);
signal out_vector_r_35 : std_logic_vector((N-1) downto 0);
signal out_vector_r_36 : std_logic_vector((N-1) downto 0);
signal out_vector_r_37 : std_logic_vector((N-1) downto 0);
signal out_vector_r_38 : std_logic_vector((N-1) downto 0);
signal out_vector_r_39 : std_logic_vector((N-1) downto 0);
signal out_vector_r_40 : std_logic_vector((N-1) downto 0);
signal out_vector_r_41 : std_logic_vector((N-1) downto 0);
signal out_vector_r_42 : std_logic_vector((N-1) downto 0);
signal out_vector_r_43 : std_logic_vector((N-1) downto 0);
signal out_vector_r_44 : std_logic_vector((N-1) downto 0);
signal out_vector_r_45 : std_logic_vector((N-1) downto 0);
signal out_vector_r_46 : std_logic_vector((N-1) downto 0);
signal out_vector_r_47 : std_logic_vector((N-1) downto 0);
signal out_vector_r_48 : std_logic_vector((N-1) downto 0);
signal out_vector_r_49 : std_logic_vector((N-1) downto 0);
signal out_vector_r_50 : std_logic_vector((N-1) downto 0);
signal out_vector_r_51 : std_logic_vector((N-1) downto 0);
signal out_vector_r_52 : std_logic_vector((N-1) downto 0);
signal out_vector_r_53 : std_logic_vector((N-1) downto 0);
signal out_vector_r_54 : std_logic_vector((N-1) downto 0);
signal out_vector_r_55 : std_logic_vector((N-1) downto 0);
signal out_vector_r_56 : std_logic_vector((N-1) downto 0);
signal out_vector_r_57 : std_logic_vector((N-1) downto 0);
signal out_vector_r_58 : std_logic_vector((N-1) downto 0);
signal out_vector_r_59 : std_logic_vector((N-1) downto 0);
signal out_vector_r_60 : std_logic_vector((N-1) downto 0);
signal out_vector_r_61 : std_logic_vector((N-1) downto 0);
signal out_vector_r_62 : std_logic_vector((N-1) downto 0);
signal out_vector_r_63 : std_logic_vector((N-1) downto 0); 
 -- Imaginary part
signal out_vector_i_0 : std_logic_vector((N-1) downto 0);
signal out_vector_i_1 : std_logic_vector((N-1) downto 0);
signal out_vector_i_2 : std_logic_vector((N-1) downto 0); 
signal out_vector_i_3 : std_logic_vector((N-1) downto 0);
signal out_vector_i_4 : std_logic_vector((N-1) downto 0);
signal out_vector_i_5 : std_logic_vector((N-1) downto 0);
signal out_vector_i_6 : std_logic_vector((N-1) downto 0);
signal out_vector_i_7 : std_logic_vector((N-1) downto 0);
signal out_vector_i_8 : std_logic_vector((N-1) downto 0);
signal out_vector_i_9 : std_logic_vector((N-1) downto 0);
signal out_vector_i_10 : std_logic_vector((N-1) downto 0);
signal out_vector_i_11 : std_logic_vector((N-1) downto 0);
signal out_vector_i_12 : std_logic_vector((N-1) downto 0);
signal out_vector_i_13 : std_logic_vector((N-1) downto 0);
signal out_vector_i_14 : std_logic_vector((N-1) downto 0);
signal out_vector_i_15 : std_logic_vector((N-1) downto 0);
signal out_vector_i_16 : std_logic_vector((N-1) downto 0);
signal out_vector_i_17 : std_logic_vector((N-1) downto 0);
signal out_vector_i_18 : std_logic_vector((N-1) downto 0);
signal out_vector_i_19 : std_logic_vector((N-1) downto 0);
signal out_vector_i_20 : std_logic_vector((N-1) downto 0);
signal out_vector_i_21 : std_logic_vector((N-1) downto 0);
signal out_vector_i_22 : std_logic_vector((N-1) downto 0);
signal out_vector_i_23 : std_logic_vector((N-1) downto 0);
signal out_vector_i_24 : std_logic_vector((N-1) downto 0);
signal out_vector_i_25 : std_logic_vector((N-1) downto 0);
signal out_vector_i_26 : std_logic_vector((N-1) downto 0);
signal out_vector_i_27 : std_logic_vector((N-1) downto 0);
signal out_vector_i_28 : std_logic_vector((N-1) downto 0);
signal out_vector_i_29 : std_logic_vector((N-1) downto 0);
signal out_vector_i_30 : std_logic_vector((N-1) downto 0);
signal out_vector_i_31 : std_logic_vector((N-1) downto 0);
signal out_vector_i_32 : std_logic_vector((N-1) downto 0);
signal out_vector_i_33 : std_logic_vector((N-1) downto 0);
signal out_vector_i_34 : std_logic_vector((N-1) downto 0);
signal out_vector_i_35 : std_logic_vector((N-1) downto 0);
signal out_vector_i_36 : std_logic_vector((N-1) downto 0);
signal out_vector_i_37 : std_logic_vector((N-1) downto 0);
signal out_vector_i_38 : std_logic_vector((N-1) downto 0);
signal out_vector_i_39 : std_logic_vector((N-1) downto 0);
signal out_vector_i_40 : std_logic_vector((N-1) downto 0);
signal out_vector_i_41 : std_logic_vector((N-1) downto 0);
signal out_vector_i_42 : std_logic_vector((N-1) downto 0);
signal out_vector_i_43 : std_logic_vector((N-1) downto 0);
signal out_vector_i_44 : std_logic_vector((N-1) downto 0);
signal out_vector_i_45 : std_logic_vector((N-1) downto 0);
signal out_vector_i_46 : std_logic_vector((N-1) downto 0);
signal out_vector_i_47 : std_logic_vector((N-1) downto 0);
signal out_vector_i_48 : std_logic_vector((N-1) downto 0);
signal out_vector_i_49 : std_logic_vector((N-1) downto 0);
signal out_vector_i_50 : std_logic_vector((N-1) downto 0);
signal out_vector_i_51 : std_logic_vector((N-1) downto 0);
signal out_vector_i_52 : std_logic_vector((N-1) downto 0);
signal out_vector_i_53 : std_logic_vector((N-1) downto 0);
signal out_vector_i_54 : std_logic_vector((N-1) downto 0);
signal out_vector_i_55 : std_logic_vector((N-1) downto 0);
signal out_vector_i_56 : std_logic_vector((N-1) downto 0);
signal out_vector_i_57 : std_logic_vector((N-1) downto 0);
signal out_vector_i_58 : std_logic_vector((N-1) downto 0);
signal out_vector_i_59 : std_logic_vector((N-1) downto 0);
signal out_vector_i_60 : std_logic_vector((N-1) downto 0);
signal out_vector_i_61 : std_logic_vector((N-1) downto 0);
signal out_vector_i_62 : std_logic_vector((N-1) downto 0);
signal out_vector_i_63 : std_logic_vector((N-1) downto 0);

begin 

-------------------------------------Shift_out------------------------------------- 
 
shift_out : process (clk, reset)
    
begin  -- process shift_out
    if reset = '0' then
        Data_valid_out  <= '0';
		count <= 0;
		Shift_out_r <= (others => '0');
		Shift_out_i <= (others => '0');
        out_vector_r_0 <= (others => '0');
        out_vector_r_1 <= (others => '0');
        out_vector_r_2 <= (others => '0'); 
        out_vector_r_3 <= (others => '0');
        out_vector_r_4 <= (others => '0');
        out_vector_r_5 <= (others => '0');
        out_vector_r_6 <= (others => '0');
        out_vector_r_7 <= (others => '0');
        out_vector_r_8 <= (others => '0');
        out_vector_r_9 <= (others => '0');
        out_vector_r_10 <= (others => '0');
        out_vector_r_11 <= (others => '0');
        out_vector_r_12 <= (others => '0');
        out_vector_r_13 <= (others => '0');
        out_vector_r_14 <= (others => '0');
        out_vector_r_15 <= (others => '0');
        out_vector_r_16 <= (others => '0');
        out_vector_r_17 <= (others => '0');
        out_vector_r_18 <= (others => '0');
        out_vector_r_19 <= (others => '0');
        out_vector_r_20 <= (others => '0');
        out_vector_r_21 <= (others => '0');
        out_vector_r_22 <= (others => '0');
        out_vector_r_23 <= (others => '0');
        out_vector_r_24 <= (others => '0');
        out_vector_r_25 <= (others => '0');
        out_vector_r_26 <= (others => '0');
        out_vector_r_27 <= (others => '0');
        out_vector_r_28 <= (others => '0');
        out_vector_r_29 <= (others => '0');
        out_vector_r_30 <= (others => '0');
        out_vector_r_31 <= (others => '0');
        out_vector_r_32 <= (others => '0');
        out_vector_r_33 <= (others => '0');
        out_vector_r_34 <= (others => '0');
        out_vector_r_35 <= (others => '0');
        out_vector_r_36 <= (others => '0');
        out_vector_r_37 <= (others => '0');
        out_vector_r_38 <= (others => '0');
        out_vector_r_39 <= (others => '0');
        out_vector_r_40 <= (others => '0');
        out_vector_r_41 <= (others => '0');
        out_vector_r_42 <= (others => '0');
        out_vector_r_43 <= (others => '0');
        out_vector_r_44 <= (others => '0');
        out_vector_r_45 <= (others => '0');
        out_vector_r_46 <= (others => '0');
        out_vector_r_47 <= (others => '0');
        out_vector_r_48 <= (others => '0');
        out_vector_r_49 <= (others => '0');
        out_vector_r_50 <= (others => '0');
        out_vector_r_51 <= (others => '0');
        out_vector_r_52 <= (others => '0');
        out_vector_r_53 <= (others => '0');
        out_vector_r_54 <= (others => '0');
        out_vector_r_55 <= (others => '0');
        out_vector_r_56 <= (others => '0');
        out_vector_r_57 <= (others => '0');
        out_vector_r_58 <= (others => '0');
        out_vector_r_59 <= (others => '0');
        out_vector_r_60 <= (others => '0');
        out_vector_r_61 <= (others => '0');
        out_vector_r_62 <= (others => '0');
        out_vector_r_63 <= (others => '0'); 
        out_vector_i_0 <= (others => '0');
        out_vector_i_1 <= (others => '0');
        out_vector_i_2 <= (others => '0'); 
        out_vector_i_3 <= (others => '0');
        out_vector_i_4 <= (others => '0');
        out_vector_i_5 <= (others => '0');
        out_vector_i_6 <= (others => '0');
        out_vector_i_7 <= (others => '0');
        out_vector_i_8 <= (others => '0');
        out_vector_i_9 <= (others => '0');
        out_vector_i_10 <= (others => '0');
        out_vector_i_11 <= (others => '0');
        out_vector_i_12 <= (others => '0');
        out_vector_i_13 <= (others => '0');
        out_vector_i_14 <= (others => '0');
        out_vector_i_15 <= (others => '0');
        out_vector_i_16 <= (others => '0');
        out_vector_i_17 <= (others => '0');
        out_vector_i_18 <= (others => '0');
        out_vector_i_19 <= (others => '0');
        out_vector_i_20 <= (others => '0');
        out_vector_i_21 <= (others => '0');
        out_vector_i_22 <= (others => '0');
        out_vector_i_23 <= (others => '0');
        out_vector_i_24 <= (others => '0');
        out_vector_i_25 <= (others => '0');
        out_vector_i_26 <= (others => '0');
        out_vector_i_27 <= (others => '0');
        out_vector_i_28 <= (others => '0');
        out_vector_i_29 <= (others => '0');
        out_vector_i_30 <= (others => '0');
        out_vector_i_31 <= (others => '0');
        out_vector_i_32 <= (others => '0');
        out_vector_i_33 <= (others => '0');
        out_vector_i_34 <= (others => '0');
        out_vector_i_35 <= (others => '0');
        out_vector_i_36 <= (others => '0');
        out_vector_i_37 <= (others => '0');
        out_vector_i_38 <= (others => '0');
        out_vector_i_39 <= (others => '0');
        out_vector_i_40 <= (others => '0');
        out_vector_i_41 <= (others => '0');
        out_vector_i_42 <= (others => '0');
        out_vector_i_43 <= (others => '0');
        out_vector_i_44 <= (others => '0');
        out_vector_i_45 <= (others => '0');
        out_vector_i_46 <= (others => '0');
        out_vector_i_47 <= (others => '0');
        out_vector_i_48 <= (others => '0');
        out_vector_i_49 <= (others => '0');
        out_vector_i_50 <= (others => '0');
        out_vector_i_51 <= (others => '0');
        out_vector_i_52 <= (others => '0');
        out_vector_i_53 <= (others => '0');
        out_vector_i_54 <= (others => '0');
        out_vector_i_55 <= (others => '0');
        out_vector_i_56 <= (others => '0');
        out_vector_i_57 <= (others => '0');
        out_vector_i_58 <= (others => '0');
        out_vector_i_59 <= (others => '0');
        out_vector_i_60 <= (others => '0');
        out_vector_i_61 <= (others => '0');
        out_vector_i_62 <= (others => '0');
        out_vector_i_63 <= (others => '0');	
   		
    elsif clk'event and clk = '1' then   
    	if valid_data = '1'  then
    	    Data_valid_out <= '1';   
    	    --Real part         
            out_vector_r_0 <= Shift_out_r_0;
            out_vector_r_1 <= Shift_out_r_1;
            out_vector_r_2 <= Shift_out_r_2; 
            out_vector_r_3 <= Shift_out_r_3;
            out_vector_r_4 <= Shift_out_r_4;
            out_vector_r_5 <= Shift_out_r_5;
            out_vector_r_6 <= Shift_out_r_6;
            out_vector_r_7 <= Shift_out_r_7;
            out_vector_r_8 <= Shift_out_r_8;
            out_vector_r_9 <= Shift_out_r_9;
            out_vector_r_10 <= Shift_out_r_10;
            out_vector_r_11 <= Shift_out_r_11;
            out_vector_r_12 <= Shift_out_r_12;
            out_vector_r_13 <= Shift_out_r_13;
            out_vector_r_14 <= Shift_out_r_14;
            out_vector_r_15 <= Shift_out_r_15;
            out_vector_r_16 <= Shift_out_r_16;
            out_vector_r_17 <= Shift_out_r_17;
            out_vector_r_18 <= Shift_out_r_18;
            out_vector_r_19 <= Shift_out_r_19;
            out_vector_r_20 <= Shift_out_r_20;
            out_vector_r_21 <= Shift_out_r_21;
            out_vector_r_22 <= Shift_out_r_22;
            out_vector_r_23 <= Shift_out_r_23;
            out_vector_r_24 <= Shift_out_r_24;
            out_vector_r_25 <= Shift_out_r_25;
            out_vector_r_26 <= Shift_out_r_26;
            out_vector_r_27 <= Shift_out_r_27;
            out_vector_r_28 <= Shift_out_r_28;
            out_vector_r_29 <= Shift_out_r_29;
            out_vector_r_30 <= Shift_out_r_30;
            out_vector_r_31 <= Shift_out_r_31;
            out_vector_r_32 <= Shift_out_r_32;
            out_vector_r_33 <= Shift_out_r_33;
            out_vector_r_34 <= Shift_out_r_34;
            out_vector_r_35 <= Shift_out_r_35;
            out_vector_r_36 <= Shift_out_r_36;
            out_vector_r_37 <= Shift_out_r_37;
            out_vector_r_38 <= Shift_out_r_38;
            out_vector_r_39 <= Shift_out_r_39;
            out_vector_r_40 <= Shift_out_r_40;
            out_vector_r_41 <= Shift_out_r_41;
            out_vector_r_42 <= Shift_out_r_42;
            out_vector_r_43 <= Shift_out_r_43;
            out_vector_r_44 <= Shift_out_r_44;
            out_vector_r_45 <= Shift_out_r_45;
            out_vector_r_46 <= Shift_out_r_46;
            out_vector_r_47 <= Shift_out_r_47;
            out_vector_r_48 <= Shift_out_r_48;
            out_vector_r_49 <= Shift_out_r_49;
            out_vector_r_50 <= Shift_out_r_50;
            out_vector_r_51 <= Shift_out_r_51;
            out_vector_r_52 <= Shift_out_r_52;
            out_vector_r_53 <= Shift_out_r_53;
            out_vector_r_54 <= Shift_out_r_54;
            out_vector_r_55 <= Shift_out_r_55;
            out_vector_r_56 <= Shift_out_r_56;
            out_vector_r_57 <= Shift_out_r_56;
            out_vector_r_58 <= Shift_out_r_58;
            out_vector_r_59 <= Shift_out_r_59;
            out_vector_r_60 <= Shift_out_r_60;
            out_vector_r_61 <= Shift_out_r_61;
            out_vector_r_62 <= Shift_out_r_62;
            out_vector_r_63 <= Shift_out_r_63; 
            -- Imaginary part
            out_vector_i_0 <= Shift_out_i_0;
            out_vector_i_1 <= Shift_out_i_1;
            out_vector_i_2 <= Shift_out_i_2; 
            out_vector_i_3 <= Shift_out_i_3;
            out_vector_i_4 <= Shift_out_i_4;
            out_vector_i_5 <= Shift_out_i_5;
            out_vector_i_6 <= Shift_out_i_6;
            out_vector_i_7 <= Shift_out_i_7;
            out_vector_i_8 <= Shift_out_i_8;
            out_vector_i_9 <= Shift_out_i_9;
            out_vector_i_10 <= Shift_out_i_10;
            out_vector_i_11 <= Shift_out_i_11;
            out_vector_i_12 <= Shift_out_i_12;
            out_vector_i_13 <= Shift_out_i_13;
            out_vector_i_14 <= Shift_out_i_14;
            out_vector_i_15 <= Shift_out_i_15;
            out_vector_i_16 <= Shift_out_i_16;
            out_vector_i_17 <= Shift_out_i_17;
            out_vector_i_18 <= Shift_out_i_18;
            out_vector_i_19 <= Shift_out_i_19;
            out_vector_i_20 <= Shift_out_i_20;
            out_vector_i_21 <= Shift_out_i_21;
            out_vector_i_22 <= Shift_out_i_22;
            out_vector_i_23 <= Shift_out_i_23;
            out_vector_i_24 <= Shift_out_i_24;
            out_vector_i_25 <= Shift_out_i_25;
            out_vector_i_26 <= Shift_out_i_26;
            out_vector_i_27 <= Shift_out_i_27;
            out_vector_i_28 <= Shift_out_i_28;
            out_vector_i_29 <= Shift_out_i_29;
            out_vector_i_30 <= Shift_out_i_30;
            out_vector_i_31 <= Shift_out_i_31;
            out_vector_i_32 <= Shift_out_i_32;
            out_vector_i_33 <= Shift_out_i_33;
            out_vector_i_34 <= Shift_out_i_34;
            out_vector_i_35 <= Shift_out_i_35;
            out_vector_i_36 <= Shift_out_i_36;
            out_vector_i_37 <= Shift_out_i_37;
            out_vector_i_38 <= Shift_out_i_38;
            out_vector_i_39 <= Shift_out_i_39;
            out_vector_i_40 <= Shift_out_i_40;
            out_vector_i_41 <= Shift_out_i_41;
            out_vector_i_42 <= Shift_out_i_42;
            out_vector_i_43 <= Shift_out_i_43;
            out_vector_i_44 <= Shift_out_i_44;
            out_vector_i_45 <= Shift_out_i_45;
            out_vector_i_46 <= Shift_out_i_46;
            out_vector_i_47 <= Shift_out_i_47;
            out_vector_i_48 <= Shift_out_i_48;
            out_vector_i_49 <= Shift_out_i_49;
            out_vector_i_50 <= Shift_out_i_50;
            out_vector_i_51 <= Shift_out_i_51;
            out_vector_i_52 <= Shift_out_i_52;
            out_vector_i_53 <= Shift_out_i_53;
            out_vector_i_54 <= Shift_out_i_54;
            out_vector_i_55 <= Shift_out_i_55;
            out_vector_i_56 <= Shift_out_i_56;
            out_vector_i_57 <= Shift_out_i_56;
            out_vector_i_58 <= Shift_out_i_58;
            out_vector_i_59 <= Shift_out_i_59;
            out_vector_i_60 <= Shift_out_i_60;
            out_vector_i_61 <= Shift_out_i_61;
            out_vector_i_62 <= Shift_out_i_62;
            out_vector_i_63 <= Shift_out_i_63;                                                
            Shift_out_r <= Shift_out_r_0;
            Shift_out_i <= Shift_out_i_0; 
            count <= 2;
        end if;
                         
        if count = 2 then
            Shift_out_r <= out_vector_r_1;  
            Shift_out_i <= out_vector_i_1;                                            
            count <= count +1;  
        elsif count = 3 then
            Shift_out_r <= out_vector_r_2;  
            Shift_out_i <= out_vector_i_2;                                            
            count <= count +1;
        elsif count = 4 then
            Shift_out_r <= out_vector_r_3;  
            Shift_out_i <= out_vector_i_3;                                            
            count <= count +1;
        elsif count = 5 then
            Shift_out_r <= out_vector_r_4;  
            Shift_out_i <= out_vector_i_4;                                            
            count <= count +1;
        elsif count = 6 then
            Shift_out_r <= out_vector_r_5;  
            Shift_out_i <= out_vector_i_5;                                            
            count <= count +1;    
        elsif count = 7 then
            Shift_out_r <= out_vector_r_6;  
            Shift_out_i <= out_vector_i_6;                                            
            count <= count +1;  
        elsif count = 8 then
            Shift_out_r <= out_vector_r_7;  
            Shift_out_i <= out_vector_i_7;                                            
            count <= count +1;
        elsif count = 9 then
            Shift_out_r <= out_vector_r_8;  
            Shift_out_i <= out_vector_i_8;                                            
            count <= count +1;
        elsif count = 10 then
            Shift_out_r <= out_vector_r_9;  
            Shift_out_i <= out_vector_i_9;                                            
            count <= count +1;
        elsif count = 11 then
            Shift_out_r <= out_vector_r_10;  
            Shift_out_i <= out_vector_i_10;                                            
            count <= count +1;   
        elsif count = 12 then
            Shift_out_r <= out_vector_r_11;  
            Shift_out_i <= out_vector_i_11;                                            
            count <= count +1;  
        elsif count = 13 then
            Shift_out_r <= out_vector_r_12;  
            Shift_out_i <= out_vector_i_12;                                            
            count <= count +1;
        elsif count = 14 then
            Shift_out_r <= out_vector_r_13;  
            Shift_out_i <= out_vector_i_13;                                            
            count <= count +1;
        elsif count = 15 then
            Shift_out_r <= out_vector_r_14;  
            Shift_out_i <= out_vector_i_14;                                            
            count <= count +1;
        elsif count = 16 then
            Shift_out_r <= out_vector_r_15;  
            Shift_out_i <= out_vector_i_15;                                            
            count <= count +1;   
        elsif count = 17 then
            Shift_out_r <= out_vector_r_16;  
            Shift_out_i <= out_vector_i_16;                                            
            count <= count +1;  
        elsif count = 18 then
            Shift_out_r <= out_vector_r_17;  
            Shift_out_i <= out_vector_i_17;                                            
            count <= count +1;
        elsif count = 19 then
            Shift_out_r <= out_vector_r_18;  
            Shift_out_i <= out_vector_i_18;                                            
            count <= count +1;
        elsif count = 20 then
            Shift_out_r <= out_vector_r_19;  
            Shift_out_i <= out_vector_i_19;                                            
            count <= count +1;
        elsif count = 21 then
            Shift_out_r <= out_vector_r_20;  
            Shift_out_i <= out_vector_i_20;                                            
            count <= count +1;   
        elsif count = 22 then
            Shift_out_r <= out_vector_r_21;  
            Shift_out_i <= out_vector_i_21;                                            
            count <= count +1;  
        elsif count = 23 then
            Shift_out_r <= out_vector_r_22;  
            Shift_out_i <= out_vector_i_22;                                            
            count <= count +1;
        elsif count = 24 then
            Shift_out_r <= out_vector_r_23;  
            Shift_out_i <= out_vector_i_23;                                            
            count <= count +1;
        elsif count = 25 then
            Shift_out_r <= out_vector_r_24;  
            Shift_out_i <= out_vector_i_24;                                            
            count <= count +1;
        elsif count = 26 then
            Shift_out_r <= out_vector_r_25;  
            Shift_out_i <= out_vector_i_25;                                            
            count <= count +1;   
        elsif count = 27 then
            Shift_out_r <= out_vector_r_26;  
            Shift_out_i <= out_vector_i_26;                                            
            count <= count +1;  
        elsif count = 28 then
            Shift_out_r <= out_vector_r_27;  
            Shift_out_i <= out_vector_i_27;                                            
            count <= count +1;
        elsif count = 29 then
            Shift_out_r <= out_vector_r_28;  
            Shift_out_i <= out_vector_i_28;                                            
            count <= count +1;
        elsif count = 30 then
            Shift_out_r <= out_vector_r_29;  
            Shift_out_i <= out_vector_i_29;                                            
            count <= count +1;
        elsif count = 31 then
            Shift_out_r <= out_vector_r_30;  
            Shift_out_i <= out_vector_i_30;                                            
            count <= count +1; 
        elsif count = 32 then
            Shift_out_r <= out_vector_r_31;  
            Shift_out_i <= out_vector_i_31;                                            
            count <= count +1;  
        elsif count = 33 then
            Shift_out_r <= out_vector_r_32;  
            Shift_out_i <= out_vector_i_32;                                            
            count <= count +1;
        elsif count = 34 then
            Shift_out_r <= out_vector_r_33;  
            Shift_out_i <= out_vector_i_33;                                            
            count <= count +1;
        elsif count = 35 then
            Shift_out_r <= out_vector_r_34;  
            Shift_out_i <= out_vector_i_34;                                            
            count <= count +1;
        elsif count = 36 then
            Shift_out_r <= out_vector_r_35;  
            Shift_out_i <= out_vector_i_35;                                            
            count <= count +1;   
        elsif count = 37 then
            Shift_out_r <= out_vector_r_36;  
            Shift_out_i <= out_vector_i_36;                                            
            count <= count +1;
        elsif count = 38 then
            Shift_out_r <= out_vector_r_37;  
            Shift_out_i <= out_vector_i_37;                                            
            count <= count +1;
        elsif count = 39 then
            Shift_out_r <= out_vector_r_38;  
            Shift_out_i <= out_vector_i_38;                                            
            count <= count +1;
        elsif count = 40 then
            Shift_out_r <= out_vector_r_39;  
            Shift_out_i <= out_vector_i_39;                                            
            count <= count +1;    
        elsif count = 41 then
            Shift_out_r <= out_vector_r_40;  
            Shift_out_i <= out_vector_i_40;                                            
            count <= count +1;  
        elsif count = 42 then
            Shift_out_r <= out_vector_r_41;  
            Shift_out_i <= out_vector_i_41;                                            
            count <= count +1;
        elsif count = 43 then
            Shift_out_r <= out_vector_r_42;  
            Shift_out_i <= out_vector_i_42;                                            
            count <= count +1;
        elsif count = 44 then
            Shift_out_r <= out_vector_r_43;  
            Shift_out_i <= out_vector_i_43;                                            
            count <= count +1;
        elsif count = 45 then
            Shift_out_r <= out_vector_r_44;  
            Shift_out_i <= out_vector_i_44;                                            
            count <= count +1;   
        elsif count = 46 then
            Shift_out_r <= out_vector_r_45;  
            Shift_out_i <= out_vector_i_45;                                            
            count <= count +1;  
        elsif count = 47 then
            Shift_out_r <= out_vector_r_46;  
            Shift_out_i <= out_vector_i_46;                                            
            count <= count +1;
        elsif count = 48 then
            Shift_out_r <= out_vector_r_47;  
            Shift_out_i <= out_vector_i_47;                                            
            count <= count +1;
        elsif count = 49 then
            Shift_out_r <= out_vector_r_48;  
            Shift_out_i <= out_vector_i_48;                                            
            count <= count +1;
        elsif count = 50 then
            Shift_out_r <= out_vector_r_49;  
            Shift_out_i <= out_vector_i_49;                                            
            count <= count +1;   
        elsif count = 51 then
            Shift_out_r <= out_vector_r_50;  
            Shift_out_i <= out_vector_i_50;                                            
            count <= count +1;  
        elsif count = 52 then
            Shift_out_r <= out_vector_r_51;  
            Shift_out_i <= out_vector_i_51;                                            
            count <= count +1;
        elsif count = 53 then
            Shift_out_r <= out_vector_r_52;  
            Shift_out_i <= out_vector_i_52;                                            
            count <= count +1;
        elsif count = 54 then
            Shift_out_r <= out_vector_r_53;  
            Shift_out_i <= out_vector_i_53;                                            
            count <= count +1;
        elsif count = 55 then
            Shift_out_r <= out_vector_r_54;  
            Shift_out_i <= out_vector_i_54;                                            
            count <= count +1;   
        elsif count = 56 then
            Shift_out_r <= out_vector_r_55;  
            Shift_out_i <= out_vector_i_55;                                            
            count <= count +1;  
        elsif count = 57 then
            Shift_out_r <= out_vector_r_56;  
            Shift_out_i <= out_vector_i_56;                                            
            count <= count +1;
        elsif count = 58 then
            Shift_out_r <= out_vector_r_57;  
            Shift_out_i <= out_vector_i_57;                                            
            count <= count +1;
        elsif count = 59 then
            Shift_out_r <= out_vector_r_58;  
            Shift_out_i <= out_vector_i_58;                                            
            count <= count +1;
        elsif count = 60 then
            Shift_out_r <= out_vector_r_59;  
            Shift_out_i <= out_vector_i_59;                                            
            count <= count +1;   
        elsif count = 61 then
            Shift_out_r <= out_vector_r_60;  
            Shift_out_i <= out_vector_i_60;                                            
            count <= count +1;  
        elsif count = 62 then
            Shift_out_r <= out_vector_r_61;  
            Shift_out_i <= out_vector_i_61;                                            
            count <= count +1;
        elsif count = 63 then
            Shift_out_r <= out_vector_r_62;  
            Shift_out_i <= out_vector_i_62;                                            
            count <= count +1;
        elsif count = 64 then
            Shift_out_r <= out_vector_r_63;  
            Shift_out_i <= out_vector_i_63;                                            
            count <= count +1;
        elsif count = 65 then
        	count <= 0;    
            Data_valid_out <= '0';    
		end if;
    end if;
end process shift_out;
end  a0 ; -- of Shift_out



---------------------------------------------------------------------------
-- Entity declaration of 'Shift_in'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Shift_in is
  generic(
    N            : Integer := 12 ;
    no_of_blocks : Integer := 64 ) ;
  port(
    Shift_in_r_0  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_1  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_2  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_3  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_4  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_5  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_6  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_7  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_8  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_9  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_10 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_11 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_12 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_13 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_14 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_15 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_16 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_17 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_18 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_19 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_20 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_21 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_22 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_23 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_24 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_25 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_26 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_27 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_28 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_29 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_30 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_31 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_32 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_33 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_34 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_35 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_36 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_37 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_38 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_39 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_40 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_41 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_42 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_43 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_44 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_45 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_46 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_47 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_48 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_49 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_50 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_51 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_52 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_53 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_54 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_55 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_56 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_57 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_58 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_59 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_60 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_61 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_62 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_63 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_0  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_1  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_2  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_3  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_4  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_5  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_6  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_7  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_8  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_9  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_10 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_11 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_12 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_13 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_14 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_15 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_16 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_17 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_18 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_19 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_20 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_21 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_22 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_23 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_24 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_25 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_26 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_27 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_28 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_29 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_30 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_31 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_32 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_33 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_34 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_35 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_36 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_37 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_38 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_39 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_40 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_41 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_42 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_43 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_44 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_45 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_46 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_47 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_48 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_49 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_50 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_51 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_52 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_53 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_54 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_55 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_56 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_58 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_59 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_57 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_60 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_61 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_62 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_63 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r    : in     std_logic_vector(11 downto 0) ;
    Shift_in_i    : in     std_logic_vector(11 downto 0) ;
    clk           : in     std_logic ;
    reset         : in     std_logic ;
    done          : out    std_logic ;
    Data_valid    : in     std_logic ) ;
end  Shift_in ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Shift_in'
---------------------------------------------------------------------------

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Shift_in'
---------------------------------------------------------------------------

architecture a0 of Shift_in is

signal count          : integer;

begin 

-------------------------------------Shift_in-------------------------------------
-- purpose : Shift_in real block
shift_in : process (clk, reset)
    
begin  -- process shift_in_r
    if reset = '0' then
        done  <= '0';
		count <= 0;    
        Shift_in_r_0  <= (others => '0');
        Shift_in_r_1  <= (others => '0');
        Shift_in_r_2  <= (others => '0');
        Shift_in_r_3  <= (others => '0');
        Shift_in_r_4  <= (others => '0');
        Shift_in_r_5  <= (others => '0');
        Shift_in_r_6  <= (others => '0');
        Shift_in_r_7  <= (others => '0');
        Shift_in_r_8  <= (others => '0');
        Shift_in_r_9  <= (others => '0');
        Shift_in_r_10 <= (others => '0');
        Shift_in_r_11 <= (others => '0');
        Shift_in_r_12 <= (others => '0');
        Shift_in_r_13 <= (others => '0');
        Shift_in_r_14 <= (others => '0');
        Shift_in_r_15 <= (others => '0');
        Shift_in_r_16 <= (others => '0');
        Shift_in_r_17 <= (others => '0');
        Shift_in_r_18 <= (others => '0');
        Shift_in_r_19 <= (others => '0');
        Shift_in_r_20 <= (others => '0');
        Shift_in_r_21 <= (others => '0');
        Shift_in_r_22 <= (others => '0');
        Shift_in_r_23 <= (others => '0');
        Shift_in_r_24 <= (others => '0');
        Shift_in_r_25 <= (others => '0');
        Shift_in_r_26 <= (others => '0');
        Shift_in_r_27 <= (others => '0');
        Shift_in_r_28 <= (others => '0');
        Shift_in_r_29 <= (others => '0');
        Shift_in_r_30 <= (others => '0');
        Shift_in_r_31 <= (others => '0');
        Shift_in_r_32 <= (others => '0');
        Shift_in_r_33 <= (others => '0');
        Shift_in_r_34 <= (others => '0');
        Shift_in_r_35 <= (others => '0');
        Shift_in_r_36 <= (others => '0');
        Shift_in_r_37 <= (others => '0');
        Shift_in_r_38 <= (others => '0');
        Shift_in_r_39 <= (others => '0');
        Shift_in_r_40 <= (others => '0');
        Shift_in_r_41 <= (others => '0');
        Shift_in_r_42 <= (others => '0');
        Shift_in_r_43 <= (others => '0');
        Shift_in_r_44 <= (others => '0');
        Shift_in_r_45 <= (others => '0');
        Shift_in_r_46 <= (others => '0');
        Shift_in_r_47 <= (others => '0');
        Shift_in_r_48 <= (others => '0');
        Shift_in_r_49 <= (others => '0');
        Shift_in_r_50 <= (others => '0');
        Shift_in_r_51 <= (others => '0');
        Shift_in_r_52 <= (others => '0');
        Shift_in_r_53 <= (others => '0');
        Shift_in_r_54 <= (others => '0');
        Shift_in_r_55 <= (others => '0');
        Shift_in_r_56 <= (others => '0');
        Shift_in_r_57 <= (others => '0');
        Shift_in_r_58 <= (others => '0');
        Shift_in_r_59 <= (others => '0');
        Shift_in_r_60 <= (others => '0');
        Shift_in_r_61 <= (others => '0');
        Shift_in_r_62 <= (others => '0');
        Shift_in_r_63 <= (others => '0');
        Shift_in_i_0  <= (others => '0');
        Shift_in_i_1  <= (others => '0');
        Shift_in_i_2  <= (others => '0');
        Shift_in_i_3  <= (others => '0');
        Shift_in_i_4  <= (others => '0');
        Shift_in_i_5  <= (others => '0');
        Shift_in_i_6  <= (others => '0');
        Shift_in_i_7  <= (others => '0');
        Shift_in_i_8  <= (others => '0');
        Shift_in_i_9  <= (others => '0');
        Shift_in_i_10 <= (others => '0');
        Shift_in_i_11 <= (others => '0');
        Shift_in_i_12 <= (others => '0');
        Shift_in_i_13 <= (others => '0');
        Shift_in_i_14 <= (others => '0');
        Shift_in_i_15 <= (others => '0');
        Shift_in_i_16 <= (others => '0');
        Shift_in_i_17 <= (others => '0');
        Shift_in_i_18 <= (others => '0');
        Shift_in_i_19 <= (others => '0');
        Shift_in_i_20 <= (others => '0');
        Shift_in_i_21 <= (others => '0');
        Shift_in_i_22 <= (others => '0');
        Shift_in_i_23 <= (others => '0');
        Shift_in_i_24 <= (others => '0');
        Shift_in_i_25 <= (others => '0');
        Shift_in_i_26 <= (others => '0');
        Shift_in_i_27 <= (others => '0');
        Shift_in_i_28 <= (others => '0');
        Shift_in_i_29 <= (others => '0');
        Shift_in_i_30 <= (others => '0');
        Shift_in_i_31 <= (others => '0');
        Shift_in_i_32 <= (others => '0');
        Shift_in_i_33 <= (others => '0');
        Shift_in_i_34 <= (others => '0');
        Shift_in_i_35 <= (others => '0');
        Shift_in_i_36 <= (others => '0');
        Shift_in_i_37 <= (others => '0');
        Shift_in_i_38 <= (others => '0');
        Shift_in_i_39 <= (others => '0');
        Shift_in_i_40 <= (others => '0');
        Shift_in_i_41 <= (others => '0');
        Shift_in_i_42 <= (others => '0');
        Shift_in_i_43 <= (others => '0');
        Shift_in_i_44 <= (others => '0');
        Shift_in_i_45 <= (others => '0');
        Shift_in_i_46 <= (others => '0');
        Shift_in_i_47 <= (others => '0');
        Shift_in_i_48 <= (others => '0');
        Shift_in_i_49 <= (others => '0');
        Shift_in_i_50 <= (others => '0');
        Shift_in_i_51 <= (others => '0');
        Shift_in_i_52 <= (others => '0');
        Shift_in_i_53 <= (others => '0');
        Shift_in_i_54 <= (others => '0');
        Shift_in_i_55 <= (others => '0');
        Shift_in_i_56 <= (others => '0');
        Shift_in_i_58 <= (others => '0');
        Shift_in_i_59 <= (others => '0');
        Shift_in_i_57 <= (others => '0');
        Shift_in_i_60 <= (others => '0');
        Shift_in_i_61 <= (others => '0');
        Shift_in_i_62 <= (others => '0');
        Shift_in_i_63 <= (others => '0');
      		
    elsif clk'event and clk = '1' then   
    	if Data_valid = '1' then  
    		if count = 0 then
    			done <= '0';    					  
                Shift_in_r_0 <= Shift_in_r;
                Shift_in_i_0 <= Shift_in_i;
                count <= count +1;
            elsif count = 1 then  
                Shift_in_r_1 <= Shift_in_r;
                Shift_in_i_1 <= Shift_in_i;
                count <= count +1;  
            elsif count = 2 then  
                Shift_in_r_2 <= Shift_in_r;
                Shift_in_i_2 <= Shift_in_i;
                count <= count +1;     
            elsif count = 3 then  
                Shift_in_r_3 <= Shift_in_r;
                Shift_in_i_3 <= Shift_in_i;
                count <= count +1;
            elsif count = 4 then  
                Shift_in_r_4 <= Shift_in_r;
                Shift_in_i_4 <= Shift_in_i;
                count <= count +1;
            elsif count = 5 then  
                Shift_in_r_5 <= Shift_in_r;
                Shift_in_i_5 <= Shift_in_i;
                count <= count +1;
            elsif count = 6 then  
                Shift_in_r_6 <= Shift_in_r;
                Shift_in_i_6 <= Shift_in_i;
                count <= count +1;                                                
            elsif count = 7 then  
                Shift_in_r_7 <= Shift_in_r;
                Shift_in_i_7 <= Shift_in_i;
                count <= count +1;
            elsif count = 8 then  
                Shift_in_r_8 <= Shift_in_r;
                Shift_in_i_8 <= Shift_in_i;
                count <= count +1;   
            elsif count = 9 then  
                Shift_in_r_9 <= Shift_in_r;
                Shift_in_i_9 <= Shift_in_i;
                count <= count +1;     
            elsif count = 10 then  
                Shift_in_r_10 <= Shift_in_r;
                Shift_in_i_10 <= Shift_in_i;
                count <= count +1;
            elsif count = 11 then  
                Shift_in_r_11 <= Shift_in_r;
                Shift_in_i_11 <= Shift_in_i;
                count <= count +1;
            elsif count = 12 then  
                Shift_in_r_12 <= Shift_in_r;
                Shift_in_i_12 <= Shift_in_i;
                count <= count +1;
            elsif count = 13 then  
                Shift_in_r_13 <= Shift_in_r;
                Shift_in_i_13 <= Shift_in_i;
                count <= count +1;                                                
            elsif count = 14 then  
                Shift_in_r_14 <= Shift_in_r;
                Shift_in_i_14 <= Shift_in_i;
                count <= count +1;
            elsif count = 15 then  
                Shift_in_r_15 <= Shift_in_r;
                Shift_in_i_15 <= Shift_in_i;
                count <= count +1; 
            elsif count = 16 then  
                Shift_in_r_16 <= Shift_in_r;
                Shift_in_i_16 <= Shift_in_i;
                count <= count +1;     
            elsif count = 17 then  
                Shift_in_r_17 <= Shift_in_r;
                Shift_in_i_17 <= Shift_in_i;
                count <= count +1;
            elsif count = 18 then  
                Shift_in_r_18 <= Shift_in_r;
                Shift_in_i_18 <= Shift_in_i;
                count <= count +1;
            elsif count = 19 then  
                Shift_in_r_19 <= Shift_in_r;
                Shift_in_i_19 <= Shift_in_i;
                count <= count +1;
            elsif count = 20 then  
                Shift_in_r_20 <= Shift_in_r;
                Shift_in_i_20 <= Shift_in_i;
                count <= count +1;                                                
            elsif count = 21 then  
                Shift_in_r_21 <= Shift_in_r;
                Shift_in_i_21 <= Shift_in_i;
                count <= count +1;
            elsif count = 22 then  
                Shift_in_r_22 <= Shift_in_r;
                Shift_in_i_22 <= Shift_in_i;
                count <= count +1; 
            elsif count = 23 then  
                Shift_in_r_23 <= Shift_in_r;
                Shift_in_i_23 <= Shift_in_i;
                count <= count +1;     
            elsif count = 24 then  
                Shift_in_r_24 <= Shift_in_r;
                Shift_in_i_24 <= Shift_in_i;
                count <= count +1;
            elsif count = 25 then  
                Shift_in_r_25 <= Shift_in_r;
                Shift_in_i_25 <= Shift_in_i;
                count <= count +1;
            elsif count = 26 then  
                Shift_in_r_26 <= Shift_in_r;
                Shift_in_i_26 <= Shift_in_i;
                count <= count +1;
            elsif count = 27 then  
                Shift_in_r_27 <= Shift_in_r;
                Shift_in_i_27 <= Shift_in_i;
                count <= count +1;                                                
            elsif count = 28 then  
                Shift_in_r_28 <= Shift_in_r;
                Shift_in_i_28 <= Shift_in_i;
                count <= count +1;
            elsif count = 29 then  
                Shift_in_r_29 <= Shift_in_r;
                Shift_in_i_29 <= Shift_in_i;
                count <= count +1; 
            elsif count = 30 then  
                Shift_in_r_30 <= Shift_in_r;
                Shift_in_i_30 <= Shift_in_i;
                count <= count +1;     
            elsif count = 31 then  
                Shift_in_r_31 <= Shift_in_r;
                Shift_in_i_31 <= Shift_in_i;
                count <= count +1;
            elsif count = 32 then  
                Shift_in_r_32 <= Shift_in_r;
                Shift_in_i_32 <= Shift_in_i;
                count <= count +1;
            elsif count = 33 then  
                Shift_in_r_33 <= Shift_in_r;
                Shift_in_i_33 <= Shift_in_i;
                count <= count +1;
            elsif count = 34 then  
                Shift_in_r_34 <= Shift_in_r;
                Shift_in_i_34 <= Shift_in_i;
                count <= count +1;                                                
            elsif count = 35 then  
                Shift_in_r_35 <= Shift_in_r;
                Shift_in_i_35 <= Shift_in_i;
                count <= count +1;
            elsif count = 36 then  
                Shift_in_r_36 <= Shift_in_r;
                Shift_in_i_36 <= Shift_in_i;
                count <= count +1; 
            elsif count = 37 then  
                Shift_in_r_37 <= Shift_in_r;
                Shift_in_i_37 <= Shift_in_i;
                count <= count +1;     
            elsif count = 38 then  
                Shift_in_r_38 <= Shift_in_r;
                Shift_in_i_38 <= Shift_in_i;
                count <= count +1;
            elsif count = 39 then  
                Shift_in_r_39 <= Shift_in_r;
                Shift_in_i_39 <= Shift_in_i;
                count <= count +1;
            elsif count = 40 then  
                Shift_in_r_40 <= Shift_in_r;
                Shift_in_i_40 <= Shift_in_i;
                count <= count +1; 
            elsif count = 41 then  
                Shift_in_r_41 <= Shift_in_r;
                Shift_in_i_41 <= Shift_in_i;
                count <= count +1;                  
            elsif count = 42 then  
                Shift_in_r_42 <= Shift_in_r;
                Shift_in_i_42 <= Shift_in_i;
                count <= count +1;                                                
            elsif count = 43 then  
                Shift_in_r_43 <= Shift_in_r;
                Shift_in_i_43 <= Shift_in_i;
                count <= count +1;
            elsif count = 44 then  
                Shift_in_r_44 <= Shift_in_r;
                Shift_in_i_44 <= Shift_in_i;
                count <= count +1; 
            elsif count = 45 then  
                Shift_in_r_45 <= Shift_in_r;
                Shift_in_i_45 <= Shift_in_i;
                count <= count +1;     
            elsif count = 46 then  
                Shift_in_r_46 <= Shift_in_r;
                Shift_in_i_46 <= Shift_in_i;
                count <= count +1;
            elsif count = 47 then  
                Shift_in_r_47 <= Shift_in_r;
                Shift_in_i_47 <= Shift_in_i;
                count <= count +1;
            elsif count = 48 then  
                Shift_in_r_48 <= Shift_in_r;
                Shift_in_i_48 <= Shift_in_i;
                count <= count +1;
            elsif count = 49 then  
                Shift_in_r_49 <= Shift_in_r;
                Shift_in_i_49 <= Shift_in_i;
                count <= count +1;                                                
            elsif count = 50 then  
                Shift_in_r_50 <= Shift_in_r;
                Shift_in_i_50 <= Shift_in_i;
                count <= count +1;
            elsif count = 51 then  
                Shift_in_r_51 <= Shift_in_r;
                Shift_in_i_51 <= Shift_in_i;
                count <= count +1; 
            elsif count = 52 then  
                Shift_in_r_52 <= Shift_in_r;
                Shift_in_i_52 <= Shift_in_i;
                count <= count +1;     
            elsif count = 53 then  
                Shift_in_r_53 <= Shift_in_r;
                Shift_in_i_53 <= Shift_in_i;
                count <= count +1;
            elsif count = 54 then  
                Shift_in_r_54 <= Shift_in_r;
                Shift_in_i_54 <= Shift_in_i;
                count <= count +1;
            elsif count = 55 then  
                Shift_in_r_55 <= Shift_in_r;
                Shift_in_i_55 <= Shift_in_i;
                count <= count +1;
            elsif count = 56 then  
                Shift_in_r_56 <= Shift_in_r;
                Shift_in_i_56 <= Shift_in_i;
                count <= count +1;                                                
            elsif count = 57 then  
                Shift_in_r_57 <= Shift_in_r;
                Shift_in_i_57 <= Shift_in_i;
                count <= count +1;
            elsif count = 58 then  
                Shift_in_r_58 <= Shift_in_r;
                Shift_in_i_58 <= Shift_in_i;
                count <= count +1; 
            elsif count = 59 then  
                Shift_in_r_59 <= Shift_in_r;
                Shift_in_i_59 <= Shift_in_i;
                count <= count +1;  
            elsif count = 60 then  
                Shift_in_r_60 <= Shift_in_r;
                Shift_in_i_60 <= Shift_in_i;
                count <= count +1;                   
            elsif count = 61 then  
                Shift_in_r_61 <= Shift_in_r;
                Shift_in_i_61 <= Shift_in_i;
                count <= count +1;
            elsif count = 62 then  
                Shift_in_r_62 <= Shift_in_r;
                Shift_in_i_62 <= Shift_in_i;
                count <= count +1;
            elsif count = 63 then  
                Shift_in_r_63 <= Shift_in_r;
                Shift_in_i_63 <= Shift_in_i;
                done  <= '1';                  
                count <= 0;              	    
        	end if; 
        else
        	count <= 0;
        	done <= '0';
        		           	
		end if;
    end if;
end process shift_in;
end  a0 ; -- of Shift_in

---------------------------------------------------------------------------
-- Entity declaration of 'Block47'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block47 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block47 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block47'
---------------------------------------------------------------------------

architecture a0 of Block47 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0000011001000110";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0011111110110001";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="1100000100111011";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0000110001111100";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1110110101101100";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="1100001011000001";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block47

---------------------------------------------------------------------------
-- Entity declaration of 'Block46'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block46 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block46 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block46'
---------------------------------------------------------------------------

architecture a0 of Block46 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0000110001111100";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0011111011000101";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="1100010011011111";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0001100001111110";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1101110001110010";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="1100101011001001";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block46

---------------------------------------------------------------------------
-- Entity declaration of 'Block45'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block45 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block45 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block45'
---------------------------------------------------------------------------

architecture a0 of Block45 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0001001010010100";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0011110100111111";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="1100101011001001";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0010001110001110";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1100111010000111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="1101011101100110";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block45

---------------------------------------------------------------------------
-- Entity declaration of 'Block44'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block44 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block44 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block44'
---------------------------------------------------------------------------

architecture a0 of Block44 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0001100001111110";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0011101100100001";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="1101001010111111";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0010110101000001";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1100010011011111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="1110011110000010";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block44

---------------------------------------------------------------------------
-- Entity declaration of 'Block43'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block43 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block43 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block43'
---------------------------------------------------------------------------

architecture a0 of Block43 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0001111000101011";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0011100001110001";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="1101110001110010";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0011010100110111";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1100000001001111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="1111100110111010";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block43

---------------------------------------------------------------------------
-- Entity declaration of 'Block42'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block42 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block42 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block42'
---------------------------------------------------------------------------

architecture a0 of Block42 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0010001110001110";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0011010100110111";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="1110011110000010";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0011101100100001";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1100000100111011";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0000110001111100";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block42

---------------------------------------------------------------------------
-- Entity declaration of 'Block41'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block41 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block41 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block41'
---------------------------------------------------------------------------

architecture a0 of Block41 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0010100010011010";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0011000101111001";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="1111001110000100";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0011111011000101";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1100011110001111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0001111000101011";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block41

---------------------------------------------------------------------------
-- Entity declaration of 'Block40'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block40 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block40 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block40'
---------------------------------------------------------------------------

architecture a0 of Block40 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0010110101000001";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0010110101000001";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="0000000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1101001010111111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0010110101000001";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block40

---------------------------------------------------------------------------
-- Entity declaration of 'Block39'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block39 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block39 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block39'
---------------------------------------------------------------------------

architecture a0 of Block39 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0011000101111001";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0010100010011010";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="0000110001111100";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0011111011000101";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1110000111010101";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0011100001110001";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block39

---------------------------------------------------------------------------
-- Entity declaration of 'Block38'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block38 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block38 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block38'
---------------------------------------------------------------------------

architecture a0 of Block38 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0011010100110111";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0010001110001110";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="0001100001111110";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0011101100100001";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="1111001110000100";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0011111011000101";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block38

---------------------------------------------------------------------------
-- Entity declaration of 'Block37'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block37 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block37 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block37'
---------------------------------------------------------------------------

architecture a0 of Block37 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0011100001110001";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0001111000101011";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="0010001110001110";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0011010100110111";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="0000011001000110";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0011111110110001";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block37

---------------------------------------------------------------------------
-- Entity declaration of 'Block36'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block36 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block36 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block36'
---------------------------------------------------------------------------

architecture a0 of Block36 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0011101100100001";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0001100001111110";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="0010110101000001";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0010110101000001";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="0001100001111110";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0011101100100001";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block36

---------------------------------------------------------------------------
-- Entity declaration of 'Block35'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block35 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block35 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block35'
---------------------------------------------------------------------------

architecture a0 of Block35 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0011110100111111";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0001001010010100";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="0011010100110111";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0010001110001110";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="0010100010011010";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0011000101111001";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block35

---------------------------------------------------------------------------
-- Entity declaration of 'Block34'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block34 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block34 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block34'
---------------------------------------------------------------------------

architecture a0 of Block34 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0011111011000101";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0000110001111100";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="0011101100100001";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0001100001111110";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="0011010100110111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0010001110001110";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block34

---------------------------------------------------------------------------
-- Entity declaration of 'Block33'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block33 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block33 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block33'
---------------------------------------------------------------------------

architecture a0 of Block33 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0011111110110001";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0000011001000110";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="0011111011000101";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0000110001111100";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="0011110100111111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0001001010010100";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block33

---------------------------------------------------------------------------
-- Entity declaration of 'Block32'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block32 is
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block32 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block32'
---------------------------------------------------------------------------

architecture a0 of Block32 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="0000000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="0000000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="0100000000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="0000000000000000";
constant round   : std_logic_vector((N+2) downto 0) :="0000000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block32

---------------------------------------------------------------------------
-- Entity declaration of 'Stage03'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Stage03 is
  generic(
    N : Integer := 16 ) ;
  port(
    S03_outr_0  : out    std_logic_vector(17 downto 0) ;
    S03_outr_1  : out    std_logic_vector(17 downto 0) ;
    S03_outr_2  : out    std_logic_vector(17 downto 0) ;
    S03_outr_3  : out    std_logic_vector(17 downto 0) ;
    S03_outr_4  : out    std_logic_vector(17 downto 0) ;
    S03_outr_5  : out    std_logic_vector(17 downto 0) ;
    S03_outr_6  : out    std_logic_vector(17 downto 0) ;
    S03_outr_7  : out    std_logic_vector(17 downto 0) ;
    S03_outr_8  : out    std_logic_vector(17 downto 0) ;
    S03_outr_9  : out    std_logic_vector(17 downto 0) ;
    S03_outr_10 : out    std_logic_vector(17 downto 0) ;
    S03_outr_11 : out    std_logic_vector(17 downto 0) ;
    S03_outr_12 : out    std_logic_vector(17 downto 0) ;
    S03_outr_13 : out    std_logic_vector(17 downto 0) ;
    S03_outr_14 : out    std_logic_vector(17 downto 0) ;
    S03_outr_15 : out    std_logic_vector(17 downto 0) ;
    S03_outr_16 : out    std_logic_vector(17 downto 0) ;
    S03_outr_17 : out    std_logic_vector(17 downto 0) ;
    S03_outr_18 : out    std_logic_vector(17 downto 0) ;
    S03_outr_19 : out    std_logic_vector(17 downto 0) ;
    S03_outr_20 : out    std_logic_vector(17 downto 0) ;
    S03_outr_21 : out    std_logic_vector(17 downto 0) ;
    S03_outr_22 : out    std_logic_vector(17 downto 0) ;
    S03_outr_23 : out    std_logic_vector(17 downto 0) ;
    S03_outr_24 : out    std_logic_vector(17 downto 0) ;
    S03_outr_25 : out    std_logic_vector(17 downto 0) ;
    S03_outr_26 : out    std_logic_vector(17 downto 0) ;
    S03_outr_27 : out    std_logic_vector(17 downto 0) ;
    S03_outr_28 : out    std_logic_vector(17 downto 0) ;
    S03_outr_29 : out    std_logic_vector(17 downto 0) ;
    S03_outr_30 : out    std_logic_vector(17 downto 0) ;
    S03_outr_31 : out    std_logic_vector(17 downto 0) ;
    S03_outr_32 : out    std_logic_vector(17 downto 0) ;
    S03_outr_33 : out    std_logic_vector(17 downto 0) ;
    S03_outr_34 : out    std_logic_vector(17 downto 0) ;
    S03_outr_35 : out    std_logic_vector(17 downto 0) ;
    S03_outr_36 : out    std_logic_vector(17 downto 0) ;
    S03_outr_37 : out    std_logic_vector(17 downto 0) ;
    S03_outr_38 : out    std_logic_vector(17 downto 0) ;
    S03_outr_39 : out    std_logic_vector(17 downto 0) ;
    S03_outr_40 : out    std_logic_vector(17 downto 0) ;
    S03_outr_41 : out    std_logic_vector(17 downto 0) ;
    S03_outr_42 : out    std_logic_vector(17 downto 0) ;
    S03_outr_43 : out    std_logic_vector(17 downto 0) ;
    S03_outr_44 : out    std_logic_vector(17 downto 0) ;
    S03_outr_45 : out    std_logic_vector(17 downto 0) ;
    S03_outr_46 : out    std_logic_vector(17 downto 0) ;
    S03_outr_47 : out    std_logic_vector(17 downto 0) ;
    S03_outr_48 : out    std_logic_vector(17 downto 0) ;
    S03_outr_49 : out    std_logic_vector(17 downto 0) ;
    S03_outr_50 : out    std_logic_vector(17 downto 0) ;
    S03_outr_51 : out    std_logic_vector(17 downto 0) ;
    S03_outr_52 : out    std_logic_vector(17 downto 0) ;
    S03_outr_53 : out    std_logic_vector(17 downto 0) ;
    S03_outr_54 : out    std_logic_vector(17 downto 0) ;
    S03_outr_55 : out    std_logic_vector(17 downto 0) ;
    S03_outr_56 : out    std_logic_vector(17 downto 0) ;
    S03_outr_57 : out    std_logic_vector(17 downto 0) ;
    S03_outr_58 : out    std_logic_vector(17 downto 0) ;
    S03_outr_59 : out    std_logic_vector(17 downto 0) ;
    S03_outr_60 : out    std_logic_vector(17 downto 0) ;
    S03_outr_61 : out    std_logic_vector(17 downto 0) ;
    S03_outr_62 : out    std_logic_vector(17 downto 0) ;
    S03_outr_63 : out    std_logic_vector(17 downto 0) ;
    S03_outi_0  : out    std_logic_vector(17 downto 0) ;
    S03_outi_1  : out    std_logic_vector(17 downto 0) ;
    S03_outi_2  : out    std_logic_vector(17 downto 0) ;
    S03_outi_3  : out    std_logic_vector(17 downto 0) ;
    S03_outi_4  : out    std_logic_vector(17 downto 0) ;
    S03_outi_5  : out    std_logic_vector(17 downto 0) ;
    S03_outi_6  : out    std_logic_vector(17 downto 0) ;
    S03_outi_7  : out    std_logic_vector(17 downto 0) ;
    S03_outi_8  : out    std_logic_vector(17 downto 0) ;
    S03_outi_9  : out    std_logic_vector(17 downto 0) ;
    S03_outi_10 : out    std_logic_vector(17 downto 0) ;
    S03_outi_11 : out    std_logic_vector(17 downto 0) ;
    S03_outi_12 : out    std_logic_vector(17 downto 0) ;
    S03_outi_13 : out    std_logic_vector(17 downto 0) ;
    S03_outi_14 : out    std_logic_vector(17 downto 0) ;
    S03_outi_15 : out    std_logic_vector(17 downto 0) ;
    S03_outi_16 : out    std_logic_vector(17 downto 0) ;
    S03_outi_17 : out    std_logic_vector(17 downto 0) ;
    S03_outi_18 : out    std_logic_vector(17 downto 0) ;
    S03_outi_19 : out    std_logic_vector(17 downto 0) ;
    S03_outi_20 : out    std_logic_vector(17 downto 0) ;
    S03_outi_21 : out    std_logic_vector(17 downto 0) ;
    S03_outi_22 : out    std_logic_vector(17 downto 0) ;
    S03_outi_23 : out    std_logic_vector(17 downto 0) ;
    S03_outi_24 : out    std_logic_vector(17 downto 0) ;
    S03_outi_25 : out    std_logic_vector(17 downto 0) ;
    S03_outi_26 : out    std_logic_vector(17 downto 0) ;
    S03_outi_27 : out    std_logic_vector(17 downto 0) ;
    S03_outi_28 : out    std_logic_vector(17 downto 0) ;
    S03_outi_29 : out    std_logic_vector(17 downto 0) ;
    S03_outi_30 : out    std_logic_vector(17 downto 0) ;
    S03_outi_31 : out    std_logic_vector(17 downto 0) ;
    S03_outi_32 : out    std_logic_vector(17 downto 0) ;
    S03_outi_33 : out    std_logic_vector(17 downto 0) ;
    S03_outi_34 : out    std_logic_vector(17 downto 0) ;
    S03_outi_35 : out    std_logic_vector(17 downto 0) ;
    S03_outi_36 : out    std_logic_vector(17 downto 0) ;
    S03_outi_37 : out    std_logic_vector(17 downto 0) ;
    S03_outi_38 : out    std_logic_vector(17 downto 0) ;
    S03_outi_39 : out    std_logic_vector(17 downto 0) ;
    S03_outi_40 : out    std_logic_vector(17 downto 0) ;
    S03_outi_41 : out    std_logic_vector(17 downto 0) ;
    S03_outi_42 : out    std_logic_vector(17 downto 0) ;
    S03_outi_43 : out    std_logic_vector(17 downto 0) ;
    S03_outi_44 : out    std_logic_vector(17 downto 0) ;
    S03_outi_45 : out    std_logic_vector(17 downto 0) ;
    S03_outi_46 : out    std_logic_vector(17 downto 0) ;
    S03_outi_47 : out    std_logic_vector(17 downto 0) ;
    S03_outi_48 : out    std_logic_vector(17 downto 0) ;
    S03_outi_49 : out    std_logic_vector(17 downto 0) ;
    S03_outi_50 : out    std_logic_vector(17 downto 0) ;
    S03_outi_51 : out    std_logic_vector(17 downto 0) ;
    S03_outi_52 : out    std_logic_vector(17 downto 0) ;
    S03_outi_53 : out    std_logic_vector(17 downto 0) ;
    S03_outi_54 : out    std_logic_vector(17 downto 0) ;
    S03_outi_55 : out    std_logic_vector(17 downto 0) ;
    S03_outi_56 : out    std_logic_vector(17 downto 0) ;
    S03_outi_58 : out    std_logic_vector(17 downto 0) ;
    S03_outi_59 : out    std_logic_vector(17 downto 0) ;
    S03_outi_57 : out    std_logic_vector(17 downto 0) ;
    S03_outi_60 : out    std_logic_vector(17 downto 0) ;
    S03_outi_61 : out    std_logic_vector(17 downto 0) ;
    S03_outi_62 : out    std_logic_vector(17 downto 0) ;
    S03_outi_63 : out    std_logic_vector(17 downto 0) ;
    S03_r_0     : in     std_logic_vector(15 downto 0) ;
    S03_r_1     : in     std_logic_vector(15 downto 0) ;
    S03_r_2     : in     std_logic_vector(15 downto 0) ;
    S03_r_3     : in     std_logic_vector(15 downto 0) ;
    S03_r_4     : in     std_logic_vector(15 downto 0) ;
    S03_r_5     : in     std_logic_vector(15 downto 0) ;
    S03_r_6     : in     std_logic_vector(15 downto 0) ;
    S03_r_7     : in     std_logic_vector(15 downto 0) ;
    S03_r_8     : in     std_logic_vector(15 downto 0) ;
    S03_r_9     : in     std_logic_vector(15 downto 0) ;
    S03_r_10    : in     std_logic_vector(15 downto 0) ;
    S03_r_11    : in     std_logic_vector(15 downto 0) ;
    S03_r_12    : in     std_logic_vector(15 downto 0) ;
    S03_r_13    : in     std_logic_vector(15 downto 0) ;
    S03_r_14    : in     std_logic_vector(15 downto 0) ;
    S03_r_15    : in     std_logic_vector(15 downto 0) ;
    S03_r_16    : in     std_logic_vector(15 downto 0) ;
    S03_r_17    : in     std_logic_vector(15 downto 0) ;
    S03_r_18    : in     std_logic_vector(15 downto 0) ;
    S03_r_19    : in     std_logic_vector(15 downto 0) ;
    S03_r_20    : in     std_logic_vector(15 downto 0) ;
    S03_r_21    : in     std_logic_vector(15 downto 0) ;
    S03_r_22    : in     std_logic_vector(15 downto 0) ;
    S03_r_23    : in     std_logic_vector(15 downto 0) ;
    S03_r_24    : in     std_logic_vector(15 downto 0) ;
    S03_r_25    : in     std_logic_vector(15 downto 0) ;
    S03_r_26    : in     std_logic_vector(15 downto 0) ;
    S03_r_27    : in     std_logic_vector(15 downto 0) ;
    S03_r_28    : in     std_logic_vector(15 downto 0) ;
    S03_r_29    : in     std_logic_vector(15 downto 0) ;
    S03_r_30    : in     std_logic_vector(15 downto 0) ;
    S03_r_31    : in     std_logic_vector(15 downto 0) ;
    S03_r_32    : in     std_logic_vector(15 downto 0) ;
    S03_r_33    : in     std_logic_vector(15 downto 0) ;
    S03_r_34    : in     std_logic_vector(15 downto 0) ;
    S03_r_35    : in     std_logic_vector(15 downto 0) ;
    S03_r_36    : in     std_logic_vector(15 downto 0) ;
    S03_r_37    : in     std_logic_vector(15 downto 0) ;
    S03_r_38    : in     std_logic_vector(15 downto 0) ;
    S03_r_39    : in     std_logic_vector(15 downto 0) ;
    S03_r_40    : in     std_logic_vector(15 downto 0) ;
    S03_r_41    : in     std_logic_vector(15 downto 0) ;
    S03_r_42    : in     std_logic_vector(15 downto 0) ;
    S03_r_43    : in     std_logic_vector(15 downto 0) ;
    S03_r_44    : in     std_logic_vector(15 downto 0) ;
    S03_r_45    : in     std_logic_vector(15 downto 0) ;
    S03_r_46    : in     std_logic_vector(15 downto 0) ;
    S03_r_47    : in     std_logic_vector(15 downto 0) ;
    S03_r_48    : in     std_logic_vector(15 downto 0) ;
    S03_r_49    : in     std_logic_vector(15 downto 0) ;
    S03_r_50    : in     std_logic_vector(15 downto 0) ;
    S03_r_51    : in     std_logic_vector(15 downto 0) ;
    S03_r_52    : in     std_logic_vector(15 downto 0) ;
    S03_r_53    : in     std_logic_vector(15 downto 0) ;
    S03_r_54    : in     std_logic_vector(15 downto 0) ;
    S03_r_55    : in     std_logic_vector(15 downto 0) ;
    S03_r_56    : in     std_logic_vector(15 downto 0) ;
    S03_r_57    : in     std_logic_vector(15 downto 0) ;
    S03_r_58    : in     std_logic_vector(15 downto 0) ;
    S03_r_59    : in     std_logic_vector(15 downto 0) ;
    S03_r_60    : in     std_logic_vector(15 downto 0) ;
    S03_r_61    : in     std_logic_vector(15 downto 0) ;
    S03_r_62    : in     std_logic_vector(15 downto 0) ;
    S03_r_63    : in     std_logic_vector(15 downto 0) ;
    S03_i_0     : in     std_logic_vector(15 downto 0) ;
    S03_i_1     : in     std_logic_vector(15 downto 0) ;
    S03_i_2     : in     std_logic_vector(15 downto 0) ;
    S03_i_3     : in     std_logic_vector(15 downto 0) ;
    S03_i_4     : in     std_logic_vector(15 downto 0) ;
    S03_i_5     : in     std_logic_vector(15 downto 0) ;
    S03_i_6     : in     std_logic_vector(15 downto 0) ;
    S03_i_7     : in     std_logic_vector(15 downto 0) ;
    S03_i_8     : in     std_logic_vector(15 downto 0) ;
    S03_i_9     : in     std_logic_vector(15 downto 0) ;
    S03_i_10    : in     std_logic_vector(15 downto 0) ;
    S03_i_11    : in     std_logic_vector(15 downto 0) ;
    S03_i_12    : in     std_logic_vector(15 downto 0) ;
    S03_i_13    : in     std_logic_vector(15 downto 0) ;
    S03_i_14    : in     std_logic_vector(15 downto 0) ;
    S03_i_15    : in     std_logic_vector(15 downto 0) ;
    S03_i_16    : in     std_logic_vector(15 downto 0) ;
    S03_i_17    : in     std_logic_vector(15 downto 0) ;
    S03_i_18    : in     std_logic_vector(15 downto 0) ;
    S03_i_19    : in     std_logic_vector(15 downto 0) ;
    S03_i_20    : in     std_logic_vector(15 downto 0) ;
    S03_i_21    : in     std_logic_vector(15 downto 0) ;
    S03_i_22    : in     std_logic_vector(15 downto 0) ;
    S03_i_23    : in     std_logic_vector(15 downto 0) ;
    S03_i_24    : in     std_logic_vector(15 downto 0) ;
    S03_i_25    : in     std_logic_vector(15 downto 0) ;
    S03_i_26    : in     std_logic_vector(15 downto 0) ;
    S03_i_27    : in     std_logic_vector(15 downto 0) ;
    S03_i_28    : in     std_logic_vector(15 downto 0) ;
    S03_i_29    : in     std_logic_vector(15 downto 0) ;
    S03_i_30    : in     std_logic_vector(15 downto 0) ;
    S03_i_31    : in     std_logic_vector(15 downto 0) ;
    S03_i_32    : in     std_logic_vector(15 downto 0) ;
    S03_i_33    : in     std_logic_vector(15 downto 0) ;
    S03_i_34    : in     std_logic_vector(15 downto 0) ;
    S03_i_35    : in     std_logic_vector(15 downto 0) ;
    S03_i_36    : in     std_logic_vector(15 downto 0) ;
    S03_i_37    : in     std_logic_vector(15 downto 0) ;
    S03_i_38    : in     std_logic_vector(15 downto 0) ;
    S03_i_39    : in     std_logic_vector(15 downto 0) ;
    S03_i_40    : in     std_logic_vector(15 downto 0) ;
    S03_i_41    : in     std_logic_vector(15 downto 0) ;
    S03_i_42    : in     std_logic_vector(15 downto 0) ;
    S03_i_43    : in     std_logic_vector(15 downto 0) ;
    S03_i_44    : in     std_logic_vector(15 downto 0) ;
    S03_i_45    : in     std_logic_vector(15 downto 0) ;
    S03_i_46    : in     std_logic_vector(15 downto 0) ;
    S03_i_47    : in     std_logic_vector(15 downto 0) ;
    S03_i_48    : in     std_logic_vector(15 downto 0) ;
    S03_i_49    : in     std_logic_vector(15 downto 0) ;
    S03_i_50    : in     std_logic_vector(15 downto 0) ;
    S03_i_51    : in     std_logic_vector(15 downto 0) ;
    S03_i_52    : in     std_logic_vector(15 downto 0) ;
    S03_i_53    : in     std_logic_vector(15 downto 0) ;
    S03_i_54    : in     std_logic_vector(15 downto 0) ;
    S03_i_55    : in     std_logic_vector(15 downto 0) ;
    S03_i_56    : in     std_logic_vector(15 downto 0) ;
    S03_i_57    : in     std_logic_vector(15 downto 0) ;
    S03_i_58    : in     std_logic_vector(15 downto 0) ;
    S03_i_59    : in     std_logic_vector(15 downto 0) ;
    S03_i_60    : in     std_logic_vector(15 downto 0) ;
    S03_i_61    : in     std_logic_vector(15 downto 0) ;
    S03_i_62    : in     std_logic_vector(15 downto 0) ;
    S03_i_63    : in     std_logic_vector(15 downto 0) ;
    clk         : in     std_logic ;
    reset       : in     std_logic ) ;
end  Stage03 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Stage03'
---------------------------------------------------------------------------

architecture a0 of Stage03 is


 component Block32
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block33
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block34
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block35
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block36
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block37
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block38
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block39
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block40
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block41
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block42
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block43
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block44
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block45
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block46
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block47
  generic(
    N : Integer := 16 ) ;
  port(
    in0r   : in     std_logic_vector(15 downto 0) ;
    in0i   : in     std_logic_vector(15 downto 0) ;
    in1r   : in     std_logic_vector(15 downto 0) ;
    in1i   : in     std_logic_vector(15 downto 0) ;
    in2r   : in     std_logic_vector(15 downto 0) ;
    in2i   : in     std_logic_vector(15 downto 0) ;
    in3r   : in     std_logic_vector(15 downto 0) ;
    in3i   : in     std_logic_vector(15 downto 0) ;
    ar_out : out    std_logic_vector(17 downto 0) ;
    ai_out : out    std_logic_vector(17 downto 0) ;
    br_out : out    std_logic_vector(17 downto 0) ;
    bi_out : out    std_logic_vector(17 downto 0) ;
    cr_out : out    std_logic_vector(17 downto 0) ;
    ci_out : out    std_logic_vector(17 downto 0) ;
    dr_out : out    std_logic_vector(17 downto 0) ;
    di_out : out    std_logic_vector(17 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;

begin

 u7: Block32
    port map(
      in0r => S03_r_0,
      in0i => S03_i_0,
      in1r => S03_r_1,
      in1i => S03_i_1,
      in2r => S03_r_2,
      in2i => S03_i_2,
      in3r => S03_r_3,
      in3i => S03_i_3,
      ar_out => S03_outr_0,
      ai_out => S03_outi_0,
      br_out => S03_outr_16,
      bi_out => S03_outi_16,
      cr_out => S03_outr_32,
      ci_out => S03_outi_32,
      dr_out => S03_outr_48,
      di_out => S03_outi_48,
      clk => clk,
      reset => reset ) ;

 u1: Block33
    port map(
      in0r => S03_r_4,
      in0i => S03_i_4,
      in1r => S03_r_5,
      in1i => S03_i_5,
      in2r => S03_r_6,
      in2i => S03_i_6,
      in3r => S03_r_7,
      in3i => S03_i_7,
      ar_out => S03_outr_1,
      ai_out => S03_outi_1,
      br_out => S03_outr_17,
      bi_out => S03_outi_17,
      cr_out => S03_outr_33,
      ci_out => S03_outi_33,
      dr_out => S03_outr_49,
      di_out => S03_outi_49,
      clk => clk,
      reset => reset ) ;

 u2: Block34
    port map(
      in0r => S03_r_8,
      in0i => S03_i_8,
      in1r => S03_r_9,
      in1i => S03_i_9,
      in2r => S03_r_10,
      in2i => S03_i_10,
      in3r => S03_r_11,
      in3i => S03_i_11,
      ar_out => S03_outr_2,
      ai_out => S03_outi_2,
      br_out => S03_outr_18,
      bi_out => S03_outi_18,
      cr_out => S03_outr_34,
      ci_out => S03_outi_34,
      dr_out => S03_outr_50,
      di_out => S03_outi_50,
      clk => clk,
      reset => reset ) ;

 u3: Block35
    port map(
      in0r => S03_r_12,
      in0i => S03_i_12,
      in1r => S03_r_13,
      in1i => S03_i_13,
      in2r => S03_r_14,
      in2i => S03_i_14,
      in3r => S03_r_15,
      in3i => S03_i_15,
      ar_out => S03_outr_3,
      ai_out => S03_outi_3,
      br_out => S03_outr_19,
      bi_out => S03_outi_19,
      cr_out => S03_outr_35,
      ci_out => S03_outi_35,
      dr_out => S03_outr_51,
      di_out => S03_outi_51,
      clk => clk,
      reset => reset ) ;

 u4: Block36
    port map(
      in0r => S03_r_16,
      in0i => S03_i_16,
      in1r => S03_r_17,
      in1i => S03_i_17,
      in2r => S03_r_18,
      in2i => S03_i_18,
      in3r => S03_r_19,
      in3i => S03_i_19,
      ar_out => S03_outr_4,
      ai_out => S03_outi_4,
      br_out => S03_outr_20,
      bi_out => S03_outi_20,
      cr_out => S03_outr_36,
      ci_out => S03_outi_36,
      dr_out => S03_outr_52,
      di_out => S03_outi_52,
      clk => clk,
      reset => reset ) ;

 u5: Block37
    port map(
      in0r => S03_r_20,
      in0i => S03_i_20,
      in1r => S03_r_21,
      in1i => S03_i_21,
      in2r => S03_r_22,
      in2i => S03_i_22,
      in3r => S03_r_23,
      in3i => S03_i_23,
      ar_out => S03_outr_5,
      ai_out => S03_outi_5,
      br_out => S03_outr_21,
      bi_out => S03_outi_21,
      cr_out => S03_outr_37,
      ci_out => S03_outi_37,
      dr_out => S03_outr_53,
      di_out => S03_outi_53,
      clk => clk,
      reset => reset ) ;

 u0: Block38
    port map(
      in0r => S03_r_24,
      in0i => S03_i_24,
      in1r => S03_r_25,
      in1i => S03_i_25,
      in2r => S03_r_26,
      in2i => S03_i_26,
      in3r => S03_r_27,
      in3i => S03_i_27,
      ar_out => S03_outr_6,
      ai_out => S03_outi_6,
      br_out => S03_outr_22,
      bi_out => S03_outi_22,
      cr_out => S03_outr_38,
      ci_out => S03_outi_38,
      dr_out => S03_outr_54,
      di_out => S03_outi_54,
      clk => clk,
      reset => reset ) ;

 u6: Block39
    port map(
      in0r => S03_r_28,
      in0i => S03_i_28,
      in1r => S03_r_29,
      in1i => S03_i_29,
      in2r => S03_r_30,
      in2i => S03_i_30,
      in3r => S03_r_31,
      in3i => S03_i_31,
      ar_out => S03_outr_7,
      ai_out => S03_outi_7,
      br_out => S03_outr_23,
      bi_out => S03_outi_23,
      cr_out => S03_outr_39,
      ci_out => S03_outi_39,
      dr_out => S03_outr_55,
      di_out => S03_outi_55,
      clk => clk,
      reset => reset ) ;

 u8: Block40
    port map(
      in0r => S03_r_32,
      in0i => S03_i_32,
      in1r => S03_r_33,
      in1i => S03_i_33,
      in2r => S03_r_34,
      in2i => S03_i_34,
      in3r => S03_r_35,
      in3i => S03_i_35,
      ar_out => S03_outr_8,
      ai_out => S03_outi_8,
      br_out => S03_outr_24,
      bi_out => S03_outi_24,
      cr_out => S03_outr_40,
      ci_out => S03_outi_40,
      dr_out => S03_outr_56,
      di_out => S03_outi_56,
      clk => clk,
      reset => reset ) ;

 u9: Block41
    port map(
      in0r => S03_r_36,
      in0i => S03_i_36,
      in1r => S03_r_37,
      in1i => S03_i_37,
      in2r => S03_r_38,
      in2i => S03_i_38,
      in3r => S03_r_39,
      in3i => S03_i_39,
      ar_out => S03_outr_9,
      ai_out => S03_outi_9,
      br_out => S03_outr_25,
      bi_out => S03_outi_25,
      cr_out => S03_outr_41,
      ci_out => S03_outi_41,
      dr_out => S03_outr_57,
      di_out => S03_outi_57,
      clk => clk,
      reset => reset ) ;

 u10: Block42
    port map(
      in0r => S03_r_40,
      in0i => S03_i_40,
      in1r => S03_r_41,
      in1i => S03_i_41,
      in2r => S03_r_42,
      in2i => S03_i_42,
      in3r => S03_r_43,
      in3i => S03_i_43,
      ar_out => S03_outr_10,
      ai_out => S03_outi_10,
      br_out => S03_outr_26,
      bi_out => S03_outi_26,
      cr_out => S03_outr_42,
      ci_out => S03_outi_42,
      dr_out => S03_outr_58,
      di_out => S03_outi_58,
      clk => clk,
      reset => reset ) ;

 u11: Block43
    port map(
      in0r => S03_r_44,
      in0i => S03_i_44,
      in1r => S03_r_45,
      in1i => S03_i_45,
      in2r => S03_r_46,
      in2i => S03_i_46,
      in3r => S03_r_47,
      in3i => S03_i_47,
      ar_out => S03_outr_11,
      ai_out => S03_outi_11,
      br_out => S03_outr_27,
      bi_out => S03_outi_27,
      cr_out => S03_outr_43,
      ci_out => S03_outi_43,
      dr_out => S03_outr_59,
      di_out => S03_outi_59,
      clk => clk,
      reset => reset ) ;

 u12: Block44
    port map(
      in0r => S03_r_48,
      in0i => S03_i_48,
      in1r => S03_r_49,
      in1i => S03_i_49,
      in2r => S03_r_50,
      in2i => S03_i_50,
      in3r => S03_r_51,
      in3i => S03_i_51,
      ar_out => S03_outr_12,
      ai_out => S03_outi_12,
      br_out => S03_outr_28,
      bi_out => S03_outi_28,
      cr_out => S03_outr_44,
      ci_out => S03_outi_44,
      dr_out => S03_outr_60,
      di_out => S03_outi_60,
      clk => clk,
      reset => reset ) ;

 u13: Block45
    port map(
      in0r => S03_r_52,
      in0i => S03_i_52,
      in1r => S03_r_53,
      in1i => S03_i_53,
      in2r => S03_r_54,
      in2i => S03_i_54,
      in3r => S03_r_55,
      in3i => S03_i_55,
      ar_out => S03_outr_13,
      ai_out => S03_outi_13,
      br_out => S03_outr_29,
      bi_out => S03_outi_29,
      cr_out => S03_outr_45,
      ci_out => S03_outi_45,
      dr_out => S03_outr_61,
      di_out => S03_outi_61,
      clk => clk,
      reset => reset ) ;

 u14: Block46
    port map(
      in0r => S03_r_56,
      in0i => S03_i_56,
      in1r => S03_r_57,
      in1i => S03_i_57,
      in2r => S03_r_58,
      in2i => S03_i_58,
      in3r => S03_r_59,
      in3i => S03_i_59,
      ar_out => S03_outr_14,
      ai_out => S03_outi_14,
      br_out => S03_outr_30,
      bi_out => S03_outi_30,
      cr_out => S03_outr_46,
      ci_out => S03_outi_46,
      dr_out => S03_outr_62,
      di_out => S03_outi_62,
      clk => clk,
      reset => reset ) ;

 u15: Block47
    port map(
      in0r => S03_r_60,
      in0i => S03_i_60,
      in1r => S03_r_61,
      in1i => S03_i_61,
      in2r => S03_r_62,
      in2i => S03_i_62,
      in3r => S03_r_63,
      in3i => S03_i_63,
      ar_out => S03_outr_15,
      ai_out => S03_outi_15,
      br_out => S03_outr_31,
      bi_out => S03_outi_31,
      cr_out => S03_outr_47,
      ci_out => S03_outi_47,
      dr_out => S03_outr_63,
      di_out => S03_outi_63,
      clk => clk,
      reset => reset ) ;
end  a0 ; -- of Stage03

---------------------------------------------------------------------------
-- Entity declaration of 'Block31'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block31 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block31 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block31'
---------------------------------------------------------------------------

architecture a0 of Block31 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="11010010110000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="11000100111000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="11100111100001";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block31

---------------------------------------------------------------------------
-- Entity declaration of 'Block30'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block30 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block30 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block30'
---------------------------------------------------------------------------

architecture a0 of Block30 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="11010010110000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="11000100111000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="11100111100001";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block30

---------------------------------------------------------------------------
-- Entity declaration of 'Block29'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block29 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block29 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block29'
---------------------------------------------------------------------------

architecture a0 of Block29 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="11010010110000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="11000100111000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="11100111100001";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block29

---------------------------------------------------------------------------
-- Entity declaration of 'Block28'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block28 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block28 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block28'
---------------------------------------------------------------------------

architecture a0 of Block28 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="11010010110000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="11000100111000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="11100111100001";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block28

---------------------------------------------------------------------------
-- Entity declaration of 'Block27'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block27 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block27 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block27'
---------------------------------------------------------------------------

architecture a0 of Block27 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="11010010110000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block27

---------------------------------------------------------------------------
-- Entity declaration of 'Block26'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block26 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block26 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block26'
---------------------------------------------------------------------------

architecture a0 of Block26 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="11010010110000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block26

---------------------------------------------------------------------------
-- Entity declaration of 'Block25'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block25 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block25 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block25'
---------------------------------------------------------------------------

architecture a0 of Block25 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="11010010110000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block25

---------------------------------------------------------------------------
-- Entity declaration of 'Block24'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block24 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block24 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block24'
---------------------------------------------------------------------------

architecture a0 of Block24 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="11010010110000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block24

---------------------------------------------------------------------------
-- Entity declaration of 'Block23'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block23 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block23 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block23'
---------------------------------------------------------------------------

architecture a0 of Block23 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block23

---------------------------------------------------------------------------
-- Entity declaration of 'Block22'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block22 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block22 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block22'
---------------------------------------------------------------------------

architecture a0 of Block22 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block22

---------------------------------------------------------------------------
-- Entity declaration of 'Block21'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block21 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block21 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block21'
---------------------------------------------------------------------------

architecture a0 of Block21 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block21

---------------------------------------------------------------------------
-- Entity declaration of 'Block20'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block20 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block20 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block20'
---------------------------------------------------------------------------

architecture a0 of Block20 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00101101010000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="00011000011111";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00111011001000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block20

---------------------------------------------------------------------------
-- Entity declaration of 'Block16'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block16 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block16 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block16'
---------------------------------------------------------------------------

architecture a0 of Block16 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block16

---------------------------------------------------------------------------
-- Entity declaration of 'Block19'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block19 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block19 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block19'
---------------------------------------------------------------------------

architecture a0 of Block19 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block19

---------------------------------------------------------------------------
-- Entity declaration of 'Block18'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block18 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block18 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block18'
---------------------------------------------------------------------------

architecture a0 of Block18 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block18

---------------------------------------------------------------------------
-- Entity declaration of 'Block17'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block17 is
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block17 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block17'
---------------------------------------------------------------------------

architecture a0 of Block17 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="01000000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="00000000000000";
constant round   : std_logic_vector((N+2) downto 0) :="00000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block17

---------------------------------------------------------------------------
-- Entity declaration of 'Stage02'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Stage02 is
  generic(
    N : Integer := 14 ) ;
  port(
    S02_outr_0  : out    std_logic_vector(15 downto 0) ;
    S02_outr_1  : out    std_logic_vector(15 downto 0) ;
    S02_outr_2  : out    std_logic_vector(15 downto 0) ;
    S02_outr_3  : out    std_logic_vector(15 downto 0) ;
    S02_outr_4  : out    std_logic_vector(15 downto 0) ;
    S02_outr_5  : out    std_logic_vector(15 downto 0) ;
    S02_outr_6  : out    std_logic_vector(15 downto 0) ;
    S02_outr_7  : out    std_logic_vector(15 downto 0) ;
    S02_outr_8  : out    std_logic_vector(15 downto 0) ;
    S02_outr_9  : out    std_logic_vector(15 downto 0) ;
    S02_outr_10 : out    std_logic_vector(15 downto 0) ;
    S02_outr_11 : out    std_logic_vector(15 downto 0) ;
    S02_outr_12 : out    std_logic_vector(15 downto 0) ;
    S02_outr_13 : out    std_logic_vector(15 downto 0) ;
    S02_outr_14 : out    std_logic_vector(15 downto 0) ;
    S02_outr_15 : out    std_logic_vector(15 downto 0) ;
    S02_outr_16 : out    std_logic_vector(15 downto 0) ;
    S02_outr_17 : out    std_logic_vector(15 downto 0) ;
    S02_outr_18 : out    std_logic_vector(15 downto 0) ;
    S02_outr_19 : out    std_logic_vector(15 downto 0) ;
    S02_outr_20 : out    std_logic_vector(15 downto 0) ;
    S02_outr_21 : out    std_logic_vector(15 downto 0) ;
    S02_outr_22 : out    std_logic_vector(15 downto 0) ;
    S02_outr_23 : out    std_logic_vector(15 downto 0) ;
    S02_outr_24 : out    std_logic_vector(15 downto 0) ;
    S02_outr_25 : out    std_logic_vector(15 downto 0) ;
    S02_outr_26 : out    std_logic_vector(15 downto 0) ;
    S02_outr_27 : out    std_logic_vector(15 downto 0) ;
    S02_outr_28 : out    std_logic_vector(15 downto 0) ;
    S02_outr_29 : out    std_logic_vector(15 downto 0) ;
    S02_outr_30 : out    std_logic_vector(15 downto 0) ;
    S02_outr_31 : out    std_logic_vector(15 downto 0) ;
    S02_outr_32 : out    std_logic_vector(15 downto 0) ;
    S02_outr_33 : out    std_logic_vector(15 downto 0) ;
    S02_outr_34 : out    std_logic_vector(15 downto 0) ;
    S02_outr_35 : out    std_logic_vector(15 downto 0) ;
    S02_outr_36 : out    std_logic_vector(15 downto 0) ;
    S02_outr_37 : out    std_logic_vector(15 downto 0) ;
    S02_outr_38 : out    std_logic_vector(15 downto 0) ;
    S02_outr_39 : out    std_logic_vector(15 downto 0) ;
    S02_outr_40 : out    std_logic_vector(15 downto 0) ;
    S02_outr_41 : out    std_logic_vector(15 downto 0) ;
    S02_outr_42 : out    std_logic_vector(15 downto 0) ;
    S02_outr_43 : out    std_logic_vector(15 downto 0) ;
    S02_outr_44 : out    std_logic_vector(15 downto 0) ;
    S02_outr_45 : out    std_logic_vector(15 downto 0) ;
    S02_outr_46 : out    std_logic_vector(15 downto 0) ;
    S02_outr_47 : out    std_logic_vector(15 downto 0) ;
    S02_outr_48 : out    std_logic_vector(15 downto 0) ;
    S02_outr_49 : out    std_logic_vector(15 downto 0) ;
    S02_outr_50 : out    std_logic_vector(15 downto 0) ;
    S02_outr_51 : out    std_logic_vector(15 downto 0) ;
    S02_outr_52 : out    std_logic_vector(15 downto 0) ;
    S02_outr_53 : out    std_logic_vector(15 downto 0) ;
    S02_outr_54 : out    std_logic_vector(15 downto 0) ;
    S02_outr_55 : out    std_logic_vector(15 downto 0) ;
    S02_outr_56 : out    std_logic_vector(15 downto 0) ;
    S02_outr_57 : out    std_logic_vector(15 downto 0) ;
    S02_outr_58 : out    std_logic_vector(15 downto 0) ;
    S02_outr_59 : out    std_logic_vector(15 downto 0) ;
    S02_outr_60 : out    std_logic_vector(15 downto 0) ;
    S02_outr_61 : out    std_logic_vector(15 downto 0) ;
    S02_outr_62 : out    std_logic_vector(15 downto 0) ;
    S02_outr_63 : out    std_logic_vector(15 downto 0) ;
    S02_outi_0  : out    std_logic_vector(15 downto 0) ;
    S02_outi_1  : out    std_logic_vector(15 downto 0) ;
    S02_outi_2  : out    std_logic_vector(15 downto 0) ;
    S02_outi_3  : out    std_logic_vector(15 downto 0) ;
    S02_outi_4  : out    std_logic_vector(15 downto 0) ;
    S02_outi_5  : out    std_logic_vector(15 downto 0) ;
    S02_outi_6  : out    std_logic_vector(15 downto 0) ;
    S02_outi_7  : out    std_logic_vector(15 downto 0) ;
    S02_outi_8  : out    std_logic_vector(15 downto 0) ;
    S02_outi_9  : out    std_logic_vector(15 downto 0) ;
    S02_outi_10 : out    std_logic_vector(15 downto 0) ;
    S02_outi_11 : out    std_logic_vector(15 downto 0) ;
    S02_outi_12 : out    std_logic_vector(15 downto 0) ;
    S02_outi_13 : out    std_logic_vector(15 downto 0) ;
    S02_outi_14 : out    std_logic_vector(15 downto 0) ;
    S02_outi_15 : out    std_logic_vector(15 downto 0) ;
    S02_outi_16 : out    std_logic_vector(15 downto 0) ;
    S02_outi_17 : out    std_logic_vector(15 downto 0) ;
    S02_outi_18 : out    std_logic_vector(15 downto 0) ;
    S02_outi_19 : out    std_logic_vector(15 downto 0) ;
    S02_outi_20 : out    std_logic_vector(15 downto 0) ;
    S02_outi_21 : out    std_logic_vector(15 downto 0) ;
    S02_outi_22 : out    std_logic_vector(15 downto 0) ;
    S02_outi_23 : out    std_logic_vector(15 downto 0) ;
    S02_outi_24 : out    std_logic_vector(15 downto 0) ;
    S02_outi_25 : out    std_logic_vector(15 downto 0) ;
    S02_outi_26 : out    std_logic_vector(15 downto 0) ;
    S02_outi_27 : out    std_logic_vector(15 downto 0) ;
    S02_outi_28 : out    std_logic_vector(15 downto 0) ;
    S02_outi_29 : out    std_logic_vector(15 downto 0) ;
    S02_outi_30 : out    std_logic_vector(15 downto 0) ;
    S02_outi_31 : out    std_logic_vector(15 downto 0) ;
    S02_outi_32 : out    std_logic_vector(15 downto 0) ;
    S02_outi_33 : out    std_logic_vector(15 downto 0) ;
    S02_outi_34 : out    std_logic_vector(15 downto 0) ;
    S02_outi_35 : out    std_logic_vector(15 downto 0) ;
    S02_outi_36 : out    std_logic_vector(15 downto 0) ;
    S02_outi_37 : out    std_logic_vector(15 downto 0) ;
    S02_outi_38 : out    std_logic_vector(15 downto 0) ;
    S02_outi_39 : out    std_logic_vector(15 downto 0) ;
    S02_outi_40 : out    std_logic_vector(15 downto 0) ;
    S02_outi_41 : out    std_logic_vector(15 downto 0) ;
    S02_outi_42 : out    std_logic_vector(15 downto 0) ;
    S02_outi_43 : out    std_logic_vector(15 downto 0) ;
    S02_outi_44 : out    std_logic_vector(15 downto 0) ;
    S02_outi_45 : out    std_logic_vector(15 downto 0) ;
    S02_outi_46 : out    std_logic_vector(15 downto 0) ;
    S02_outi_47 : out    std_logic_vector(15 downto 0) ;
    S02_outi_48 : out    std_logic_vector(15 downto 0) ;
    S02_outi_49 : out    std_logic_vector(15 downto 0) ;
    S02_outi_50 : out    std_logic_vector(15 downto 0) ;
    S02_outi_51 : out    std_logic_vector(15 downto 0) ;
    S02_outi_52 : out    std_logic_vector(15 downto 0) ;
    S02_outi_53 : out    std_logic_vector(15 downto 0) ;
    S02_outi_54 : out    std_logic_vector(15 downto 0) ;
    S02_outi_55 : out    std_logic_vector(15 downto 0) ;
    S02_outi_56 : out    std_logic_vector(15 downto 0) ;
    S02_outi_58 : out    std_logic_vector(15 downto 0) ;
    S02_outi_59 : out    std_logic_vector(15 downto 0) ;
    S02_outi_57 : out    std_logic_vector(15 downto 0) ;
    S02_outi_60 : out    std_logic_vector(15 downto 0) ;
    S02_outi_61 : out    std_logic_vector(15 downto 0) ;
    S02_outi_62 : out    std_logic_vector(15 downto 0) ;
    S02_outi_63 : out    std_logic_vector(15 downto 0) ;
    S02_r_0     : in     std_logic_vector(13 downto 0) ;
    S02_r_1     : in     std_logic_vector(13 downto 0) ;
    S02_r_2     : in     std_logic_vector(13 downto 0) ;
    S02_r_3     : in     std_logic_vector(13 downto 0) ;
    S02_r_4     : in     std_logic_vector(13 downto 0) ;
    S02_r_5     : in     std_logic_vector(13 downto 0) ;
    S02_r_6     : in     std_logic_vector(13 downto 0) ;
    S02_r_7     : in     std_logic_vector(13 downto 0) ;
    S02_r_8     : in     std_logic_vector(13 downto 0) ;
    S02_r_9     : in     std_logic_vector(13 downto 0) ;
    S02_r_10    : in     std_logic_vector(13 downto 0) ;
    S02_r_11    : in     std_logic_vector(13 downto 0) ;
    S02_r_12    : in     std_logic_vector(13 downto 0) ;
    S02_r_13    : in     std_logic_vector(13 downto 0) ;
    S02_r_14    : in     std_logic_vector(13 downto 0) ;
    S02_r_15    : in     std_logic_vector(13 downto 0) ;
    S02_r_16    : in     std_logic_vector(13 downto 0) ;
    S02_r_17    : in     std_logic_vector(13 downto 0) ;
    S02_r_18    : in     std_logic_vector(13 downto 0) ;
    S02_r_19    : in     std_logic_vector(13 downto 0) ;
    S02_r_20    : in     std_logic_vector(13 downto 0) ;
    S02_r_21    : in     std_logic_vector(13 downto 0) ;
    S02_r_22    : in     std_logic_vector(13 downto 0) ;
    S02_r_23    : in     std_logic_vector(13 downto 0) ;
    S02_r_24    : in     std_logic_vector(13 downto 0) ;
    S02_r_25    : in     std_logic_vector(13 downto 0) ;
    S02_r_26    : in     std_logic_vector(13 downto 0) ;
    S02_r_27    : in     std_logic_vector(13 downto 0) ;
    S02_r_28    : in     std_logic_vector(13 downto 0) ;
    S02_r_29    : in     std_logic_vector(13 downto 0) ;
    S02_r_30    : in     std_logic_vector(13 downto 0) ;
    S02_r_31    : in     std_logic_vector(13 downto 0) ;
    S02_r_32    : in     std_logic_vector(13 downto 0) ;
    S02_r_33    : in     std_logic_vector(13 downto 0) ;
    S02_r_34    : in     std_logic_vector(13 downto 0) ;
    S02_r_35    : in     std_logic_vector(13 downto 0) ;
    S02_r_36    : in     std_logic_vector(13 downto 0) ;
    S02_r_37    : in     std_logic_vector(13 downto 0) ;
    S02_r_38    : in     std_logic_vector(13 downto 0) ;
    S02_r_39    : in     std_logic_vector(13 downto 0) ;
    S02_r_40    : in     std_logic_vector(13 downto 0) ;
    S02_r_41    : in     std_logic_vector(13 downto 0) ;
    S02_r_42    : in     std_logic_vector(13 downto 0) ;
    S02_r_43    : in     std_logic_vector(13 downto 0) ;
    S02_r_44    : in     std_logic_vector(13 downto 0) ;
    S02_r_45    : in     std_logic_vector(13 downto 0) ;
    S02_r_46    : in     std_logic_vector(13 downto 0) ;
    S02_r_47    : in     std_logic_vector(13 downto 0) ;
    S02_r_48    : in     std_logic_vector(13 downto 0) ;
    S02_r_49    : in     std_logic_vector(13 downto 0) ;
    S02_r_50    : in     std_logic_vector(13 downto 0) ;
    S02_r_51    : in     std_logic_vector(13 downto 0) ;
    S02_r_52    : in     std_logic_vector(13 downto 0) ;
    S02_r_53    : in     std_logic_vector(13 downto 0) ;
    S02_r_54    : in     std_logic_vector(13 downto 0) ;
    S02_r_55    : in     std_logic_vector(13 downto 0) ;
    S02_r_56    : in     std_logic_vector(13 downto 0) ;
    S02_r_57    : in     std_logic_vector(13 downto 0) ;
    S02_r_58    : in     std_logic_vector(13 downto 0) ;
    S02_r_59    : in     std_logic_vector(13 downto 0) ;
    S02_r_60    : in     std_logic_vector(13 downto 0) ;
    S02_r_61    : in     std_logic_vector(13 downto 0) ;
    S02_r_62    : in     std_logic_vector(13 downto 0) ;
    S02_r_63    : in     std_logic_vector(13 downto 0) ;
    S02_i_0     : in     std_logic_vector(13 downto 0) ;
    S02_i_1     : in     std_logic_vector(13 downto 0) ;
    S02_i_2     : in     std_logic_vector(13 downto 0) ;
    S02_i_3     : in     std_logic_vector(13 downto 0) ;
    S02_i_4     : in     std_logic_vector(13 downto 0) ;
    S02_i_5     : in     std_logic_vector(13 downto 0) ;
    S02_i_6     : in     std_logic_vector(13 downto 0) ;
    S02_i_7     : in     std_logic_vector(13 downto 0) ;
    S02_i_8     : in     std_logic_vector(13 downto 0) ;
    S02_i_9     : in     std_logic_vector(13 downto 0) ;
    S02_i_10    : in     std_logic_vector(13 downto 0) ;
    S02_i_11    : in     std_logic_vector(13 downto 0) ;
    S02_i_12    : in     std_logic_vector(13 downto 0) ;
    S02_i_13    : in     std_logic_vector(13 downto 0) ;
    S02_i_14    : in     std_logic_vector(13 downto 0) ;
    S02_i_15    : in     std_logic_vector(13 downto 0) ;
    S02_i_16    : in     std_logic_vector(13 downto 0) ;
    S02_i_17    : in     std_logic_vector(13 downto 0) ;
    S02_i_18    : in     std_logic_vector(13 downto 0) ;
    S02_i_19    : in     std_logic_vector(13 downto 0) ;
    S02_i_20    : in     std_logic_vector(13 downto 0) ;
    S02_i_21    : in     std_logic_vector(13 downto 0) ;
    S02_i_22    : in     std_logic_vector(13 downto 0) ;
    S02_i_23    : in     std_logic_vector(13 downto 0) ;
    S02_i_24    : in     std_logic_vector(13 downto 0) ;
    S02_i_25    : in     std_logic_vector(13 downto 0) ;
    S02_i_26    : in     std_logic_vector(13 downto 0) ;
    S02_i_27    : in     std_logic_vector(13 downto 0) ;
    S02_i_28    : in     std_logic_vector(13 downto 0) ;
    S02_i_29    : in     std_logic_vector(13 downto 0) ;
    S02_i_30    : in     std_logic_vector(13 downto 0) ;
    S02_i_31    : in     std_logic_vector(13 downto 0) ;
    S02_i_32    : in     std_logic_vector(13 downto 0) ;
    S02_i_33    : in     std_logic_vector(13 downto 0) ;
    S02_i_34    : in     std_logic_vector(13 downto 0) ;
    S02_i_35    : in     std_logic_vector(13 downto 0) ;
    S02_i_36    : in     std_logic_vector(13 downto 0) ;
    S02_i_37    : in     std_logic_vector(13 downto 0) ;
    S02_i_38    : in     std_logic_vector(13 downto 0) ;
    S02_i_39    : in     std_logic_vector(13 downto 0) ;
    S02_i_40    : in     std_logic_vector(13 downto 0) ;
    S02_i_41    : in     std_logic_vector(13 downto 0) ;
    S02_i_42    : in     std_logic_vector(13 downto 0) ;
    S02_i_43    : in     std_logic_vector(13 downto 0) ;
    S02_i_44    : in     std_logic_vector(13 downto 0) ;
    S02_i_45    : in     std_logic_vector(13 downto 0) ;
    S02_i_46    : in     std_logic_vector(13 downto 0) ;
    S02_i_47    : in     std_logic_vector(13 downto 0) ;
    S02_i_48    : in     std_logic_vector(13 downto 0) ;
    S02_i_49    : in     std_logic_vector(13 downto 0) ;
    S02_i_50    : in     std_logic_vector(13 downto 0) ;
    S02_i_51    : in     std_logic_vector(13 downto 0) ;
    S02_i_52    : in     std_logic_vector(13 downto 0) ;
    S02_i_53    : in     std_logic_vector(13 downto 0) ;
    S02_i_54    : in     std_logic_vector(13 downto 0) ;
    S02_i_55    : in     std_logic_vector(13 downto 0) ;
    S02_i_56    : in     std_logic_vector(13 downto 0) ;
    S02_i_57    : in     std_logic_vector(13 downto 0) ;
    S02_i_58    : in     std_logic_vector(13 downto 0) ;
    S02_i_59    : in     std_logic_vector(13 downto 0) ;
    S02_i_60    : in     std_logic_vector(13 downto 0) ;
    S02_i_61    : in     std_logic_vector(13 downto 0) ;
    S02_i_62    : in     std_logic_vector(13 downto 0) ;
    S02_i_63    : in     std_logic_vector(13 downto 0) ;
    clk         : in     std_logic ;
    reset       : in     std_logic ) ;
end  Stage02 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Stage02'
---------------------------------------------------------------------------

architecture a0 of Stage02 is


 component Block17
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block18
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block19
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block16
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block20
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block21
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block22
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block23
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block24
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block25
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block26
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block27
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block28
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block29
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block30
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block31
  generic(
    N : Integer := 14 ) ;
  port(
    in0r   : in     std_logic_vector(13 downto 0) ;
    in0i   : in     std_logic_vector(13 downto 0) ;
    in1r   : in     std_logic_vector(13 downto 0) ;
    in1i   : in     std_logic_vector(13 downto 0) ;
    in2r   : in     std_logic_vector(13 downto 0) ;
    in2i   : in     std_logic_vector(13 downto 0) ;
    in3r   : in     std_logic_vector(13 downto 0) ;
    in3i   : in     std_logic_vector(13 downto 0) ;
    ar_out : out    std_logic_vector(15 downto 0) ;
    ai_out : out    std_logic_vector(15 downto 0) ;
    br_out : out    std_logic_vector(15 downto 0) ;
    bi_out : out    std_logic_vector(15 downto 0) ;
    cr_out : out    std_logic_vector(15 downto 0) ;
    ci_out : out    std_logic_vector(15 downto 0) ;
    dr_out : out    std_logic_vector(15 downto 0) ;
    di_out : out    std_logic_vector(15 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;

begin

 u1: Block17
    port map(
      in0r => S02_r_1,
      in0i => S02_i_1,
      in1r => S02_r_5,
      in1i => S02_i_5,
      in2r => S02_r_9,
      in2i => S02_i_9,
      in3r => S02_r_13,
      in3i => S02_i_13,
      ar_out => S02_outr_1,
      ai_out => S02_outi_1,
      br_out => S02_outr_17,
      bi_out => S02_outi_17,
      cr_out => S02_outr_33,
      ci_out => S02_outi_33,
      dr_out => S02_outr_49,
      di_out => S02_outi_49,
      clk => clk,
      reset => reset ) ;

 u2: Block18
    port map(
      in0r => S02_r_2,
      in0i => S02_i_2,
      in1r => S02_r_6,
      in1i => S02_i_6,
      in2r => S02_r_10,
      in2i => S02_i_10,
      in3r => S02_r_14,
      in3i => S02_i_14,
      ar_out => S02_outr_2,
      ai_out => S02_outi_2,
      br_out => S02_outr_18,
      bi_out => S02_outi_18,
      cr_out => S02_outr_34,
      ci_out => S02_outi_34,
      dr_out => S02_outr_50,
      di_out => S02_outi_50,
      clk => clk,
      reset => reset ) ;

 u3: Block19
    port map(
      in0r => S02_r_3,
      in0i => S02_i_3,
      in1r => S02_r_7,
      in1i => S02_i_7,
      in2r => S02_r_11,
      in2i => S02_i_11,
      in3r => S02_r_15,
      in3i => S02_i_15,
      ar_out => S02_outr_3,
      ai_out => S02_outi_3,
      br_out => S02_outr_19,
      bi_out => S02_outi_19,
      cr_out => S02_outr_35,
      ci_out => S02_outi_35,
      dr_out => S02_outr_51,
      di_out => S02_outi_51,
      clk => clk,
      reset => reset ) ;

 u7: Block16
    port map(
      in0r => S02_r_0,
      in0i => S02_i_0,
      in1r => S02_r_4,
      in1i => S02_i_4,
      in2r => S02_r_8,
      in2i => S02_i_8,
      in3r => S02_r_12,
      in3i => S02_i_12,
      ar_out => S02_outr_0,
      ai_out => S02_outi_0,
      br_out => S02_outr_16,
      bi_out => S02_outi_16,
      cr_out => S02_outr_32,
      ci_out => S02_outi_32,
      dr_out => S02_outr_48,
      di_out => S02_outi_48,
      clk => clk,
      reset => reset ) ;

 u4: Block20
    port map(
      in0r => S02_r_16,
      in0i => S02_i_16,
      in1r => S02_r_20,
      in1i => S02_i_20,
      in2r => S02_r_24,
      in2i => S02_i_24,
      in3r => S02_r_28,
      in3i => S02_i_28,
      ar_out => S02_outr_4,
      ai_out => S02_outi_4,
      br_out => S02_outr_20,
      bi_out => S02_outi_20,
      cr_out => S02_outr_36,
      ci_out => S02_outi_36,
      dr_out => S02_outr_52,
      di_out => S02_outi_52,
      clk => clk,
      reset => reset ) ;

 u5: Block21
    port map(
      in0r => S02_r_17,
      in0i => S02_i_17,
      in1r => S02_r_21,
      in1i => S02_i_21,
      in2r => S02_r_25,
      in2i => S02_i_25,
      in3r => S02_r_29,
      in3i => S02_i_29,
      ar_out => S02_outr_5,
      ai_out => S02_outi_5,
      br_out => S02_outr_21,
      bi_out => S02_outi_21,
      cr_out => S02_outr_37,
      ci_out => S02_outi_37,
      dr_out => S02_outr_53,
      di_out => S02_outi_53,
      clk => clk,
      reset => reset ) ;

 u0: Block22
    port map(
      in0r => S02_r_18,
      in0i => S02_i_18,
      in1r => S02_r_22,
      in1i => S02_i_22,
      in2r => S02_r_26,
      in2i => S02_i_26,
      in3r => S02_r_30,
      in3i => S02_i_30,
      ar_out => S02_outr_6,
      ai_out => S02_outi_6,
      br_out => S02_outr_22,
      bi_out => S02_outi_22,
      cr_out => S02_outr_38,
      ci_out => S02_outi_38,
      dr_out => S02_outr_54,
      di_out => S02_outi_54,
      clk => clk,
      reset => reset ) ;

 u6: Block23
    port map(
      in0r => S02_r_19,
      in0i => S02_i_19,
      in1r => S02_r_23,
      in1i => S02_i_23,
      in2r => S02_r_27,
      in2i => S02_i_27,
      in3r => S02_r_31,
      in3i => S02_i_31,
      ar_out => S02_outr_7,
      ai_out => S02_outi_7,
      br_out => S02_outr_23,
      bi_out => S02_outi_23,
      cr_out => S02_outr_39,
      ci_out => S02_outi_39,
      dr_out => S02_outr_55,
      di_out => S02_outi_55,
      clk => clk,
      reset => reset ) ;

 u8: Block24
    port map(
      in0r => S02_r_32,
      in0i => S02_i_32,
      in1r => S02_r_36,
      in1i => S02_i_36,
      in2r => S02_r_40,
      in2i => S02_i_40,
      in3r => S02_r_44,
      in3i => S02_i_44,
      ar_out => S02_outr_8,
      ai_out => S02_outi_8,
      br_out => S02_outr_24,
      bi_out => S02_outi_24,
      cr_out => S02_outr_40,
      ci_out => S02_outi_40,
      dr_out => S02_outr_56,
      di_out => S02_outi_56,
      clk => clk,
      reset => reset ) ;

 u9: Block25
    port map(
      in0r => S02_r_33,
      in0i => S02_i_33,
      in1r => S02_r_37,
      in1i => S02_i_37,
      in2r => S02_r_41,
      in2i => S02_i_41,
      in3r => S02_r_45,
      in3i => S02_i_45,
      ar_out => S02_outi_9,
      ai_out => S02_outr_9,
      br_out => S02_outi_25,
      bi_out => S02_outr_25,
      cr_out => S02_outi_41,
      ci_out => S02_outr_41,
      dr_out => S02_outr_57,
      di_out => S02_outi_57,
      clk => clk,
      reset => reset ) ;

 u10: Block26
    port map(
      in0r => S02_r_34,
      in0i => S02_i_34,
      in1r => S02_r_38,
      in1i => S02_i_38,
      in2r => S02_r_42,
      in2i => S02_i_42,
      in3r => S02_r_46,
      in3i => S02_i_46,
      ar_out => S02_outr_10,
      ai_out => S02_outi_10,
      br_out => S02_outr_26,
      bi_out => S02_outi_26,
      cr_out => S02_outr_42,
      ci_out => S02_outi_42,
      dr_out => S02_outr_58,
      di_out => S02_outi_58,
      clk => clk,
      reset => reset ) ;

 u11: Block27
    port map(
      in0r => S02_r_35,
      in0i => S02_i_35,
      in1r => S02_r_39,
      in1i => S02_i_39,
      in2r => S02_r_43,
      in2i => S02_i_43,
      in3r => S02_r_47,
      in3i => S02_i_47,
      ar_out => S02_outr_11,
      ai_out => S02_outi_11,
      br_out => S02_outr_27,
      bi_out => S02_outi_27,
      cr_out => S02_outr_43,
      ci_out => S02_outi_43,
      dr_out => S02_outr_59,
      di_out => S02_outi_59,
      clk => clk,
      reset => reset ) ;

 u12: Block28
    port map(
      in0r => S02_r_48,
      in0i => S02_i_48,
      in1r => S02_r_52,
      in1i => S02_i_52,
      in2r => S02_r_56,
      in2i => S02_i_56,
      in3r => S02_r_60,
      in3i => S02_i_60,
      ar_out => S02_outr_12,
      ai_out => S02_outi_12,
      br_out => S02_outr_28,
      bi_out => S02_outi_28,
      cr_out => S02_outr_44,
      ci_out => S02_outi_44,
      dr_out => S02_outr_60,
      di_out => S02_outi_60,
      clk => clk,
      reset => reset ) ;

 u13: Block29
    port map(
      in0r => S02_r_49,
      in0i => S02_i_49,
      in1r => S02_r_53,
      in1i => S02_i_53,
      in2r => S02_r_57,
      in2i => S02_i_57,
      in3r => S02_r_61,
      in3i => S02_i_61,
      ar_out => S02_outr_13,
      ai_out => S02_outi_13,
      br_out => S02_outr_29,
      bi_out => S02_outi_29,
      cr_out => S02_outr_45,
      ci_out => S02_outi_45,
      dr_out => S02_outr_61,
      di_out => S02_outi_61,
      clk => clk,
      reset => reset ) ;

 u14: Block30
    port map(
      in0r => S02_r_50,
      in0i => S02_i_50,
      in1r => S02_r_54,
      in1i => S02_i_54,
      in2r => S02_r_58,
      in2i => S02_i_58,
      in3r => S02_r_62,
      in3i => S02_i_62,
      ar_out => S02_outr_14,
      ai_out => S02_outi_14,
      br_out => S02_outr_30,
      bi_out => S02_outi_30,
      cr_out => S02_outr_46,
      ci_out => S02_outi_46,
      dr_out => S02_outr_62,
      di_out => S02_outi_62,
      clk => clk,
      reset => reset ) ;

 u15: Block31
    port map(
      in0r => S02_r_51,
      in0i => S02_i_51,
      in1r => S02_r_55,
      in1i => S02_i_55,
      in2r => S02_r_59,
      in2i => S02_i_59,
      in3r => S02_r_63,
      in3i => S02_i_63,
      ar_out => S02_outr_15,
      ai_out => S02_outi_15,
      br_out => S02_outr_31,
      bi_out => S02_outi_31,
      cr_out => S02_outr_47,
      ci_out => S02_outi_47,
      dr_out => S02_outr_63,
      di_out => S02_outi_63,
      clk => clk,
      reset => reset ) ;
end  a0 ; -- of Stage02

---------------------------------------------------------------------------
-- Entity declaration of 'Block15'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block15 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block15 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block15'
---------------------------------------------------------------------------

architecture a0 of Block15 is
 
constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block15

---------------------------------------------------------------------------
-- Entity declaration of 'Block3'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block3 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block3 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block3'
---------------------------------------------------------------------------

architecture a0 of Block3 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block3

---------------------------------------------------------------------------
-- Entity declaration of 'Block2'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block2 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block2 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block2'
---------------------------------------------------------------------------

architecture a0 of Block2 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block2

---------------------------------------------------------------------------
-- Entity declaration of 'Block1'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block1 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block1 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block1'
---------------------------------------------------------------------------

architecture a0 of Block1 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block1

---------------------------------------------------------------------------
-- Entity declaration of 'Block0'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block0 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block0 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block0'
---------------------------------------------------------------------------

architecture a0 of Block0 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block0

---------------------------------------------------------------------------
-- Entity declaration of 'Block14'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block14 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block14 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block14'
---------------------------------------------------------------------------

architecture a0 of Block14 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block14

---------------------------------------------------------------------------
-- Entity declaration of 'Block13'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block13 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block13 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block13'
---------------------------------------------------------------------------

architecture a0 of Block13 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block13

---------------------------------------------------------------------------
-- Entity declaration of 'Block12'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block12 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block12 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block12'
---------------------------------------------------------------------------

architecture a0 of Block12 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block12

---------------------------------------------------------------------------
-- Entity declaration of 'Block11'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block11 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block11 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block11'
---------------------------------------------------------------------------

architecture a0 of Block11 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block11

---------------------------------------------------------------------------
-- Entity declaration of 'Block10'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block10 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block10 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block10'
---------------------------------------------------------------------------

architecture a0 of Block10 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block10

---------------------------------------------------------------------------
-- Entity declaration of 'Block9'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block9 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block9 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block9'
---------------------------------------------------------------------------

architecture a0 of Block9 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block9

---------------------------------------------------------------------------
-- Entity declaration of 'Block8'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block8 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block8 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block8'
---------------------------------------------------------------------------

architecture a0 of Block8 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block8

---------------------------------------------------------------------------
-- Entity declaration of 'Block6'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block6 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block6 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block6'
---------------------------------------------------------------------------

architecture a0 of Block6 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block6

---------------------------------------------------------------------------
-- Entity declaration of 'Block7'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block7 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block7 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block7'
---------------------------------------------------------------------------

architecture a0 of Block7 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block7

---------------------------------------------------------------------------
-- Entity declaration of 'Block5'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block5 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block5 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block5'
---------------------------------------------------------------------------

architecture a0 of Block5 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block5

---------------------------------------------------------------------------
-- Entity declaration of 'Block4'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Block4 is
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
end  Block4 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Block4'
---------------------------------------------------------------------------

architecture a0 of Block4 is

constant cos_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_0j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant cos_1j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_1j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_2j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_2j  : std_logic_vector((N-1) downto 0) :="000000000000";
constant cos_3j  : std_logic_vector((N-1) downto 0) :="010000000000";
constant sin_3j  : std_logic_vector((N-1) downto 0) :="000000000000";  
constant round   : std_logic_vector((N+2) downto 0) :="000000000000001";

signal ar_temp   : std_logic_vector((2*N-1) downto 0);
signal ai_temp   : std_logic_vector((2*N-1) downto 0);
signal br_temp   : std_logic_vector((2*N-1) downto 0);
signal bi_temp   : std_logic_vector((2*N-1) downto 0);
signal cr_temp   : std_logic_vector((2*N-1) downto 0);
signal ci_temp   : std_logic_vector((2*N-1) downto 0);
signal dr_temp   : std_logic_vector((2*N-1) downto 0);
signal di_temp   : std_logic_vector((2*N-1) downto 0);

signal ar_round  : std_logic_vector((N+2) downto 0);
signal ai_round  : std_logic_vector((N+2) downto 0);
signal br_round  : std_logic_vector((N+2) downto 0);
signal bi_round  : std_logic_vector((N+2) downto 0);
signal cr_round  : std_logic_vector((N+2) downto 0);
signal ci_round  : std_logic_vector((N+2) downto 0);
signal dr_round  : std_logic_vector((N+2) downto 0);
signal di_round  : std_logic_vector((N+2) downto 0);

signal br_temp1  : std_logic_vector((2*N-1) downto 0);
signal bi_temp1  : std_logic_vector((2*N-1) downto 0);
signal cr_temp1  : std_logic_vector((2*N-1) downto 0);
signal ci_temp1  : std_logic_vector((2*N-1) downto 0);
signal dr_temp1  : std_logic_vector((2*N-1) downto 0);
signal di_temp1  : std_logic_vector((2*N-1) downto 0);

signal br_temp2  : std_logic_vector((2*N-1) downto 0);
signal bi_temp2  : std_logic_vector((2*N-1) downto 0);
signal cr_temp2  : std_logic_vector((2*N-1) downto 0);
signal ci_temp2  : std_logic_vector((2*N-1) downto 0);
signal dr_temp2  : std_logic_vector((2*N-1) downto 0);
signal di_temp2  : std_logic_vector((2*N-1) downto 0);

signal temp1r    : std_logic_vector((N+1) downto 0);
signal temp1i    : std_logic_vector((N+1) downto 0);
signal temp2r    : std_logic_vector((N+1) downto 0);
signal temp2i    : std_logic_vector((N+1) downto 0);

begin 

-------------------------------------Radix-4-------------------------------------
-- purpose : Radix-4 block
-- inputs  : clk, reset, in0r, in0i, in1r, in1i, in2r, in2i, in3r, in3i 
-- outputs : ar_out, ai_out, br_out, bi_out, cr_out, ci_out, dr_out, di_out
radix4 : process (clk, reset)
    
begin  -- process radix4
    if reset = '0' then
     ar_temp  <= (others => '0'); 
     ai_temp  <= (others => '0'); 	 
     br_temp  <= (others => '0'); 
     bi_temp  <= (others => '0'); 
     cr_temp  <= (others => '0'); 
     ci_temp  <= (others => '0'); 
     dr_temp  <= (others => '0'); 
     di_temp  <= (others => '0'); 
     ar_out   <= (others => '0');   
	 ai_out   <= (others => '0');   
     br_out   <= (others => '0');      
     bi_out   <= (others => '0');       
     cr_out   <= (others => '0');     
	 ci_out   <= (others => '0');     
	 dr_out   <= (others => '0');   
	 di_out   <= (others => '0');   
     temp1r   <= (others => '0');     
	 temp1i   <= (others => '0');     
	 temp2r   <= (others => '0');   
	 temp2i   <= (others => '0');
	 br_temp1 <= (others => '0'); 
     bi_temp1 <= (others => '0'); 
     cr_temp1 <= (others => '0'); 
     ci_temp1 <= (others => '0'); 
     dr_temp1 <= (others => '0'); 
     di_temp1 <= (others => '0');
	 br_temp2 <= (others => '0'); 
     bi_temp2 <= (others => '0'); 
     cr_temp2 <= (others => '0'); 
     ci_temp2 <= (others => '0'); 
     dr_temp2 <= (others => '0'); 
     di_temp2 <= (others => '0');
	 ar_round <= (others => '0'); 
     ai_round <= (others => '0'); 	 
     br_round <= (others => '0'); 
     bi_round <= (others => '0'); 
     cr_round <= (others => '0'); 
     ci_round <= (others => '0'); 
     dr_round <= (others => '0'); 
     di_round <= (others => '0'); 
	
    elsif clk'event and clk = '1' then
	 ar_temp  <= in0r*cos_0j;
	 ai_temp  <= in0i*sin_0j;   
	 br_temp1 <= in1r*cos_1j;
	 br_temp2 <= in1i*sin_1j;
	 bi_temp1 <= in1i*cos_1j;
	 bi_temp2 <= in1r*sin_1j;
	 cr_temp1 <= in2r*cos_2j;
	 cr_temp2 <= in2i*sin_2j;
	 ci_temp1 <= in2i*cos_2j;
	 ci_temp2 <= in2r*sin_2j;
	 dr_temp1 <= in3r*cos_3j;
	 dr_temp2 <= in3i*sin_3j;	 
	 di_temp1 <= in3i*cos_3j;
	 di_temp2 <= in3r*sin_3j;

	 br_temp  <= br_temp1 + br_temp2;
	 bi_temp  <= bi_temp1 - bi_temp2;
     cr_temp  <= cr_temp1 + cr_temp2;
     ci_temp  <= ci_temp1 - ci_temp2;
     dr_temp  <= dr_temp1 + dr_temp2;
     di_temp  <= di_temp1 - di_temp2;
	     
	 ar_round <= ar_temp((N*2-1) downto (N-3)) + round;		
	 ai_round <= ai_temp((N*2-1) downto (N-3)) + round;
	 br_round <= br_temp((N*2-1) downto (N-3)) + round;
	 bi_round <= bi_temp((N*2-1) downto (N-3)) + round;
	 cr_round <= cr_temp((N*2-1) downto (N-3)) + round;
	 ci_round <= ci_temp((N*2-1) downto (N-3)) + round;
	 dr_round <= dr_temp((N*2-1) downto (N-3)) + round;
	 di_round <= di_temp((N*2-1) downto (N-3)) + round;	 

	 temp1r   <= ar_round((N+2) downto 1) + cr_round((N+2) downto 1);
	 temp1i   <= ai_round((N+2) downto 1) + ci_round((N+2) downto 1);
	 temp2r   <= ar_round((N+2) downto 1) - cr_round((N+2) downto 1);
	 temp2i   <= ai_round((N+2) downto 1) - ci_round((N+2) downto 1);

	 ar_out   <= temp1r + br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 ai_out   <= temp1i + bi_round((N+2) downto 1) + di_round((N+2) downto 1);	
	 br_out   <= temp2r + bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 bi_out   <= temp2i - br_round((N+2) downto 1) + dr_round((N+2) downto 1);
	 cr_out   <= temp1r - br_round((N+2) downto 1) - dr_round((N+2) downto 1);
	 ci_out   <= temp1i - bi_round((N+2) downto 1) - di_round((N+2) downto 1);
	 dr_out   <= temp2r - bi_round((N+2) downto 1) + di_round((N+2) downto 1);
	 di_out   <= temp2i + br_round((N+2) downto 1) - dr_round((N+2) downto 1);	 
    end if;
end process radix4;
end  a0 ; -- of Block4

---------------------------------------------------------------------------
-- Entity declaration of 'Stage01'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Stage01 is
  generic(
    N : Integer := 12 ) ;
  port(
    S01_r_0     : in     std_logic_vector(11 downto 0) ;
    S01_r_1     : in     std_logic_vector(11 downto 0) ;
    S01_r_2     : in     std_logic_vector(11 downto 0) ;
    S01_r_3     : in     std_logic_vector(11 downto 0) ;
    S01_r_4     : in     std_logic_vector(11 downto 0) ;
    S01_r_5     : in     std_logic_vector(11 downto 0) ;
    S01_r_6     : in     std_logic_vector(11 downto 0) ;
    S01_r_7     : in     std_logic_vector(11 downto 0) ;
    S01_r_8     : in     std_logic_vector(11 downto 0) ;
    S01_r_9     : in     std_logic_vector(11 downto 0) ;
    S01_r_10    : in     std_logic_vector(11 downto 0) ;
    S01_r_11    : in     std_logic_vector(11 downto 0) ;
    S01_r_12    : in     std_logic_vector(11 downto 0) ;
    S01_r_13    : in     std_logic_vector(11 downto 0) ;
    S01_r_14    : in     std_logic_vector(11 downto 0) ;
    S01_r_15    : in     std_logic_vector(11 downto 0) ;
    S01_r_16    : in     std_logic_vector(11 downto 0) ;
    S01_r_17    : in     std_logic_vector(11 downto 0) ;
    S01_r_18    : in     std_logic_vector(11 downto 0) ;
    S01_r_19    : in     std_logic_vector(11 downto 0) ;
    S01_r_20    : in     std_logic_vector(11 downto 0) ;
    S01_r_21    : in     std_logic_vector(11 downto 0) ;
    S01_r_22    : in     std_logic_vector(11 downto 0) ;
    S01_r_23    : in     std_logic_vector(11 downto 0) ;
    S01_r_24    : in     std_logic_vector(11 downto 0) ;
    S01_r_25    : in     std_logic_vector(11 downto 0) ;
    S01_r_26    : in     std_logic_vector(11 downto 0) ;
    S01_r_27    : in     std_logic_vector(11 downto 0) ;
    S01_r_28    : in     std_logic_vector(11 downto 0) ;
    S01_r_29    : in     std_logic_vector(11 downto 0) ;
    S01_r_30    : in     std_logic_vector(11 downto 0) ;
    S01_r_31    : in     std_logic_vector(11 downto 0) ;
    S01_r_32    : in     std_logic_vector(11 downto 0) ;
    S01_r_33    : in     std_logic_vector(11 downto 0) ;
    S01_r_34    : in     std_logic_vector(11 downto 0) ;
    S01_r_35    : in     std_logic_vector(11 downto 0) ;
    S01_r_36    : in     std_logic_vector(11 downto 0) ;
    S01_r_37    : in     std_logic_vector(11 downto 0) ;
    S01_r_38    : in     std_logic_vector(11 downto 0) ;
    S01_r_39    : in     std_logic_vector(11 downto 0) ;
    S01_r_40    : in     std_logic_vector(11 downto 0) ;
    S01_r_41    : in     std_logic_vector(11 downto 0) ;
    S01_r_42    : in     std_logic_vector(11 downto 0) ;
    S01_r_43    : in     std_logic_vector(11 downto 0) ;
    S01_r_44    : in     std_logic_vector(11 downto 0) ;
    S01_r_45    : in     std_logic_vector(11 downto 0) ;
    S01_r_46    : in     std_logic_vector(11 downto 0) ;
    S01_r_47    : in     std_logic_vector(11 downto 0) ;
    S01_r_48    : in     std_logic_vector(11 downto 0) ;
    S01_r_49    : in     std_logic_vector(11 downto 0) ;
    S01_r_50    : in     std_logic_vector(11 downto 0) ;
    S01_r_51    : in     std_logic_vector(11 downto 0) ;
    S01_r_52    : in     std_logic_vector(11 downto 0) ;
    S01_r_53    : in     std_logic_vector(11 downto 0) ;
    S01_r_54    : in     std_logic_vector(11 downto 0) ;
    S01_r_55    : in     std_logic_vector(11 downto 0) ;
    S01_r_56    : in     std_logic_vector(11 downto 0) ;
    S01_r_57    : in     std_logic_vector(11 downto 0) ;
    S01_r_58    : in     std_logic_vector(11 downto 0) ;
    S01_r_59    : in     std_logic_vector(11 downto 0) ;
    S01_r_60    : in     std_logic_vector(11 downto 0) ;
    S01_r_61    : in     std_logic_vector(11 downto 0) ;
    S01_r_62    : in     std_logic_vector(11 downto 0) ;
    S01_r_63    : in     std_logic_vector(11 downto 0) ;
    S01_i_0     : in     std_logic_vector(11 downto 0) ;
    S01_i_1     : in     std_logic_vector(11 downto 0) ;
    S01_i_2     : in     std_logic_vector(11 downto 0) ;
    S01_i_3     : in     std_logic_vector(11 downto 0) ;
    S01_i_4     : in     std_logic_vector(11 downto 0) ;
    S01_i_5     : in     std_logic_vector(11 downto 0) ;
    S01_i_6     : in     std_logic_vector(11 downto 0) ;
    S01_i_7     : in     std_logic_vector(11 downto 0) ;
    S01_i_8     : in     std_logic_vector(11 downto 0) ;
    S01_i_9     : in     std_logic_vector(11 downto 0) ;
    S01_i_10    : in     std_logic_vector(11 downto 0) ;
    S01_i_11    : in     std_logic_vector(11 downto 0) ;
    S01_i_12    : in     std_logic_vector(11 downto 0) ;
    S01_i_13    : in     std_logic_vector(11 downto 0) ;
    S01_i_14    : in     std_logic_vector(11 downto 0) ;
    S01_i_15    : in     std_logic_vector(11 downto 0) ;
    S01_i_16    : in     std_logic_vector(11 downto 0) ;
    S01_i_17    : in     std_logic_vector(11 downto 0) ;
    S01_i_18    : in     std_logic_vector(11 downto 0) ;
    S01_i_19    : in     std_logic_vector(11 downto 0) ;
    S01_i_20    : in     std_logic_vector(11 downto 0) ;
    S01_i_21    : in     std_logic_vector(11 downto 0) ;
    S01_i_22    : in     std_logic_vector(11 downto 0) ;
    S01_i_23    : in     std_logic_vector(11 downto 0) ;
    S01_i_24    : in     std_logic_vector(11 downto 0) ;
    S01_i_25    : in     std_logic_vector(11 downto 0) ;
    S01_i_26    : in     std_logic_vector(11 downto 0) ;
    S01_i_27    : in     std_logic_vector(11 downto 0) ;
    S01_i_28    : in     std_logic_vector(11 downto 0) ;
    S01_i_29    : in     std_logic_vector(11 downto 0) ;
    S01_i_30    : in     std_logic_vector(11 downto 0) ;
    S01_i_31    : in     std_logic_vector(11 downto 0) ;
    S01_i_32    : in     std_logic_vector(11 downto 0) ;
    S01_i_33    : in     std_logic_vector(11 downto 0) ;
    S01_i_34    : in     std_logic_vector(11 downto 0) ;
    S01_i_35    : in     std_logic_vector(11 downto 0) ;
    S01_i_36    : in     std_logic_vector(11 downto 0) ;
    S01_i_37    : in     std_logic_vector(11 downto 0) ;
    S01_i_38    : in     std_logic_vector(11 downto 0) ;
    S01_i_39    : in     std_logic_vector(11 downto 0) ;
    S01_i_40    : in     std_logic_vector(11 downto 0) ;
    S01_i_41    : in     std_logic_vector(11 downto 0) ;
    S01_i_42    : in     std_logic_vector(11 downto 0) ;
    S01_i_43    : in     std_logic_vector(11 downto 0) ;
    S01_i_44    : in     std_logic_vector(11 downto 0) ;
    S01_i_45    : in     std_logic_vector(11 downto 0) ;
    S01_i_46    : in     std_logic_vector(11 downto 0) ;
    S01_i_47    : in     std_logic_vector(11 downto 0) ;
    S01_i_48    : in     std_logic_vector(11 downto 0) ;
    S01_i_49    : in     std_logic_vector(11 downto 0) ;
    S01_i_50    : in     std_logic_vector(11 downto 0) ;
    S01_i_51    : in     std_logic_vector(11 downto 0) ;
    S01_i_52    : in     std_logic_vector(11 downto 0) ;
    S01_i_53    : in     std_logic_vector(11 downto 0) ;
    S01_i_54    : in     std_logic_vector(11 downto 0) ;
    S01_i_55    : in     std_logic_vector(11 downto 0) ;
    S01_i_56    : in     std_logic_vector(11 downto 0) ;
    S01_i_57    : in     std_logic_vector(11 downto 0) ;
    S01_i_58    : in     std_logic_vector(11 downto 0) ;
    S01_i_59    : in     std_logic_vector(11 downto 0) ;
    S01_i_60    : in     std_logic_vector(11 downto 0) ;
    S01_i_61    : in     std_logic_vector(11 downto 0) ;
    S01_i_62    : in     std_logic_vector(11 downto 0) ;
    S01_i_63    : in     std_logic_vector(11 downto 0) ;
    S01_outr_0  : out    std_logic_vector(13 downto 0) ;
    S01_outr_1  : out    std_logic_vector(13 downto 0) ;
    S01_outr_2  : out    std_logic_vector(13 downto 0) ;
    S01_outr_3  : out    std_logic_vector(13 downto 0) ;
    S01_outr_4  : out    std_logic_vector(13 downto 0) ;
    S01_outr_5  : out    std_logic_vector(13 downto 0) ;
    S01_outr_6  : out    std_logic_vector(13 downto 0) ;
    S01_outr_7  : out    std_logic_vector(13 downto 0) ;
    S01_outr_8  : out    std_logic_vector(13 downto 0) ;
    S01_outr_9  : out    std_logic_vector(13 downto 0) ;
    S01_outr_10 : out    std_logic_vector(13 downto 0) ;
    S01_outr_11 : out    std_logic_vector(13 downto 0) ;
    S01_outr_12 : out    std_logic_vector(13 downto 0) ;
    S01_outr_13 : out    std_logic_vector(13 downto 0) ;
    S01_outr_14 : out    std_logic_vector(13 downto 0) ;
    S01_outr_15 : out    std_logic_vector(13 downto 0) ;
    S01_outr_16 : out    std_logic_vector(13 downto 0) ;
    S01_outr_17 : out    std_logic_vector(13 downto 0) ;
    S01_outr_18 : out    std_logic_vector(13 downto 0) ;
    S01_outr_19 : out    std_logic_vector(13 downto 0) ;
    S01_outr_20 : out    std_logic_vector(13 downto 0) ;
    S01_outr_21 : out    std_logic_vector(13 downto 0) ;
    S01_outr_22 : out    std_logic_vector(13 downto 0) ;
    S01_outr_23 : out    std_logic_vector(13 downto 0) ;
    S01_outr_24 : out    std_logic_vector(13 downto 0) ;
    S01_outr_25 : out    std_logic_vector(13 downto 0) ;
    S01_outr_26 : out    std_logic_vector(13 downto 0) ;
    S01_outr_27 : out    std_logic_vector(13 downto 0) ;
    S01_outr_28 : out    std_logic_vector(13 downto 0) ;
    S01_outr_29 : out    std_logic_vector(13 downto 0) ;
    S01_outr_30 : out    std_logic_vector(13 downto 0) ;
    S01_outr_31 : out    std_logic_vector(13 downto 0) ;
    S01_outr_32 : out    std_logic_vector(13 downto 0) ;
    S01_outr_33 : out    std_logic_vector(13 downto 0) ;
    S01_outr_34 : out    std_logic_vector(13 downto 0) ;
    S01_outr_35 : out    std_logic_vector(13 downto 0) ;
    S01_outr_36 : out    std_logic_vector(13 downto 0) ;
    S01_outr_37 : out    std_logic_vector(13 downto 0) ;
    S01_outr_38 : out    std_logic_vector(13 downto 0) ;
    S01_outr_39 : out    std_logic_vector(13 downto 0) ;
    S01_outr_40 : out    std_logic_vector(13 downto 0) ;
    S01_outr_41 : out    std_logic_vector(13 downto 0) ;
    S01_outr_42 : out    std_logic_vector(13 downto 0) ;
    S01_outr_43 : out    std_logic_vector(13 downto 0) ;
    S01_outr_44 : out    std_logic_vector(13 downto 0) ;
    S01_outr_45 : out    std_logic_vector(13 downto 0) ;
    S01_outr_46 : out    std_logic_vector(13 downto 0) ;
    S01_outr_47 : out    std_logic_vector(13 downto 0) ;
    S01_outr_48 : out    std_logic_vector(13 downto 0) ;
    S01_outr_49 : out    std_logic_vector(13 downto 0) ;
    S01_outr_50 : out    std_logic_vector(13 downto 0) ;
    S01_outr_51 : out    std_logic_vector(13 downto 0) ;
    S01_outr_52 : out    std_logic_vector(13 downto 0) ;
    S01_outr_53 : out    std_logic_vector(13 downto 0) ;
    S01_outr_54 : out    std_logic_vector(13 downto 0) ;
    S01_outr_55 : out    std_logic_vector(13 downto 0) ;
    S01_outr_56 : out    std_logic_vector(13 downto 0) ;
    S01_outr_57 : out    std_logic_vector(13 downto 0) ;
    S01_outr_58 : out    std_logic_vector(13 downto 0) ;
    S01_outr_59 : out    std_logic_vector(13 downto 0) ;
    S01_outr_60 : out    std_logic_vector(13 downto 0) ;
    S01_outr_61 : out    std_logic_vector(13 downto 0) ;
    S01_outr_62 : out    std_logic_vector(13 downto 0) ;
    S01_outr_63 : out    std_logic_vector(13 downto 0) ;
    S01_outi_0  : out    std_logic_vector(13 downto 0) ;
    S01_outi_1  : out    std_logic_vector(13 downto 0) ;
    S01_outi_2  : out    std_logic_vector(13 downto 0) ;
    S01_outi_3  : out    std_logic_vector(13 downto 0) ;
    S01_outi_4  : out    std_logic_vector(13 downto 0) ;
    S01_outi_5  : out    std_logic_vector(13 downto 0) ;
    S01_outi_6  : out    std_logic_vector(13 downto 0) ;
    S01_outi_7  : out    std_logic_vector(13 downto 0) ;
    S01_outi_8  : out    std_logic_vector(13 downto 0) ;
    S01_outi_9  : out    std_logic_vector(13 downto 0) ;
    S01_outi_10 : out    std_logic_vector(13 downto 0) ;
    S01_outi_11 : out    std_logic_vector(13 downto 0) ;
    S01_outi_12 : out    std_logic_vector(13 downto 0) ;
    S01_outi_13 : out    std_logic_vector(13 downto 0) ;
    S01_outi_14 : out    std_logic_vector(13 downto 0) ;
    S01_outi_15 : out    std_logic_vector(13 downto 0) ;
    S01_outi_16 : out    std_logic_vector(13 downto 0) ;
    S01_outi_17 : out    std_logic_vector(13 downto 0) ;
    S01_outi_18 : out    std_logic_vector(13 downto 0) ;
    S01_outi_19 : out    std_logic_vector(13 downto 0) ;
    S01_outi_20 : out    std_logic_vector(13 downto 0) ;
    S01_outi_21 : out    std_logic_vector(13 downto 0) ;
    S01_outi_22 : out    std_logic_vector(13 downto 0) ;
    S01_outi_23 : out    std_logic_vector(13 downto 0) ;
    S01_outi_24 : out    std_logic_vector(13 downto 0) ;
    S01_outi_25 : out    std_logic_vector(13 downto 0) ;
    S01_outi_26 : out    std_logic_vector(13 downto 0) ;
    S01_outi_27 : out    std_logic_vector(13 downto 0) ;
    S01_outi_28 : out    std_logic_vector(13 downto 0) ;
    S01_outi_29 : out    std_logic_vector(13 downto 0) ;
    S01_outi_30 : out    std_logic_vector(13 downto 0) ;
    S01_outi_31 : out    std_logic_vector(13 downto 0) ;
    S01_outi_32 : out    std_logic_vector(13 downto 0) ;
    S01_outi_33 : out    std_logic_vector(13 downto 0) ;
    S01_outi_34 : out    std_logic_vector(13 downto 0) ;
    S01_outi_35 : out    std_logic_vector(13 downto 0) ;
    S01_outi_36 : out    std_logic_vector(13 downto 0) ;
    S01_outi_37 : out    std_logic_vector(13 downto 0) ;
    S01_outi_38 : out    std_logic_vector(13 downto 0) ;
    S01_outi_39 : out    std_logic_vector(13 downto 0) ;
    S01_outi_40 : out    std_logic_vector(13 downto 0) ;
    S01_outi_41 : out    std_logic_vector(13 downto 0) ;
    S01_outi_42 : out    std_logic_vector(13 downto 0) ;
    S01_outi_43 : out    std_logic_vector(13 downto 0) ;
    S01_outi_44 : out    std_logic_vector(13 downto 0) ;
    S01_outi_45 : out    std_logic_vector(13 downto 0) ;
    S01_outi_46 : out    std_logic_vector(13 downto 0) ;
    S01_outi_47 : out    std_logic_vector(13 downto 0) ;
    S01_outi_48 : out    std_logic_vector(13 downto 0) ;
    S01_outi_49 : out    std_logic_vector(13 downto 0) ;
    S01_outi_50 : out    std_logic_vector(13 downto 0) ;
    S01_outi_51 : out    std_logic_vector(13 downto 0) ;
    S01_outi_52 : out    std_logic_vector(13 downto 0) ;
    S01_outi_53 : out    std_logic_vector(13 downto 0) ;
    S01_outi_54 : out    std_logic_vector(13 downto 0) ;
    S01_outi_55 : out    std_logic_vector(13 downto 0) ;
    S01_outi_56 : out    std_logic_vector(13 downto 0) ;
    S01_outi_58 : out    std_logic_vector(13 downto 0) ;
    S01_outi_59 : out    std_logic_vector(13 downto 0) ;
    S01_outi_57 : out    std_logic_vector(13 downto 0) ;
    S01_outi_60 : out    std_logic_vector(13 downto 0) ;
    S01_outi_61 : out    std_logic_vector(13 downto 0) ;
    S01_outi_62 : out    std_logic_vector(13 downto 0) ;
    S01_outi_63 : out    std_logic_vector(13 downto 0) ;
    clk         : in     std_logic ;
    reset       : in     std_logic ) ;
end  Stage01 ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Stage01'
---------------------------------------------------------------------------

architecture a0 of Stage01 is


 component Block4
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block5
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block7
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block6
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block8
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block9
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block10
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block11
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block12
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block13
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block14
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block0
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block1
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block2
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block3
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;
 component Block15
  generic(
    N : Integer := 12 ) ;
  port(
    in0r   : in     std_logic_vector(11 downto 0) ;
    in0i   : in     std_logic_vector(11 downto 0) ;
    in1r   : in     std_logic_vector(11 downto 0) ;
    in1i   : in     std_logic_vector(11 downto 0) ;
    in2r   : in     std_logic_vector(11 downto 0) ;
    in2i   : in     std_logic_vector(11 downto 0) ;
    in3r   : in     std_logic_vector(11 downto 0) ;
    in3i   : in     std_logic_vector(11 downto 0) ;
    ar_out : out    std_logic_vector(13 downto 0) ;
    ai_out : out    std_logic_vector(13 downto 0) ;
    br_out : out    std_logic_vector(13 downto 0) ;
    bi_out : out    std_logic_vector(13 downto 0) ;
    cr_out : out    std_logic_vector(13 downto 0) ;
    ci_out : out    std_logic_vector(13 downto 0) ;
    dr_out : out    std_logic_vector(13 downto 0) ;
    di_out : out    std_logic_vector(13 downto 0) ;
    clk    : in     std_logic ;
    reset  : in     std_logic ) ;
 end component ;

begin

 u4: Block4
    port map(
      in0r => S01_r_4,
      in0i => S01_i_4,
      in1r => S01_r_20,
      in1i => S01_i_20,
      in2r => S01_r_36,
      in2i => S01_i_36,
      in3r => S01_r_52,
      in3i => S01_i_52,
      ar_out => S01_outr_4,
      ai_out => S01_outi_4,
      br_out => S01_outr_20,
      bi_out => S01_outi_20,
      cr_out => S01_outr_36,
      ci_out => S01_outi_36,
      dr_out => S01_outr_52,
      di_out => S01_outi_52,
      clk => clk,
      reset => reset ) ;

 u5: Block5
    port map(
      in0r => S01_r_5,
      in0i => S01_i_5,
      in1r => S01_r_21,
      in1i => S01_i_21,
      in2r => S01_r_37,
      in2i => S01_i_37,
      in3r => S01_r_53,
      in3i => S01_i_53,
      ar_out => S01_outr_5,
      ai_out => S01_outi_5,
      br_out => S01_outr_21,
      bi_out => S01_outi_21,
      cr_out => S01_outr_37,
      ci_out => S01_outi_37,
      dr_out => S01_outr_53,
      di_out => S01_outi_53,
      clk => clk,
      reset => reset ) ;

 u6: Block7
    port map(
      in0r => S01_r_7,
      in0i => S01_i_7,
      in1r => S01_r_23,
      in1i => S01_i_23,
      in2r => S01_r_39,
      in2i => S01_i_39,
      in3r => S01_r_55,
      in3i => S01_i_55,
      ar_out => S01_outr_7,
      ai_out => S01_outi_7,
      br_out => S01_outr_23,
      bi_out => S01_outi_23,
      cr_out => S01_outr_39,
      ci_out => S01_outi_39,
      dr_out => S01_outr_55,
      di_out => S01_outi_55,
      clk => clk,
      reset => reset ) ;

 u0: Block6
    port map(
      in0r => S01_r_6,
      in0i => S01_i_6,
      in1r => S01_r_22,
      in1i => S01_i_22,
      in2r => S01_r_38,
      in2i => S01_i_38,
      in3r => S01_r_54,
      in3i => S01_i_54,
      ar_out => S01_outr_6,
      ai_out => S01_outi_6,
      br_out => S01_outr_22,
      bi_out => S01_outi_22,
      cr_out => S01_outr_38,
      ci_out => S01_outi_38,
      dr_out => S01_outr_54,
      di_out => S01_outi_54,
      clk => clk,
      reset => reset ) ;

 u8: Block8
    port map(
      in0r => S01_r_8,
      in0i => S01_i_8,
      in1r => S01_r_24,
      in1i => S01_i_24,
      in2r => S01_r_40,
      in2i => S01_i_40,
      in3r => S01_r_56,
      in3i => S01_i_56,
      ar_out => S01_outr_8,
      ai_out => S01_outi_8,
      br_out => S01_outr_24,
      bi_out => S01_outi_24,
      cr_out => S01_outr_40,
      ci_out => S01_outi_40,
      dr_out => S01_outr_56,
      di_out => S01_outi_56,
      clk => clk,
      reset => reset ) ;

 u9: Block9
    port map(
      in0r => S01_r_9,
      in0i => S01_i_9,
      in1r => S01_r_25,
      in1i => S01_i_25,
      in2r => S01_r_41,
      in2i => S01_i_41,
      in3r => S01_r_57,
      in3i => S01_i_57,
      ar_out => S01_outr_9,
      ai_out => S01_outi_9,
      br_out => S01_outr_25,
      bi_out => S01_outi_25,
      cr_out => S01_outr_41,
      ci_out => S01_outi_41,
      dr_out => S01_outr_57,
      di_out => S01_outi_58,
      clk => clk,
      reset => reset ) ;

 u10: Block10
    port map(
      in0r => S01_r_10,
      in0i => S01_i_10,
      in1r => S01_r_26,
      in1i => S01_i_26,
      in2r => S01_r_42,
      in2i => S01_i_42,
      in3r => S01_r_58,
      in3i => S01_i_58,
      ar_out => S01_outr_10,
      ai_out => S01_outi_10,
      br_out => S01_outr_26,
      bi_out => S01_outi_26,
      cr_out => S01_outr_42,
      ci_out => S01_outi_42,
      dr_out => S01_outr_58,
      di_out => S01_outi_59,
      clk => clk,
      reset => reset ) ;

 u11: Block11
    port map(
      in0r => S01_r_11,
      in0i => S01_i_11,
      in1r => S01_r_27,
      in1i => S01_i_27,
      in2r => S01_r_43,
      in2i => S01_i_43,
      in3r => S01_r_59,
      in3i => S01_i_59,
      ar_out => S01_outr_11,
      ai_out => S01_outi_11,
      br_out => S01_outr_27,
      bi_out => S01_outi_27,
      cr_out => S01_outr_43,
      ci_out => S01_outi_43,
      dr_out => S01_outr_59,
      di_out => S01_outi_57,
      clk => clk,
      reset => reset ) ;

 u12: Block12
    port map(
      in0r => S01_r_12,
      in0i => S01_i_12,
      in1r => S01_r_28,
      in1i => S01_i_28,
      in2r => S01_r_44,
      in2i => S01_i_44,
      in3r => S01_r_60,
      in3i => S01_i_60,
      ar_out => S01_outr_12,
      ai_out => S01_outi_12,
      br_out => S01_outr_28,
      bi_out => S01_outi_28,
      cr_out => S01_outr_44,
      ci_out => S01_outi_44,
      dr_out => S01_outr_60,
      di_out => S01_outi_60,
      clk => clk,
      reset => reset ) ;

 u13: Block13
    port map(
      in0r => S01_r_13,
      in0i => S01_i_13,
      in1r => S01_r_29,
      in1i => S01_i_29,
      in2r => S01_r_45,
      in2i => S01_i_45,
      in3r => S01_r_61,
      in3i => S01_i_61,
      ar_out => S01_outr_13,
      ai_out => S01_outi_13,
      br_out => S01_outr_29,
      bi_out => S01_outi_29,
      cr_out => S01_outr_45,
      ci_out => S01_outi_45,
      dr_out => S01_outr_61,
      di_out => S01_outi_61,
      clk => clk,
      reset => reset ) ;

 u14: Block14
    port map(
      in0r => S01_r_14,
      in0i => S01_i_14,
      in1r => S01_r_30,
      in1i => S01_i_30,
      in2r => S01_r_46,
      in2i => S01_i_46,
      in3r => S01_r_62,
      in3i => S01_i_62,
      ar_out => S01_outr_14,
      ai_out => S01_outi_14,
      br_out => S01_outr_30,
      bi_out => S01_outi_30,
      cr_out => S01_outr_46,
      ci_out => S01_outi_46,
      dr_out => S01_outr_62,
      di_out => S01_outi_62,
      clk => clk,
      reset => reset ) ;

 u7: Block0
    port map(
      in0r => S01_r_0,
      in0i => S01_i_0,
      in1r => S01_r_16,
      in1i => S01_i_16,
      in2r => S01_r_32,
      in2i => S01_i_32,
      in3r => S01_r_48,
      in3i => S01_i_48,
      ar_out => S01_outr_0,
      ai_out => S01_outi_0,
      br_out => S01_outr_16,
      bi_out => S01_outi_16,
      cr_out => S01_outr_32,
      ci_out => S01_outi_32,
      dr_out => S01_outr_48,
      di_out => S01_outi_48,
      clk => clk,
      reset => reset ) ;

 u1: Block1
    port map(
      in0r => S01_r_1,
      in0i => S01_i_1,
      in1r => S01_r_17,
      in1i => S01_i_17,
      in2r => S01_r_33,
      in2i => S01_i_33,
      in3r => S01_r_49,
      in3i => S01_i_49,
      ar_out => S01_outr_1,
      ai_out => S01_outi_1,
      br_out => S01_outr_17,
      bi_out => S01_outi_17,
      cr_out => S01_outr_33,
      ci_out => S01_outi_33,
      dr_out => S01_outr_49,
      di_out => S01_outi_49,
      clk => clk,
      reset => reset ) ;

 u2: Block2
    port map(
      in0r => S01_r_2,
      in0i => S01_i_2,
      in1r => S01_r_18,
      in1i => S01_i_18,
      in2r => S01_r_34,
      in2i => S01_i_34,
      in3r => S01_r_50,
      in3i => S01_i_50,
      ar_out => S01_outr_2,
      ai_out => S01_outi_2,
      br_out => S01_outr_18,
      bi_out => S01_outi_18,
      cr_out => S01_outr_34,
      ci_out => S01_outi_34,
      dr_out => S01_outr_50,
      di_out => S01_outi_50,
      clk => clk,
      reset => reset ) ;

 u3: Block3
    port map(
      in0r => S01_r_3,
      in0i => S01_i_3,
      in1r => S01_r_19,
      in1i => S01_i_19,
      in2r => S01_r_35,
      in2i => S01_i_35,
      in3r => S01_r_51,
      in3i => S01_i_51,
      ar_out => S01_outr_3,
      ai_out => S01_outi_3,
      br_out => S01_outr_19,
      bi_out => S01_outi_19,
      cr_out => S01_outr_35,
      ci_out => S01_outi_35,
      dr_out => S01_outr_51,
      di_out => S01_outi_51,
      clk => clk,
      reset => reset ) ;

 u15: Block15
    port map(
      in0r => S01_r_15,
      in0i => S01_i_15,
      in1r => S01_r_31,
      in1i => S01_i_31,
      in2r => S01_r_47,
      in2i => S01_i_47,
      in3r => S01_r_63,
      in3i => S01_i_63,
      ar_out => S01_outr_15,
      ai_out => S01_outi_15,
      br_out => S01_outr_31,
      bi_out => S01_outi_31,
      cr_out => S01_outr_47,
      ci_out => S01_outi_47,
      dr_out => S01_outr_63,
      di_out => S01_outi_63,
      clk => clk,
      reset => reset ) ;
end  a0 ; -- of Stage01

---------------------------------------------------------------------------
-- Entity declaration of 'Top_Level'.
---------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_signed.all ;
use ieee.std_logic_1164.all ;

entity Top_Level is
  generic(
    N : Integer := 12 ) ;
  port(
    clk            : in     std_logic ;
    reset          : in     std_logic ;
    Shift_in_r     : in     std_logic_vector(11 downto 0) ;
    Shift_in_i     : in     std_logic_vector(11 downto 0) ;
    Shift_out_r    : out    std_logic_vector(17 downto 0) ;
    Shift_out_i    : out    std_logic_vector(17 downto 0) ;
    Data_valid_out : out    std_logic ;
    Data_valid_in  : in     std_logic ) ;
end  Top_Level ;

---------------------------------------------------------------------------
-- Architecture 'a0' of 'Top_Level'
---------------------------------------------------------------------------

architecture a0 of Top_Level is


 component Stage01
  generic(
    N : Integer := 12 ) ;
  port(
    S01_r_0     : in     std_logic_vector(11 downto 0) ;
    S01_r_1     : in     std_logic_vector(11 downto 0) ;
    S01_r_2     : in     std_logic_vector(11 downto 0) ;
    S01_r_3     : in     std_logic_vector(11 downto 0) ;
    S01_r_4     : in     std_logic_vector(11 downto 0) ;
    S01_r_5     : in     std_logic_vector(11 downto 0) ;
    S01_r_6     : in     std_logic_vector(11 downto 0) ;
    S01_r_7     : in     std_logic_vector(11 downto 0) ;
    S01_r_8     : in     std_logic_vector(11 downto 0) ;
    S01_r_9     : in     std_logic_vector(11 downto 0) ;
    S01_r_10    : in     std_logic_vector(11 downto 0) ;
    S01_r_11    : in     std_logic_vector(11 downto 0) ;
    S01_r_12    : in     std_logic_vector(11 downto 0) ;
    S01_r_13    : in     std_logic_vector(11 downto 0) ;
    S01_r_14    : in     std_logic_vector(11 downto 0) ;
    S01_r_15    : in     std_logic_vector(11 downto 0) ;
    S01_r_16    : in     std_logic_vector(11 downto 0) ;
    S01_r_17    : in     std_logic_vector(11 downto 0) ;
    S01_r_18    : in     std_logic_vector(11 downto 0) ;
    S01_r_19    : in     std_logic_vector(11 downto 0) ;
    S01_r_20    : in     std_logic_vector(11 downto 0) ;
    S01_r_21    : in     std_logic_vector(11 downto 0) ;
    S01_r_22    : in     std_logic_vector(11 downto 0) ;
    S01_r_23    : in     std_logic_vector(11 downto 0) ;
    S01_r_24    : in     std_logic_vector(11 downto 0) ;
    S01_r_25    : in     std_logic_vector(11 downto 0) ;
    S01_r_26    : in     std_logic_vector(11 downto 0) ;
    S01_r_27    : in     std_logic_vector(11 downto 0) ;
    S01_r_28    : in     std_logic_vector(11 downto 0) ;
    S01_r_29    : in     std_logic_vector(11 downto 0) ;
    S01_r_30    : in     std_logic_vector(11 downto 0) ;
    S01_r_31    : in     std_logic_vector(11 downto 0) ;
    S01_r_32    : in     std_logic_vector(11 downto 0) ;
    S01_r_33    : in     std_logic_vector(11 downto 0) ;
    S01_r_34    : in     std_logic_vector(11 downto 0) ;
    S01_r_35    : in     std_logic_vector(11 downto 0) ;
    S01_r_36    : in     std_logic_vector(11 downto 0) ;
    S01_r_37    : in     std_logic_vector(11 downto 0) ;
    S01_r_38    : in     std_logic_vector(11 downto 0) ;
    S01_r_39    : in     std_logic_vector(11 downto 0) ;
    S01_r_40    : in     std_logic_vector(11 downto 0) ;
    S01_r_41    : in     std_logic_vector(11 downto 0) ;
    S01_r_42    : in     std_logic_vector(11 downto 0) ;
    S01_r_43    : in     std_logic_vector(11 downto 0) ;
    S01_r_44    : in     std_logic_vector(11 downto 0) ;
    S01_r_45    : in     std_logic_vector(11 downto 0) ;
    S01_r_46    : in     std_logic_vector(11 downto 0) ;
    S01_r_47    : in     std_logic_vector(11 downto 0) ;
    S01_r_48    : in     std_logic_vector(11 downto 0) ;
    S01_r_49    : in     std_logic_vector(11 downto 0) ;
    S01_r_50    : in     std_logic_vector(11 downto 0) ;
    S01_r_51    : in     std_logic_vector(11 downto 0) ;
    S01_r_52    : in     std_logic_vector(11 downto 0) ;
    S01_r_53    : in     std_logic_vector(11 downto 0) ;
    S01_r_54    : in     std_logic_vector(11 downto 0) ;
    S01_r_55    : in     std_logic_vector(11 downto 0) ;
    S01_r_56    : in     std_logic_vector(11 downto 0) ;
    S01_r_57    : in     std_logic_vector(11 downto 0) ;
    S01_r_58    : in     std_logic_vector(11 downto 0) ;
    S01_r_59    : in     std_logic_vector(11 downto 0) ;
    S01_r_60    : in     std_logic_vector(11 downto 0) ;
    S01_r_61    : in     std_logic_vector(11 downto 0) ;
    S01_r_62    : in     std_logic_vector(11 downto 0) ;
    S01_r_63    : in     std_logic_vector(11 downto 0) ;
    S01_i_0     : in     std_logic_vector(11 downto 0) ;
    S01_i_1     : in     std_logic_vector(11 downto 0) ;
    S01_i_2     : in     std_logic_vector(11 downto 0) ;
    S01_i_3     : in     std_logic_vector(11 downto 0) ;
    S01_i_4     : in     std_logic_vector(11 downto 0) ;
    S01_i_5     : in     std_logic_vector(11 downto 0) ;
    S01_i_6     : in     std_logic_vector(11 downto 0) ;
    S01_i_7     : in     std_logic_vector(11 downto 0) ;
    S01_i_8     : in     std_logic_vector(11 downto 0) ;
    S01_i_9     : in     std_logic_vector(11 downto 0) ;
    S01_i_10    : in     std_logic_vector(11 downto 0) ;
    S01_i_11    : in     std_logic_vector(11 downto 0) ;
    S01_i_12    : in     std_logic_vector(11 downto 0) ;
    S01_i_13    : in     std_logic_vector(11 downto 0) ;
    S01_i_14    : in     std_logic_vector(11 downto 0) ;
    S01_i_15    : in     std_logic_vector(11 downto 0) ;
    S01_i_16    : in     std_logic_vector(11 downto 0) ;
    S01_i_17    : in     std_logic_vector(11 downto 0) ;
    S01_i_18    : in     std_logic_vector(11 downto 0) ;
    S01_i_19    : in     std_logic_vector(11 downto 0) ;
    S01_i_20    : in     std_logic_vector(11 downto 0) ;
    S01_i_21    : in     std_logic_vector(11 downto 0) ;
    S01_i_22    : in     std_logic_vector(11 downto 0) ;
    S01_i_23    : in     std_logic_vector(11 downto 0) ;
    S01_i_24    : in     std_logic_vector(11 downto 0) ;
    S01_i_25    : in     std_logic_vector(11 downto 0) ;
    S01_i_26    : in     std_logic_vector(11 downto 0) ;
    S01_i_27    : in     std_logic_vector(11 downto 0) ;
    S01_i_28    : in     std_logic_vector(11 downto 0) ;
    S01_i_29    : in     std_logic_vector(11 downto 0) ;
    S01_i_30    : in     std_logic_vector(11 downto 0) ;
    S01_i_31    : in     std_logic_vector(11 downto 0) ;
    S01_i_32    : in     std_logic_vector(11 downto 0) ;
    S01_i_33    : in     std_logic_vector(11 downto 0) ;
    S01_i_34    : in     std_logic_vector(11 downto 0) ;
    S01_i_35    : in     std_logic_vector(11 downto 0) ;
    S01_i_36    : in     std_logic_vector(11 downto 0) ;
    S01_i_37    : in     std_logic_vector(11 downto 0) ;
    S01_i_38    : in     std_logic_vector(11 downto 0) ;
    S01_i_39    : in     std_logic_vector(11 downto 0) ;
    S01_i_40    : in     std_logic_vector(11 downto 0) ;
    S01_i_41    : in     std_logic_vector(11 downto 0) ;
    S01_i_42    : in     std_logic_vector(11 downto 0) ;
    S01_i_43    : in     std_logic_vector(11 downto 0) ;
    S01_i_44    : in     std_logic_vector(11 downto 0) ;
    S01_i_45    : in     std_logic_vector(11 downto 0) ;
    S01_i_46    : in     std_logic_vector(11 downto 0) ;
    S01_i_47    : in     std_logic_vector(11 downto 0) ;
    S01_i_48    : in     std_logic_vector(11 downto 0) ;
    S01_i_49    : in     std_logic_vector(11 downto 0) ;
    S01_i_50    : in     std_logic_vector(11 downto 0) ;
    S01_i_51    : in     std_logic_vector(11 downto 0) ;
    S01_i_52    : in     std_logic_vector(11 downto 0) ;
    S01_i_53    : in     std_logic_vector(11 downto 0) ;
    S01_i_54    : in     std_logic_vector(11 downto 0) ;
    S01_i_55    : in     std_logic_vector(11 downto 0) ;
    S01_i_56    : in     std_logic_vector(11 downto 0) ;
    S01_i_57    : in     std_logic_vector(11 downto 0) ;
    S01_i_58    : in     std_logic_vector(11 downto 0) ;
    S01_i_59    : in     std_logic_vector(11 downto 0) ;
    S01_i_60    : in     std_logic_vector(11 downto 0) ;
    S01_i_61    : in     std_logic_vector(11 downto 0) ;
    S01_i_62    : in     std_logic_vector(11 downto 0) ;
    S01_i_63    : in     std_logic_vector(11 downto 0) ;
    S01_outr_0  : out    std_logic_vector(13 downto 0) ;
    S01_outr_1  : out    std_logic_vector(13 downto 0) ;
    S01_outr_2  : out    std_logic_vector(13 downto 0) ;
    S01_outr_3  : out    std_logic_vector(13 downto 0) ;
    S01_outr_4  : out    std_logic_vector(13 downto 0) ;
    S01_outr_5  : out    std_logic_vector(13 downto 0) ;
    S01_outr_6  : out    std_logic_vector(13 downto 0) ;
    S01_outr_7  : out    std_logic_vector(13 downto 0) ;
    S01_outr_8  : out    std_logic_vector(13 downto 0) ;
    S01_outr_9  : out    std_logic_vector(13 downto 0) ;
    S01_outr_10 : out    std_logic_vector(13 downto 0) ;
    S01_outr_11 : out    std_logic_vector(13 downto 0) ;
    S01_outr_12 : out    std_logic_vector(13 downto 0) ;
    S01_outr_13 : out    std_logic_vector(13 downto 0) ;
    S01_outr_14 : out    std_logic_vector(13 downto 0) ;
    S01_outr_15 : out    std_logic_vector(13 downto 0) ;
    S01_outr_16 : out    std_logic_vector(13 downto 0) ;
    S01_outr_17 : out    std_logic_vector(13 downto 0) ;
    S01_outr_18 : out    std_logic_vector(13 downto 0) ;
    S01_outr_19 : out    std_logic_vector(13 downto 0) ;
    S01_outr_20 : out    std_logic_vector(13 downto 0) ;
    S01_outr_21 : out    std_logic_vector(13 downto 0) ;
    S01_outr_22 : out    std_logic_vector(13 downto 0) ;
    S01_outr_23 : out    std_logic_vector(13 downto 0) ;
    S01_outr_24 : out    std_logic_vector(13 downto 0) ;
    S01_outr_25 : out    std_logic_vector(13 downto 0) ;
    S01_outr_26 : out    std_logic_vector(13 downto 0) ;
    S01_outr_27 : out    std_logic_vector(13 downto 0) ;
    S01_outr_28 : out    std_logic_vector(13 downto 0) ;
    S01_outr_29 : out    std_logic_vector(13 downto 0) ;
    S01_outr_30 : out    std_logic_vector(13 downto 0) ;
    S01_outr_31 : out    std_logic_vector(13 downto 0) ;
    S01_outr_32 : out    std_logic_vector(13 downto 0) ;
    S01_outr_33 : out    std_logic_vector(13 downto 0) ;
    S01_outr_34 : out    std_logic_vector(13 downto 0) ;
    S01_outr_35 : out    std_logic_vector(13 downto 0) ;
    S01_outr_36 : out    std_logic_vector(13 downto 0) ;
    S01_outr_37 : out    std_logic_vector(13 downto 0) ;
    S01_outr_38 : out    std_logic_vector(13 downto 0) ;
    S01_outr_39 : out    std_logic_vector(13 downto 0) ;
    S01_outr_40 : out    std_logic_vector(13 downto 0) ;
    S01_outr_41 : out    std_logic_vector(13 downto 0) ;
    S01_outr_42 : out    std_logic_vector(13 downto 0) ;
    S01_outr_43 : out    std_logic_vector(13 downto 0) ;
    S01_outr_44 : out    std_logic_vector(13 downto 0) ;
    S01_outr_45 : out    std_logic_vector(13 downto 0) ;
    S01_outr_46 : out    std_logic_vector(13 downto 0) ;
    S01_outr_47 : out    std_logic_vector(13 downto 0) ;
    S01_outr_48 : out    std_logic_vector(13 downto 0) ;
    S01_outr_49 : out    std_logic_vector(13 downto 0) ;
    S01_outr_50 : out    std_logic_vector(13 downto 0) ;
    S01_outr_51 : out    std_logic_vector(13 downto 0) ;
    S01_outr_52 : out    std_logic_vector(13 downto 0) ;
    S01_outr_53 : out    std_logic_vector(13 downto 0) ;
    S01_outr_54 : out    std_logic_vector(13 downto 0) ;
    S01_outr_55 : out    std_logic_vector(13 downto 0) ;
    S01_outr_56 : out    std_logic_vector(13 downto 0) ;
    S01_outr_57 : out    std_logic_vector(13 downto 0) ;
    S01_outr_58 : out    std_logic_vector(13 downto 0) ;
    S01_outr_59 : out    std_logic_vector(13 downto 0) ;
    S01_outr_60 : out    std_logic_vector(13 downto 0) ;
    S01_outr_61 : out    std_logic_vector(13 downto 0) ;
    S01_outr_62 : out    std_logic_vector(13 downto 0) ;
    S01_outr_63 : out    std_logic_vector(13 downto 0) ;
    S01_outi_0  : out    std_logic_vector(13 downto 0) ;
    S01_outi_1  : out    std_logic_vector(13 downto 0) ;
    S01_outi_2  : out    std_logic_vector(13 downto 0) ;
    S01_outi_3  : out    std_logic_vector(13 downto 0) ;
    S01_outi_4  : out    std_logic_vector(13 downto 0) ;
    S01_outi_5  : out    std_logic_vector(13 downto 0) ;
    S01_outi_6  : out    std_logic_vector(13 downto 0) ;
    S01_outi_7  : out    std_logic_vector(13 downto 0) ;
    S01_outi_8  : out    std_logic_vector(13 downto 0) ;
    S01_outi_9  : out    std_logic_vector(13 downto 0) ;
    S01_outi_10 : out    std_logic_vector(13 downto 0) ;
    S01_outi_11 : out    std_logic_vector(13 downto 0) ;
    S01_outi_12 : out    std_logic_vector(13 downto 0) ;
    S01_outi_13 : out    std_logic_vector(13 downto 0) ;
    S01_outi_14 : out    std_logic_vector(13 downto 0) ;
    S01_outi_15 : out    std_logic_vector(13 downto 0) ;
    S01_outi_16 : out    std_logic_vector(13 downto 0) ;
    S01_outi_17 : out    std_logic_vector(13 downto 0) ;
    S01_outi_18 : out    std_logic_vector(13 downto 0) ;
    S01_outi_19 : out    std_logic_vector(13 downto 0) ;
    S01_outi_20 : out    std_logic_vector(13 downto 0) ;
    S01_outi_21 : out    std_logic_vector(13 downto 0) ;
    S01_outi_22 : out    std_logic_vector(13 downto 0) ;
    S01_outi_23 : out    std_logic_vector(13 downto 0) ;
    S01_outi_24 : out    std_logic_vector(13 downto 0) ;
    S01_outi_25 : out    std_logic_vector(13 downto 0) ;
    S01_outi_26 : out    std_logic_vector(13 downto 0) ;
    S01_outi_27 : out    std_logic_vector(13 downto 0) ;
    S01_outi_28 : out    std_logic_vector(13 downto 0) ;
    S01_outi_29 : out    std_logic_vector(13 downto 0) ;
    S01_outi_30 : out    std_logic_vector(13 downto 0) ;
    S01_outi_31 : out    std_logic_vector(13 downto 0) ;
    S01_outi_32 : out    std_logic_vector(13 downto 0) ;
    S01_outi_33 : out    std_logic_vector(13 downto 0) ;
    S01_outi_34 : out    std_logic_vector(13 downto 0) ;
    S01_outi_35 : out    std_logic_vector(13 downto 0) ;
    S01_outi_36 : out    std_logic_vector(13 downto 0) ;
    S01_outi_37 : out    std_logic_vector(13 downto 0) ;
    S01_outi_38 : out    std_logic_vector(13 downto 0) ;
    S01_outi_39 : out    std_logic_vector(13 downto 0) ;
    S01_outi_40 : out    std_logic_vector(13 downto 0) ;
    S01_outi_41 : out    std_logic_vector(13 downto 0) ;
    S01_outi_42 : out    std_logic_vector(13 downto 0) ;
    S01_outi_43 : out    std_logic_vector(13 downto 0) ;
    S01_outi_44 : out    std_logic_vector(13 downto 0) ;
    S01_outi_45 : out    std_logic_vector(13 downto 0) ;
    S01_outi_46 : out    std_logic_vector(13 downto 0) ;
    S01_outi_47 : out    std_logic_vector(13 downto 0) ;
    S01_outi_48 : out    std_logic_vector(13 downto 0) ;
    S01_outi_49 : out    std_logic_vector(13 downto 0) ;
    S01_outi_50 : out    std_logic_vector(13 downto 0) ;
    S01_outi_51 : out    std_logic_vector(13 downto 0) ;
    S01_outi_52 : out    std_logic_vector(13 downto 0) ;
    S01_outi_53 : out    std_logic_vector(13 downto 0) ;
    S01_outi_54 : out    std_logic_vector(13 downto 0) ;
    S01_outi_55 : out    std_logic_vector(13 downto 0) ;
    S01_outi_56 : out    std_logic_vector(13 downto 0) ;
    S01_outi_58 : out    std_logic_vector(13 downto 0) ;
    S01_outi_59 : out    std_logic_vector(13 downto 0) ;
    S01_outi_57 : out    std_logic_vector(13 downto 0) ;
    S01_outi_60 : out    std_logic_vector(13 downto 0) ;
    S01_outi_61 : out    std_logic_vector(13 downto 0) ;
    S01_outi_62 : out    std_logic_vector(13 downto 0) ;
    S01_outi_63 : out    std_logic_vector(13 downto 0) ;
    clk         : in     std_logic ;
    reset       : in     std_logic ) ;
 end component ;
 component Stage02
  generic(
    N : Integer := 14 ) ;
  port(
    S02_outr_0  : out    std_logic_vector(15 downto 0) ;
    S02_outr_1  : out    std_logic_vector(15 downto 0) ;
    S02_outr_2  : out    std_logic_vector(15 downto 0) ;
    S02_outr_3  : out    std_logic_vector(15 downto 0) ;
    S02_outr_4  : out    std_logic_vector(15 downto 0) ;
    S02_outr_5  : out    std_logic_vector(15 downto 0) ;
    S02_outr_6  : out    std_logic_vector(15 downto 0) ;
    S02_outr_7  : out    std_logic_vector(15 downto 0) ;
    S02_outr_8  : out    std_logic_vector(15 downto 0) ;
    S02_outr_9  : out    std_logic_vector(15 downto 0) ;
    S02_outr_10 : out    std_logic_vector(15 downto 0) ;
    S02_outr_11 : out    std_logic_vector(15 downto 0) ;
    S02_outr_12 : out    std_logic_vector(15 downto 0) ;
    S02_outr_13 : out    std_logic_vector(15 downto 0) ;
    S02_outr_14 : out    std_logic_vector(15 downto 0) ;
    S02_outr_15 : out    std_logic_vector(15 downto 0) ;
    S02_outr_16 : out    std_logic_vector(15 downto 0) ;
    S02_outr_17 : out    std_logic_vector(15 downto 0) ;
    S02_outr_18 : out    std_logic_vector(15 downto 0) ;
    S02_outr_19 : out    std_logic_vector(15 downto 0) ;
    S02_outr_20 : out    std_logic_vector(15 downto 0) ;
    S02_outr_21 : out    std_logic_vector(15 downto 0) ;
    S02_outr_22 : out    std_logic_vector(15 downto 0) ;
    S02_outr_23 : out    std_logic_vector(15 downto 0) ;
    S02_outr_24 : out    std_logic_vector(15 downto 0) ;
    S02_outr_25 : out    std_logic_vector(15 downto 0) ;
    S02_outr_26 : out    std_logic_vector(15 downto 0) ;
    S02_outr_27 : out    std_logic_vector(15 downto 0) ;
    S02_outr_28 : out    std_logic_vector(15 downto 0) ;
    S02_outr_29 : out    std_logic_vector(15 downto 0) ;
    S02_outr_30 : out    std_logic_vector(15 downto 0) ;
    S02_outr_31 : out    std_logic_vector(15 downto 0) ;
    S02_outr_32 : out    std_logic_vector(15 downto 0) ;
    S02_outr_33 : out    std_logic_vector(15 downto 0) ;
    S02_outr_34 : out    std_logic_vector(15 downto 0) ;
    S02_outr_35 : out    std_logic_vector(15 downto 0) ;
    S02_outr_36 : out    std_logic_vector(15 downto 0) ;
    S02_outr_37 : out    std_logic_vector(15 downto 0) ;
    S02_outr_38 : out    std_logic_vector(15 downto 0) ;
    S02_outr_39 : out    std_logic_vector(15 downto 0) ;
    S02_outr_40 : out    std_logic_vector(15 downto 0) ;
    S02_outr_41 : out    std_logic_vector(15 downto 0) ;
    S02_outr_42 : out    std_logic_vector(15 downto 0) ;
    S02_outr_43 : out    std_logic_vector(15 downto 0) ;
    S02_outr_44 : out    std_logic_vector(15 downto 0) ;
    S02_outr_45 : out    std_logic_vector(15 downto 0) ;
    S02_outr_46 : out    std_logic_vector(15 downto 0) ;
    S02_outr_47 : out    std_logic_vector(15 downto 0) ;
    S02_outr_48 : out    std_logic_vector(15 downto 0) ;
    S02_outr_49 : out    std_logic_vector(15 downto 0) ;
    S02_outr_50 : out    std_logic_vector(15 downto 0) ;
    S02_outr_51 : out    std_logic_vector(15 downto 0) ;
    S02_outr_52 : out    std_logic_vector(15 downto 0) ;
    S02_outr_53 : out    std_logic_vector(15 downto 0) ;
    S02_outr_54 : out    std_logic_vector(15 downto 0) ;
    S02_outr_55 : out    std_logic_vector(15 downto 0) ;
    S02_outr_56 : out    std_logic_vector(15 downto 0) ;
    S02_outr_57 : out    std_logic_vector(15 downto 0) ;
    S02_outr_58 : out    std_logic_vector(15 downto 0) ;
    S02_outr_59 : out    std_logic_vector(15 downto 0) ;
    S02_outr_60 : out    std_logic_vector(15 downto 0) ;
    S02_outr_61 : out    std_logic_vector(15 downto 0) ;
    S02_outr_62 : out    std_logic_vector(15 downto 0) ;
    S02_outr_63 : out    std_logic_vector(15 downto 0) ;
    S02_outi_0  : out    std_logic_vector(15 downto 0) ;
    S02_outi_1  : out    std_logic_vector(15 downto 0) ;
    S02_outi_2  : out    std_logic_vector(15 downto 0) ;
    S02_outi_3  : out    std_logic_vector(15 downto 0) ;
    S02_outi_4  : out    std_logic_vector(15 downto 0) ;
    S02_outi_5  : out    std_logic_vector(15 downto 0) ;
    S02_outi_6  : out    std_logic_vector(15 downto 0) ;
    S02_outi_7  : out    std_logic_vector(15 downto 0) ;
    S02_outi_8  : out    std_logic_vector(15 downto 0) ;
    S02_outi_9  : out    std_logic_vector(15 downto 0) ;
    S02_outi_10 : out    std_logic_vector(15 downto 0) ;
    S02_outi_11 : out    std_logic_vector(15 downto 0) ;
    S02_outi_12 : out    std_logic_vector(15 downto 0) ;
    S02_outi_13 : out    std_logic_vector(15 downto 0) ;
    S02_outi_14 : out    std_logic_vector(15 downto 0) ;
    S02_outi_15 : out    std_logic_vector(15 downto 0) ;
    S02_outi_16 : out    std_logic_vector(15 downto 0) ;
    S02_outi_17 : out    std_logic_vector(15 downto 0) ;
    S02_outi_18 : out    std_logic_vector(15 downto 0) ;
    S02_outi_19 : out    std_logic_vector(15 downto 0) ;
    S02_outi_20 : out    std_logic_vector(15 downto 0) ;
    S02_outi_21 : out    std_logic_vector(15 downto 0) ;
    S02_outi_22 : out    std_logic_vector(15 downto 0) ;
    S02_outi_23 : out    std_logic_vector(15 downto 0) ;
    S02_outi_24 : out    std_logic_vector(15 downto 0) ;
    S02_outi_25 : out    std_logic_vector(15 downto 0) ;
    S02_outi_26 : out    std_logic_vector(15 downto 0) ;
    S02_outi_27 : out    std_logic_vector(15 downto 0) ;
    S02_outi_28 : out    std_logic_vector(15 downto 0) ;
    S02_outi_29 : out    std_logic_vector(15 downto 0) ;
    S02_outi_30 : out    std_logic_vector(15 downto 0) ;
    S02_outi_31 : out    std_logic_vector(15 downto 0) ;
    S02_outi_32 : out    std_logic_vector(15 downto 0) ;
    S02_outi_33 : out    std_logic_vector(15 downto 0) ;
    S02_outi_34 : out    std_logic_vector(15 downto 0) ;
    S02_outi_35 : out    std_logic_vector(15 downto 0) ;
    S02_outi_36 : out    std_logic_vector(15 downto 0) ;
    S02_outi_37 : out    std_logic_vector(15 downto 0) ;
    S02_outi_38 : out    std_logic_vector(15 downto 0) ;
    S02_outi_39 : out    std_logic_vector(15 downto 0) ;
    S02_outi_40 : out    std_logic_vector(15 downto 0) ;
    S02_outi_41 : out    std_logic_vector(15 downto 0) ;
    S02_outi_42 : out    std_logic_vector(15 downto 0) ;
    S02_outi_43 : out    std_logic_vector(15 downto 0) ;
    S02_outi_44 : out    std_logic_vector(15 downto 0) ;
    S02_outi_45 : out    std_logic_vector(15 downto 0) ;
    S02_outi_46 : out    std_logic_vector(15 downto 0) ;
    S02_outi_47 : out    std_logic_vector(15 downto 0) ;
    S02_outi_48 : out    std_logic_vector(15 downto 0) ;
    S02_outi_49 : out    std_logic_vector(15 downto 0) ;
    S02_outi_50 : out    std_logic_vector(15 downto 0) ;
    S02_outi_51 : out    std_logic_vector(15 downto 0) ;
    S02_outi_52 : out    std_logic_vector(15 downto 0) ;
    S02_outi_53 : out    std_logic_vector(15 downto 0) ;
    S02_outi_54 : out    std_logic_vector(15 downto 0) ;
    S02_outi_55 : out    std_logic_vector(15 downto 0) ;
    S02_outi_56 : out    std_logic_vector(15 downto 0) ;
    S02_outi_58 : out    std_logic_vector(15 downto 0) ;
    S02_outi_59 : out    std_logic_vector(15 downto 0) ;
    S02_outi_57 : out    std_logic_vector(15 downto 0) ;
    S02_outi_60 : out    std_logic_vector(15 downto 0) ;
    S02_outi_61 : out    std_logic_vector(15 downto 0) ;
    S02_outi_62 : out    std_logic_vector(15 downto 0) ;
    S02_outi_63 : out    std_logic_vector(15 downto 0) ;
    S02_r_0     : in     std_logic_vector(13 downto 0) ;
    S02_r_1     : in     std_logic_vector(13 downto 0) ;
    S02_r_2     : in     std_logic_vector(13 downto 0) ;
    S02_r_3     : in     std_logic_vector(13 downto 0) ;
    S02_r_4     : in     std_logic_vector(13 downto 0) ;
    S02_r_5     : in     std_logic_vector(13 downto 0) ;
    S02_r_6     : in     std_logic_vector(13 downto 0) ;
    S02_r_7     : in     std_logic_vector(13 downto 0) ;
    S02_r_8     : in     std_logic_vector(13 downto 0) ;
    S02_r_9     : in     std_logic_vector(13 downto 0) ;
    S02_r_10    : in     std_logic_vector(13 downto 0) ;
    S02_r_11    : in     std_logic_vector(13 downto 0) ;
    S02_r_12    : in     std_logic_vector(13 downto 0) ;
    S02_r_13    : in     std_logic_vector(13 downto 0) ;
    S02_r_14    : in     std_logic_vector(13 downto 0) ;
    S02_r_15    : in     std_logic_vector(13 downto 0) ;
    S02_r_16    : in     std_logic_vector(13 downto 0) ;
    S02_r_17    : in     std_logic_vector(13 downto 0) ;
    S02_r_18    : in     std_logic_vector(13 downto 0) ;
    S02_r_19    : in     std_logic_vector(13 downto 0) ;
    S02_r_20    : in     std_logic_vector(13 downto 0) ;
    S02_r_21    : in     std_logic_vector(13 downto 0) ;
    S02_r_22    : in     std_logic_vector(13 downto 0) ;
    S02_r_23    : in     std_logic_vector(13 downto 0) ;
    S02_r_24    : in     std_logic_vector(13 downto 0) ;
    S02_r_25    : in     std_logic_vector(13 downto 0) ;
    S02_r_26    : in     std_logic_vector(13 downto 0) ;
    S02_r_27    : in     std_logic_vector(13 downto 0) ;
    S02_r_28    : in     std_logic_vector(13 downto 0) ;
    S02_r_29    : in     std_logic_vector(13 downto 0) ;
    S02_r_30    : in     std_logic_vector(13 downto 0) ;
    S02_r_31    : in     std_logic_vector(13 downto 0) ;
    S02_r_32    : in     std_logic_vector(13 downto 0) ;
    S02_r_33    : in     std_logic_vector(13 downto 0) ;
    S02_r_34    : in     std_logic_vector(13 downto 0) ;
    S02_r_35    : in     std_logic_vector(13 downto 0) ;
    S02_r_36    : in     std_logic_vector(13 downto 0) ;
    S02_r_37    : in     std_logic_vector(13 downto 0) ;
    S02_r_38    : in     std_logic_vector(13 downto 0) ;
    S02_r_39    : in     std_logic_vector(13 downto 0) ;
    S02_r_40    : in     std_logic_vector(13 downto 0) ;
    S02_r_41    : in     std_logic_vector(13 downto 0) ;
    S02_r_42    : in     std_logic_vector(13 downto 0) ;
    S02_r_43    : in     std_logic_vector(13 downto 0) ;
    S02_r_44    : in     std_logic_vector(13 downto 0) ;
    S02_r_45    : in     std_logic_vector(13 downto 0) ;
    S02_r_46    : in     std_logic_vector(13 downto 0) ;
    S02_r_47    : in     std_logic_vector(13 downto 0) ;
    S02_r_48    : in     std_logic_vector(13 downto 0) ;
    S02_r_49    : in     std_logic_vector(13 downto 0) ;
    S02_r_50    : in     std_logic_vector(13 downto 0) ;
    S02_r_51    : in     std_logic_vector(13 downto 0) ;
    S02_r_52    : in     std_logic_vector(13 downto 0) ;
    S02_r_53    : in     std_logic_vector(13 downto 0) ;
    S02_r_54    : in     std_logic_vector(13 downto 0) ;
    S02_r_55    : in     std_logic_vector(13 downto 0) ;
    S02_r_56    : in     std_logic_vector(13 downto 0) ;
    S02_r_57    : in     std_logic_vector(13 downto 0) ;
    S02_r_58    : in     std_logic_vector(13 downto 0) ;
    S02_r_59    : in     std_logic_vector(13 downto 0) ;
    S02_r_60    : in     std_logic_vector(13 downto 0) ;
    S02_r_61    : in     std_logic_vector(13 downto 0) ;
    S02_r_62    : in     std_logic_vector(13 downto 0) ;
    S02_r_63    : in     std_logic_vector(13 downto 0) ;
    S02_i_0     : in     std_logic_vector(13 downto 0) ;
    S02_i_1     : in     std_logic_vector(13 downto 0) ;
    S02_i_2     : in     std_logic_vector(13 downto 0) ;
    S02_i_3     : in     std_logic_vector(13 downto 0) ;
    S02_i_4     : in     std_logic_vector(13 downto 0) ;
    S02_i_5     : in     std_logic_vector(13 downto 0) ;
    S02_i_6     : in     std_logic_vector(13 downto 0) ;
    S02_i_7     : in     std_logic_vector(13 downto 0) ;
    S02_i_8     : in     std_logic_vector(13 downto 0) ;
    S02_i_9     : in     std_logic_vector(13 downto 0) ;
    S02_i_10    : in     std_logic_vector(13 downto 0) ;
    S02_i_11    : in     std_logic_vector(13 downto 0) ;
    S02_i_12    : in     std_logic_vector(13 downto 0) ;
    S02_i_13    : in     std_logic_vector(13 downto 0) ;
    S02_i_14    : in     std_logic_vector(13 downto 0) ;
    S02_i_15    : in     std_logic_vector(13 downto 0) ;
    S02_i_16    : in     std_logic_vector(13 downto 0) ;
    S02_i_17    : in     std_logic_vector(13 downto 0) ;
    S02_i_18    : in     std_logic_vector(13 downto 0) ;
    S02_i_19    : in     std_logic_vector(13 downto 0) ;
    S02_i_20    : in     std_logic_vector(13 downto 0) ;
    S02_i_21    : in     std_logic_vector(13 downto 0) ;
    S02_i_22    : in     std_logic_vector(13 downto 0) ;
    S02_i_23    : in     std_logic_vector(13 downto 0) ;
    S02_i_24    : in     std_logic_vector(13 downto 0) ;
    S02_i_25    : in     std_logic_vector(13 downto 0) ;
    S02_i_26    : in     std_logic_vector(13 downto 0) ;
    S02_i_27    : in     std_logic_vector(13 downto 0) ;
    S02_i_28    : in     std_logic_vector(13 downto 0) ;
    S02_i_29    : in     std_logic_vector(13 downto 0) ;
    S02_i_30    : in     std_logic_vector(13 downto 0) ;
    S02_i_31    : in     std_logic_vector(13 downto 0) ;
    S02_i_32    : in     std_logic_vector(13 downto 0) ;
    S02_i_33    : in     std_logic_vector(13 downto 0) ;
    S02_i_34    : in     std_logic_vector(13 downto 0) ;
    S02_i_35    : in     std_logic_vector(13 downto 0) ;
    S02_i_36    : in     std_logic_vector(13 downto 0) ;
    S02_i_37    : in     std_logic_vector(13 downto 0) ;
    S02_i_38    : in     std_logic_vector(13 downto 0) ;
    S02_i_39    : in     std_logic_vector(13 downto 0) ;
    S02_i_40    : in     std_logic_vector(13 downto 0) ;
    S02_i_41    : in     std_logic_vector(13 downto 0) ;
    S02_i_42    : in     std_logic_vector(13 downto 0) ;
    S02_i_43    : in     std_logic_vector(13 downto 0) ;
    S02_i_44    : in     std_logic_vector(13 downto 0) ;
    S02_i_45    : in     std_logic_vector(13 downto 0) ;
    S02_i_46    : in     std_logic_vector(13 downto 0) ;
    S02_i_47    : in     std_logic_vector(13 downto 0) ;
    S02_i_48    : in     std_logic_vector(13 downto 0) ;
    S02_i_49    : in     std_logic_vector(13 downto 0) ;
    S02_i_50    : in     std_logic_vector(13 downto 0) ;
    S02_i_51    : in     std_logic_vector(13 downto 0) ;
    S02_i_52    : in     std_logic_vector(13 downto 0) ;
    S02_i_53    : in     std_logic_vector(13 downto 0) ;
    S02_i_54    : in     std_logic_vector(13 downto 0) ;
    S02_i_55    : in     std_logic_vector(13 downto 0) ;
    S02_i_56    : in     std_logic_vector(13 downto 0) ;
    S02_i_57    : in     std_logic_vector(13 downto 0) ;
    S02_i_58    : in     std_logic_vector(13 downto 0) ;
    S02_i_59    : in     std_logic_vector(13 downto 0) ;
    S02_i_60    : in     std_logic_vector(13 downto 0) ;
    S02_i_61    : in     std_logic_vector(13 downto 0) ;
    S02_i_62    : in     std_logic_vector(13 downto 0) ;
    S02_i_63    : in     std_logic_vector(13 downto 0) ;
    clk         : in     std_logic ;
    reset       : in     std_logic ) ;
 end component ;
 component Stage03
  generic(
    N : Integer := 16 ) ;
  port(
    S03_outr_0  : out    std_logic_vector(17 downto 0) ;
    S03_outr_1  : out    std_logic_vector(17 downto 0) ;
    S03_outr_2  : out    std_logic_vector(17 downto 0) ;
    S03_outr_3  : out    std_logic_vector(17 downto 0) ;
    S03_outr_4  : out    std_logic_vector(17 downto 0) ;
    S03_outr_5  : out    std_logic_vector(17 downto 0) ;
    S03_outr_6  : out    std_logic_vector(17 downto 0) ;
    S03_outr_7  : out    std_logic_vector(17 downto 0) ;
    S03_outr_8  : out    std_logic_vector(17 downto 0) ;
    S03_outr_9  : out    std_logic_vector(17 downto 0) ;
    S03_outr_10 : out    std_logic_vector(17 downto 0) ;
    S03_outr_11 : out    std_logic_vector(17 downto 0) ;
    S03_outr_12 : out    std_logic_vector(17 downto 0) ;
    S03_outr_13 : out    std_logic_vector(17 downto 0) ;
    S03_outr_14 : out    std_logic_vector(17 downto 0) ;
    S03_outr_15 : out    std_logic_vector(17 downto 0) ;
    S03_outr_16 : out    std_logic_vector(17 downto 0) ;
    S03_outr_17 : out    std_logic_vector(17 downto 0) ;
    S03_outr_18 : out    std_logic_vector(17 downto 0) ;
    S03_outr_19 : out    std_logic_vector(17 downto 0) ;
    S03_outr_20 : out    std_logic_vector(17 downto 0) ;
    S03_outr_21 : out    std_logic_vector(17 downto 0) ;
    S03_outr_22 : out    std_logic_vector(17 downto 0) ;
    S03_outr_23 : out    std_logic_vector(17 downto 0) ;
    S03_outr_24 : out    std_logic_vector(17 downto 0) ;
    S03_outr_25 : out    std_logic_vector(17 downto 0) ;
    S03_outr_26 : out    std_logic_vector(17 downto 0) ;
    S03_outr_27 : out    std_logic_vector(17 downto 0) ;
    S03_outr_28 : out    std_logic_vector(17 downto 0) ;
    S03_outr_29 : out    std_logic_vector(17 downto 0) ;
    S03_outr_30 : out    std_logic_vector(17 downto 0) ;
    S03_outr_31 : out    std_logic_vector(17 downto 0) ;
    S03_outr_32 : out    std_logic_vector(17 downto 0) ;
    S03_outr_33 : out    std_logic_vector(17 downto 0) ;
    S03_outr_34 : out    std_logic_vector(17 downto 0) ;
    S03_outr_35 : out    std_logic_vector(17 downto 0) ;
    S03_outr_36 : out    std_logic_vector(17 downto 0) ;
    S03_outr_37 : out    std_logic_vector(17 downto 0) ;
    S03_outr_38 : out    std_logic_vector(17 downto 0) ;
    S03_outr_39 : out    std_logic_vector(17 downto 0) ;
    S03_outr_40 : out    std_logic_vector(17 downto 0) ;
    S03_outr_41 : out    std_logic_vector(17 downto 0) ;
    S03_outr_42 : out    std_logic_vector(17 downto 0) ;
    S03_outr_43 : out    std_logic_vector(17 downto 0) ;
    S03_outr_44 : out    std_logic_vector(17 downto 0) ;
    S03_outr_45 : out    std_logic_vector(17 downto 0) ;
    S03_outr_46 : out    std_logic_vector(17 downto 0) ;
    S03_outr_47 : out    std_logic_vector(17 downto 0) ;
    S03_outr_48 : out    std_logic_vector(17 downto 0) ;
    S03_outr_49 : out    std_logic_vector(17 downto 0) ;
    S03_outr_50 : out    std_logic_vector(17 downto 0) ;
    S03_outr_51 : out    std_logic_vector(17 downto 0) ;
    S03_outr_52 : out    std_logic_vector(17 downto 0) ;
    S03_outr_53 : out    std_logic_vector(17 downto 0) ;
    S03_outr_54 : out    std_logic_vector(17 downto 0) ;
    S03_outr_55 : out    std_logic_vector(17 downto 0) ;
    S03_outr_56 : out    std_logic_vector(17 downto 0) ;
    S03_outr_57 : out    std_logic_vector(17 downto 0) ;
    S03_outr_58 : out    std_logic_vector(17 downto 0) ;
    S03_outr_59 : out    std_logic_vector(17 downto 0) ;
    S03_outr_60 : out    std_logic_vector(17 downto 0) ;
    S03_outr_61 : out    std_logic_vector(17 downto 0) ;
    S03_outr_62 : out    std_logic_vector(17 downto 0) ;
    S03_outr_63 : out    std_logic_vector(17 downto 0) ;
    S03_outi_0  : out    std_logic_vector(17 downto 0) ;
    S03_outi_1  : out    std_logic_vector(17 downto 0) ;
    S03_outi_2  : out    std_logic_vector(17 downto 0) ;
    S03_outi_3  : out    std_logic_vector(17 downto 0) ;
    S03_outi_4  : out    std_logic_vector(17 downto 0) ;
    S03_outi_5  : out    std_logic_vector(17 downto 0) ;
    S03_outi_6  : out    std_logic_vector(17 downto 0) ;
    S03_outi_7  : out    std_logic_vector(17 downto 0) ;
    S03_outi_8  : out    std_logic_vector(17 downto 0) ;
    S03_outi_9  : out    std_logic_vector(17 downto 0) ;
    S03_outi_10 : out    std_logic_vector(17 downto 0) ;
    S03_outi_11 : out    std_logic_vector(17 downto 0) ;
    S03_outi_12 : out    std_logic_vector(17 downto 0) ;
    S03_outi_13 : out    std_logic_vector(17 downto 0) ;
    S03_outi_14 : out    std_logic_vector(17 downto 0) ;
    S03_outi_15 : out    std_logic_vector(17 downto 0) ;
    S03_outi_16 : out    std_logic_vector(17 downto 0) ;
    S03_outi_17 : out    std_logic_vector(17 downto 0) ;
    S03_outi_18 : out    std_logic_vector(17 downto 0) ;
    S03_outi_19 : out    std_logic_vector(17 downto 0) ;
    S03_outi_20 : out    std_logic_vector(17 downto 0) ;
    S03_outi_21 : out    std_logic_vector(17 downto 0) ;
    S03_outi_22 : out    std_logic_vector(17 downto 0) ;
    S03_outi_23 : out    std_logic_vector(17 downto 0) ;
    S03_outi_24 : out    std_logic_vector(17 downto 0) ;
    S03_outi_25 : out    std_logic_vector(17 downto 0) ;
    S03_outi_26 : out    std_logic_vector(17 downto 0) ;
    S03_outi_27 : out    std_logic_vector(17 downto 0) ;
    S03_outi_28 : out    std_logic_vector(17 downto 0) ;
    S03_outi_29 : out    std_logic_vector(17 downto 0) ;
    S03_outi_30 : out    std_logic_vector(17 downto 0) ;
    S03_outi_31 : out    std_logic_vector(17 downto 0) ;
    S03_outi_32 : out    std_logic_vector(17 downto 0) ;
    S03_outi_33 : out    std_logic_vector(17 downto 0) ;
    S03_outi_34 : out    std_logic_vector(17 downto 0) ;
    S03_outi_35 : out    std_logic_vector(17 downto 0) ;
    S03_outi_36 : out    std_logic_vector(17 downto 0) ;
    S03_outi_37 : out    std_logic_vector(17 downto 0) ;
    S03_outi_38 : out    std_logic_vector(17 downto 0) ;
    S03_outi_39 : out    std_logic_vector(17 downto 0) ;
    S03_outi_40 : out    std_logic_vector(17 downto 0) ;
    S03_outi_41 : out    std_logic_vector(17 downto 0) ;
    S03_outi_42 : out    std_logic_vector(17 downto 0) ;
    S03_outi_43 : out    std_logic_vector(17 downto 0) ;
    S03_outi_44 : out    std_logic_vector(17 downto 0) ;
    S03_outi_45 : out    std_logic_vector(17 downto 0) ;
    S03_outi_46 : out    std_logic_vector(17 downto 0) ;
    S03_outi_47 : out    std_logic_vector(17 downto 0) ;
    S03_outi_48 : out    std_logic_vector(17 downto 0) ;
    S03_outi_49 : out    std_logic_vector(17 downto 0) ;
    S03_outi_50 : out    std_logic_vector(17 downto 0) ;
    S03_outi_51 : out    std_logic_vector(17 downto 0) ;
    S03_outi_52 : out    std_logic_vector(17 downto 0) ;
    S03_outi_53 : out    std_logic_vector(17 downto 0) ;
    S03_outi_54 : out    std_logic_vector(17 downto 0) ;
    S03_outi_55 : out    std_logic_vector(17 downto 0) ;
    S03_outi_56 : out    std_logic_vector(17 downto 0) ;
    S03_outi_58 : out    std_logic_vector(17 downto 0) ;
    S03_outi_59 : out    std_logic_vector(17 downto 0) ;
    S03_outi_57 : out    std_logic_vector(17 downto 0) ;
    S03_outi_60 : out    std_logic_vector(17 downto 0) ;
    S03_outi_61 : out    std_logic_vector(17 downto 0) ;
    S03_outi_62 : out    std_logic_vector(17 downto 0) ;
    S03_outi_63 : out    std_logic_vector(17 downto 0) ;
    S03_r_0     : in     std_logic_vector(15 downto 0) ;
    S03_r_1     : in     std_logic_vector(15 downto 0) ;
    S03_r_2     : in     std_logic_vector(15 downto 0) ;
    S03_r_3     : in     std_logic_vector(15 downto 0) ;
    S03_r_4     : in     std_logic_vector(15 downto 0) ;
    S03_r_5     : in     std_logic_vector(15 downto 0) ;
    S03_r_6     : in     std_logic_vector(15 downto 0) ;
    S03_r_7     : in     std_logic_vector(15 downto 0) ;
    S03_r_8     : in     std_logic_vector(15 downto 0) ;
    S03_r_9     : in     std_logic_vector(15 downto 0) ;
    S03_r_10    : in     std_logic_vector(15 downto 0) ;
    S03_r_11    : in     std_logic_vector(15 downto 0) ;
    S03_r_12    : in     std_logic_vector(15 downto 0) ;
    S03_r_13    : in     std_logic_vector(15 downto 0) ;
    S03_r_14    : in     std_logic_vector(15 downto 0) ;
    S03_r_15    : in     std_logic_vector(15 downto 0) ;
    S03_r_16    : in     std_logic_vector(15 downto 0) ;
    S03_r_17    : in     std_logic_vector(15 downto 0) ;
    S03_r_18    : in     std_logic_vector(15 downto 0) ;
    S03_r_19    : in     std_logic_vector(15 downto 0) ;
    S03_r_20    : in     std_logic_vector(15 downto 0) ;
    S03_r_21    : in     std_logic_vector(15 downto 0) ;
    S03_r_22    : in     std_logic_vector(15 downto 0) ;
    S03_r_23    : in     std_logic_vector(15 downto 0) ;
    S03_r_24    : in     std_logic_vector(15 downto 0) ;
    S03_r_25    : in     std_logic_vector(15 downto 0) ;
    S03_r_26    : in     std_logic_vector(15 downto 0) ;
    S03_r_27    : in     std_logic_vector(15 downto 0) ;
    S03_r_28    : in     std_logic_vector(15 downto 0) ;
    S03_r_29    : in     std_logic_vector(15 downto 0) ;
    S03_r_30    : in     std_logic_vector(15 downto 0) ;
    S03_r_31    : in     std_logic_vector(15 downto 0) ;
    S03_r_32    : in     std_logic_vector(15 downto 0) ;
    S03_r_33    : in     std_logic_vector(15 downto 0) ;
    S03_r_34    : in     std_logic_vector(15 downto 0) ;
    S03_r_35    : in     std_logic_vector(15 downto 0) ;
    S03_r_36    : in     std_logic_vector(15 downto 0) ;
    S03_r_37    : in     std_logic_vector(15 downto 0) ;
    S03_r_38    : in     std_logic_vector(15 downto 0) ;
    S03_r_39    : in     std_logic_vector(15 downto 0) ;
    S03_r_40    : in     std_logic_vector(15 downto 0) ;
    S03_r_41    : in     std_logic_vector(15 downto 0) ;
    S03_r_42    : in     std_logic_vector(15 downto 0) ;
    S03_r_43    : in     std_logic_vector(15 downto 0) ;
    S03_r_44    : in     std_logic_vector(15 downto 0) ;
    S03_r_45    : in     std_logic_vector(15 downto 0) ;
    S03_r_46    : in     std_logic_vector(15 downto 0) ;
    S03_r_47    : in     std_logic_vector(15 downto 0) ;
    S03_r_48    : in     std_logic_vector(15 downto 0) ;
    S03_r_49    : in     std_logic_vector(15 downto 0) ;
    S03_r_50    : in     std_logic_vector(15 downto 0) ;
    S03_r_51    : in     std_logic_vector(15 downto 0) ;
    S03_r_52    : in     std_logic_vector(15 downto 0) ;
    S03_r_53    : in     std_logic_vector(15 downto 0) ;
    S03_r_54    : in     std_logic_vector(15 downto 0) ;
    S03_r_55    : in     std_logic_vector(15 downto 0) ;
    S03_r_56    : in     std_logic_vector(15 downto 0) ;
    S03_r_57    : in     std_logic_vector(15 downto 0) ;
    S03_r_58    : in     std_logic_vector(15 downto 0) ;
    S03_r_59    : in     std_logic_vector(15 downto 0) ;
    S03_r_60    : in     std_logic_vector(15 downto 0) ;
    S03_r_61    : in     std_logic_vector(15 downto 0) ;
    S03_r_62    : in     std_logic_vector(15 downto 0) ;
    S03_r_63    : in     std_logic_vector(15 downto 0) ;
    S03_i_0     : in     std_logic_vector(15 downto 0) ;
    S03_i_1     : in     std_logic_vector(15 downto 0) ;
    S03_i_2     : in     std_logic_vector(15 downto 0) ;
    S03_i_3     : in     std_logic_vector(15 downto 0) ;
    S03_i_4     : in     std_logic_vector(15 downto 0) ;
    S03_i_5     : in     std_logic_vector(15 downto 0) ;
    S03_i_6     : in     std_logic_vector(15 downto 0) ;
    S03_i_7     : in     std_logic_vector(15 downto 0) ;
    S03_i_8     : in     std_logic_vector(15 downto 0) ;
    S03_i_9     : in     std_logic_vector(15 downto 0) ;
    S03_i_10    : in     std_logic_vector(15 downto 0) ;
    S03_i_11    : in     std_logic_vector(15 downto 0) ;
    S03_i_12    : in     std_logic_vector(15 downto 0) ;
    S03_i_13    : in     std_logic_vector(15 downto 0) ;
    S03_i_14    : in     std_logic_vector(15 downto 0) ;
    S03_i_15    : in     std_logic_vector(15 downto 0) ;
    S03_i_16    : in     std_logic_vector(15 downto 0) ;
    S03_i_17    : in     std_logic_vector(15 downto 0) ;
    S03_i_18    : in     std_logic_vector(15 downto 0) ;
    S03_i_19    : in     std_logic_vector(15 downto 0) ;
    S03_i_20    : in     std_logic_vector(15 downto 0) ;
    S03_i_21    : in     std_logic_vector(15 downto 0) ;
    S03_i_22    : in     std_logic_vector(15 downto 0) ;
    S03_i_23    : in     std_logic_vector(15 downto 0) ;
    S03_i_24    : in     std_logic_vector(15 downto 0) ;
    S03_i_25    : in     std_logic_vector(15 downto 0) ;
    S03_i_26    : in     std_logic_vector(15 downto 0) ;
    S03_i_27    : in     std_logic_vector(15 downto 0) ;
    S03_i_28    : in     std_logic_vector(15 downto 0) ;
    S03_i_29    : in     std_logic_vector(15 downto 0) ;
    S03_i_30    : in     std_logic_vector(15 downto 0) ;
    S03_i_31    : in     std_logic_vector(15 downto 0) ;
    S03_i_32    : in     std_logic_vector(15 downto 0) ;
    S03_i_33    : in     std_logic_vector(15 downto 0) ;
    S03_i_34    : in     std_logic_vector(15 downto 0) ;
    S03_i_35    : in     std_logic_vector(15 downto 0) ;
    S03_i_36    : in     std_logic_vector(15 downto 0) ;
    S03_i_37    : in     std_logic_vector(15 downto 0) ;
    S03_i_38    : in     std_logic_vector(15 downto 0) ;
    S03_i_39    : in     std_logic_vector(15 downto 0) ;
    S03_i_40    : in     std_logic_vector(15 downto 0) ;
    S03_i_41    : in     std_logic_vector(15 downto 0) ;
    S03_i_42    : in     std_logic_vector(15 downto 0) ;
    S03_i_43    : in     std_logic_vector(15 downto 0) ;
    S03_i_44    : in     std_logic_vector(15 downto 0) ;
    S03_i_45    : in     std_logic_vector(15 downto 0) ;
    S03_i_46    : in     std_logic_vector(15 downto 0) ;
    S03_i_47    : in     std_logic_vector(15 downto 0) ;
    S03_i_48    : in     std_logic_vector(15 downto 0) ;
    S03_i_49    : in     std_logic_vector(15 downto 0) ;
    S03_i_50    : in     std_logic_vector(15 downto 0) ;
    S03_i_51    : in     std_logic_vector(15 downto 0) ;
    S03_i_52    : in     std_logic_vector(15 downto 0) ;
    S03_i_53    : in     std_logic_vector(15 downto 0) ;
    S03_i_54    : in     std_logic_vector(15 downto 0) ;
    S03_i_55    : in     std_logic_vector(15 downto 0) ;
    S03_i_56    : in     std_logic_vector(15 downto 0) ;
    S03_i_57    : in     std_logic_vector(15 downto 0) ;
    S03_i_58    : in     std_logic_vector(15 downto 0) ;
    S03_i_59    : in     std_logic_vector(15 downto 0) ;
    S03_i_60    : in     std_logic_vector(15 downto 0) ;
    S03_i_61    : in     std_logic_vector(15 downto 0) ;
    S03_i_62    : in     std_logic_vector(15 downto 0) ;
    S03_i_63    : in     std_logic_vector(15 downto 0) ;
    clk         : in     std_logic ;
    reset       : in     std_logic ) ;
 end component ;
 component Shift_in
  generic(
    N            : Integer := 12 ;
    no_of_blocks : Integer := 64 ) ;
  port(
    Shift_in_r_0  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_1  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_2  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_3  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_4  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_5  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_6  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_7  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_8  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_9  : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_10 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_11 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_12 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_13 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_14 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_15 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_16 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_17 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_18 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_19 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_20 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_21 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_22 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_23 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_24 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_25 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_26 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_27 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_28 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_29 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_30 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_31 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_32 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_33 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_34 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_35 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_36 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_37 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_38 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_39 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_40 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_41 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_42 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_43 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_44 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_45 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_46 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_47 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_48 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_49 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_50 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_51 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_52 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_53 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_54 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_55 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_56 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_57 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_58 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_59 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_60 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_61 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_62 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r_63 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_0  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_1  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_2  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_3  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_4  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_5  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_6  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_7  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_8  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_9  : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_10 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_11 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_12 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_13 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_14 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_15 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_16 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_17 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_18 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_19 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_20 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_21 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_22 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_23 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_24 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_25 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_26 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_27 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_28 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_29 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_30 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_31 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_32 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_33 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_34 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_35 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_36 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_37 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_38 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_39 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_40 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_41 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_42 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_43 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_44 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_45 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_46 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_47 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_48 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_49 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_50 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_51 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_52 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_53 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_54 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_55 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_56 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_58 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_59 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_57 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_60 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_61 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_62 : out    std_logic_vector(11 downto 0) ;
    Shift_in_i_63 : out    std_logic_vector(11 downto 0) ;
    Shift_in_r    : in     std_logic_vector(11 downto 0) ;
    Shift_in_i    : in     std_logic_vector(11 downto 0) ;
    clk           : in     std_logic ;
    reset         : in     std_logic ;
    done          : out    std_logic ;
    Data_valid    : in     std_logic ) ;
 end component ;
 component Shift_out
  generic(
    N            : Integer := 18 ;
    no_of_blocks : Integer := 64 ) ;
  port(
    Shift_out_r_0  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_1  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_2  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_3  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_4  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_5  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_6  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_7  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_8  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_9  : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_10 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_11 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_12 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_13 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_14 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_15 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_16 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_17 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_18 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_19 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_20 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_21 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_22 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_23 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_24 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_25 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_26 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_27 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_28 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_29 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_30 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_31 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_32 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_33 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_34 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_35 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_36 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_37 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_38 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_39 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_40 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_41 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_42 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_43 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_44 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_45 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_46 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_47 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_48 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_49 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_50 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_51 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_52 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_53 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_54 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_55 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_56 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_57 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_58 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_59 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_60 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_61 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_62 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r_63 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_0  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_1  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_2  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_3  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_4  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_5  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_6  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_7  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_8  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_9  : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_10 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_11 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_12 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_13 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_14 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_15 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_16 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_17 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_18 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_19 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_20 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_21 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_22 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_23 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_24 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_25 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_26 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_27 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_28 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_29 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_30 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_31 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_32 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_33 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_34 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_35 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_36 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_37 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_38 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_39 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_40 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_41 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_42 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_43 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_44 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_45 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_46 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_47 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_48 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_49 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_50 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_51 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_52 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_53 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_54 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_55 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_56 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_57 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_58 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_59 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_60 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_61 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_62 : in     std_logic_vector(17 downto 0) ;
    Shift_out_i_63 : in     std_logic_vector(17 downto 0) ;
    Shift_out_r    : out    std_logic_vector(17 downto 0) ;
    Shift_out_i    : out    std_logic_vector(17 downto 0) ;
    clk            : in     std_logic ;
    valid_data     : in     std_logic ;
    Data_valid_out : out    std_logic ;
    reset          : in     std_logic ) ;
 end component ;
 component Valid
  generic(
    delay : Integer := 15 ) ;
  port(
    done_shift_in : in     std_logic ;
    clk           : in     std_logic ;
    reset         : in     std_logic ;
    data_valid    : out    std_logic ) ;
 end component ;
 signal Net_0   : std_logic_vector(13 downto 0) ;
 signal Net_1   : std_logic_vector(13 downto 0) ;
 signal Net_2   : std_logic_vector(13 downto 0) ;
 signal Net_3   : std_logic_vector(13 downto 0) ;
 signal Net_4   : std_logic_vector(13 downto 0) ;
 signal Net_5   : std_logic_vector(13 downto 0) ;
 signal Net_6   : std_logic_vector(13 downto 0) ;
 signal Net_7   : std_logic_vector(13 downto 0) ;
 signal Net_8   : std_logic_vector(13 downto 0) ;
 signal Net_9   : std_logic_vector(13 downto 0) ;
 signal Net_10  : std_logic_vector(13 downto 0) ;
 signal Net_11  : std_logic_vector(13 downto 0) ;
 signal Net_12  : std_logic_vector(13 downto 0) ;
 signal Net_13  : std_logic_vector(13 downto 0) ;
 signal Net_14  : std_logic_vector(13 downto 0) ;
 signal Net_15  : std_logic_vector(13 downto 0) ;
 signal Net_16  : std_logic_vector(13 downto 0) ;
 signal Net_17  : std_logic_vector(13 downto 0) ;
 signal Net_18  : std_logic_vector(13 downto 0) ;
 signal Net_19  : std_logic_vector(13 downto 0) ;
 signal Net_20  : std_logic_vector(13 downto 0) ;
 signal Net_21  : std_logic_vector(13 downto 0) ;
 signal Net_22  : std_logic_vector(13 downto 0) ;
 signal Net_23  : std_logic_vector(13 downto 0) ;
 signal Net_24  : std_logic_vector(13 downto 0) ;
 signal Net_25  : std_logic_vector(13 downto 0) ;
 signal Net_26  : std_logic_vector(13 downto 0) ;
 signal Net_27  : std_logic_vector(13 downto 0) ;
 signal Net_28  : std_logic_vector(13 downto 0) ;
 signal Net_29  : std_logic_vector(13 downto 0) ;
 signal Net_30  : std_logic_vector(13 downto 0) ;
 signal Net_31  : std_logic_vector(13 downto 0) ;
 signal Net_32  : std_logic_vector(13 downto 0) ;
 signal Net_33  : std_logic_vector(13 downto 0) ;
 signal Net_34  : std_logic_vector(13 downto 0) ;
 signal Net_35  : std_logic_vector(13 downto 0) ;
 signal Net_36  : std_logic_vector(13 downto 0) ;
 signal Net_37  : std_logic_vector(13 downto 0) ;
 signal Net_38  : std_logic_vector(13 downto 0) ;
 signal Net_39  : std_logic_vector(13 downto 0) ;
 signal Net_40  : std_logic_vector(13 downto 0) ;
 signal Net_41  : std_logic_vector(13 downto 0) ;
 signal Net_42  : std_logic_vector(13 downto 0) ;
 signal Net_43  : std_logic_vector(13 downto 0) ;
 signal Net_44  : std_logic_vector(13 downto 0) ;
 signal Net_45  : std_logic_vector(13 downto 0) ;
 signal Net_46  : std_logic_vector(13 downto 0) ;
 signal Net_47  : std_logic_vector(13 downto 0) ;
 signal Net_48  : std_logic_vector(13 downto 0) ;
 signal Net_49  : std_logic_vector(13 downto 0) ;
 signal Net_50  : std_logic_vector(13 downto 0) ;
 signal Net_51  : std_logic_vector(13 downto 0) ;
 signal Net_52  : std_logic_vector(13 downto 0) ;
 signal Net_53  : std_logic_vector(13 downto 0) ;
 signal Net_54  : std_logic_vector(13 downto 0) ;
 signal Net_55  : std_logic_vector(13 downto 0) ;
 signal Net_56  : std_logic_vector(13 downto 0) ;
 signal Net_57  : std_logic_vector(13 downto 0) ;
 signal Net_58  : std_logic_vector(13 downto 0) ;
 signal Net_59  : std_logic_vector(13 downto 0) ;
 signal Net_60  : std_logic_vector(13 downto 0) ;
 signal Net_61  : std_logic_vector(13 downto 0) ;
 signal Net_62  : std_logic_vector(13 downto 0) ;
 signal Net_63  : std_logic_vector(13 downto 0) ;
 signal Net_64  : std_logic_vector(13 downto 0) ;
 signal Net_65  : std_logic_vector(13 downto 0) ;
 signal Net_66  : std_logic_vector(13 downto 0) ;
 signal Net_67  : std_logic_vector(13 downto 0) ;
 signal Net_68  : std_logic_vector(13 downto 0) ;
 signal Net_69  : std_logic_vector(13 downto 0) ;
 signal Net_70  : std_logic_vector(13 downto 0) ;
 signal Net_71  : std_logic_vector(13 downto 0) ;
 signal Net_72  : std_logic_vector(13 downto 0) ;
 signal Net_73  : std_logic_vector(13 downto 0) ;
 signal Net_74  : std_logic_vector(13 downto 0) ;
 signal Net_75  : std_logic_vector(13 downto 0) ;
 signal Net_76  : std_logic_vector(13 downto 0) ;
 signal Net_77  : std_logic_vector(13 downto 0) ;
 signal Net_78  : std_logic_vector(13 downto 0) ;
 signal Net_79  : std_logic_vector(13 downto 0) ;
 signal Net_80  : std_logic_vector(13 downto 0) ;
 signal Net_81  : std_logic_vector(13 downto 0) ;
 signal Net_82  : std_logic_vector(13 downto 0) ;
 signal Net_83  : std_logic_vector(13 downto 0) ;
 signal Net_84  : std_logic_vector(13 downto 0) ;
 signal Net_85  : std_logic_vector(13 downto 0) ;
 signal Net_86  : std_logic_vector(13 downto 0) ;
 signal Net_87  : std_logic_vector(13 downto 0) ;
 signal Net_88  : std_logic_vector(13 downto 0) ;
 signal Net_89  : std_logic_vector(13 downto 0) ;
 signal Net_90  : std_logic_vector(13 downto 0) ;
 signal Net_91  : std_logic_vector(13 downto 0) ;
 signal Net_92  : std_logic_vector(13 downto 0) ;
 signal Net_93  : std_logic_vector(13 downto 0) ;
 signal Net_94  : std_logic_vector(13 downto 0) ;
 signal Net_95  : std_logic_vector(13 downto 0) ;
 signal Net_96  : std_logic_vector(13 downto 0) ;
 signal Net_97  : std_logic_vector(13 downto 0) ;
 signal Net_98  : std_logic_vector(13 downto 0) ;
 signal Net_99  : std_logic_vector(13 downto 0) ;
 signal Net_100 : std_logic_vector(13 downto 0) ;
 signal Net_101 : std_logic_vector(13 downto 0) ;
 signal Net_102 : std_logic_vector(13 downto 0) ;
 signal Net_103 : std_logic_vector(13 downto 0) ;
 signal Net_104 : std_logic_vector(13 downto 0) ;
 signal Net_105 : std_logic_vector(13 downto 0) ;
 signal Net_106 : std_logic_vector(13 downto 0) ;
 signal Net_107 : std_logic_vector(13 downto 0) ;
 signal Net_108 : std_logic_vector(13 downto 0) ;
 signal Net_109 : std_logic_vector(13 downto 0) ;
 signal Net_110 : std_logic_vector(13 downto 0) ;
 signal Net_111 : std_logic_vector(13 downto 0) ;
 signal Net_112 : std_logic_vector(13 downto 0) ;
 signal Net_113 : std_logic_vector(13 downto 0) ;
 signal Net_114 : std_logic_vector(13 downto 0) ;
 signal Net_115 : std_logic_vector(13 downto 0) ;
 signal Net_116 : std_logic_vector(13 downto 0) ;
 signal Net_117 : std_logic_vector(13 downto 0) ;
 signal Net_118 : std_logic_vector(13 downto 0) ;
 signal Net_119 : std_logic_vector(13 downto 0) ;
 signal Net_120 : std_logic_vector(13 downto 0) ;
 signal Net_121 : std_logic_vector(13 downto 0) ;
 signal Net_122 : std_logic_vector(13 downto 0) ;
 signal Net_123 : std_logic_vector(13 downto 0) ;
 signal Net_124 : std_logic_vector(13 downto 0) ;
 signal Net_125 : std_logic_vector(13 downto 0) ;
 signal Net_126 : std_logic_vector(13 downto 0) ;
 signal Net_127 : std_logic_vector(13 downto 0) ;
 signal Net_129 : std_logic_vector(15 downto 0) ;
 signal Net_130 : std_logic_vector(15 downto 0) ;
 signal Net_131 : std_logic_vector(15 downto 0) ;
 signal Net_132 : std_logic_vector(15 downto 0) ;
 signal Net_133 : std_logic_vector(15 downto 0) ;
 signal Net_134 : std_logic_vector(15 downto 0) ;
 signal Net_135 : std_logic_vector(15 downto 0) ;
 signal Net_136 : std_logic_vector(15 downto 0) ;
 signal Net_137 : std_logic_vector(15 downto 0) ;
 signal Net_138 : std_logic_vector(15 downto 0) ;
 signal Net_139 : std_logic_vector(15 downto 0) ;
 signal Net_140 : std_logic_vector(15 downto 0) ;
 signal Net_141 : std_logic_vector(15 downto 0) ;
 signal Net_142 : std_logic_vector(15 downto 0) ;
 signal Net_143 : std_logic_vector(15 downto 0) ;
 signal Net_144 : std_logic_vector(15 downto 0) ;
 signal Net_145 : std_logic_vector(15 downto 0) ;
 signal Net_146 : std_logic_vector(15 downto 0) ;
 signal Net_147 : std_logic_vector(15 downto 0) ;
 signal Net_148 : std_logic_vector(15 downto 0) ;
 signal Net_149 : std_logic_vector(15 downto 0) ;
 signal Net_150 : std_logic_vector(15 downto 0) ;
 signal Net_151 : std_logic_vector(15 downto 0) ;
 signal Net_152 : std_logic_vector(15 downto 0) ;
 signal Net_153 : std_logic_vector(15 downto 0) ;
 signal Net_154 : std_logic_vector(15 downto 0) ;
 signal Net_155 : std_logic_vector(15 downto 0) ;
 signal Net_156 : std_logic_vector(15 downto 0) ;
 signal Net_157 : std_logic_vector(15 downto 0) ;
 signal Net_158 : std_logic_vector(15 downto 0) ;
 signal Net_159 : std_logic_vector(15 downto 0) ;
 signal Net_160 : std_logic_vector(15 downto 0) ;
 signal Net_161 : std_logic_vector(15 downto 0) ;
 signal Net_162 : std_logic_vector(15 downto 0) ;
 signal Net_163 : std_logic_vector(15 downto 0) ;
 signal Net_164 : std_logic_vector(15 downto 0) ;
 signal Net_165 : std_logic_vector(15 downto 0) ;
 signal Net_166 : std_logic_vector(15 downto 0) ;
 signal Net_167 : std_logic_vector(15 downto 0) ;
 signal Net_168 : std_logic_vector(15 downto 0) ;
 signal Net_169 : std_logic_vector(15 downto 0) ;
 signal Net_170 : std_logic_vector(15 downto 0) ;
 signal Net_171 : std_logic_vector(15 downto 0) ;
 signal Net_172 : std_logic_vector(15 downto 0) ;
 signal Net_173 : std_logic_vector(15 downto 0) ;
 signal Net_174 : std_logic_vector(15 downto 0) ;
 signal Net_175 : std_logic_vector(15 downto 0) ;
 signal Net_176 : std_logic_vector(15 downto 0) ;
 signal Net_177 : std_logic_vector(15 downto 0) ;
 signal Net_178 : std_logic_vector(15 downto 0) ;
 signal Net_179 : std_logic_vector(15 downto 0) ;
 signal Net_180 : std_logic_vector(15 downto 0) ;
 signal Net_181 : std_logic_vector(15 downto 0) ;
 signal Net_182 : std_logic_vector(15 downto 0) ;
 signal Net_183 : std_logic_vector(15 downto 0) ;
 signal Net_184 : std_logic_vector(15 downto 0) ;
 signal Net_185 : std_logic_vector(15 downto 0) ;
 signal Net_186 : std_logic_vector(15 downto 0) ;
 signal Net_187 : std_logic_vector(15 downto 0) ;
 signal Net_188 : std_logic_vector(15 downto 0) ;
 signal Net_189 : std_logic_vector(15 downto 0) ;
 signal Net_190 : std_logic_vector(15 downto 0) ;
 signal Net_191 : std_logic_vector(15 downto 0) ;
 signal Net_192 : std_logic_vector(15 downto 0) ;
 signal Net_193 : std_logic_vector(15 downto 0) ;
 signal Net_194 : std_logic_vector(15 downto 0) ;
 signal Net_195 : std_logic_vector(15 downto 0) ;
 signal Net_196 : std_logic_vector(15 downto 0) ;
 signal Net_197 : std_logic_vector(15 downto 0) ;
 signal Net_198 : std_logic_vector(15 downto 0) ;
 signal Net_199 : std_logic_vector(15 downto 0) ;
 signal Net_200 : std_logic_vector(15 downto 0) ;
 signal Net_201 : std_logic_vector(15 downto 0) ;
 signal Net_202 : std_logic_vector(15 downto 0) ;
 signal Net_203 : std_logic_vector(15 downto 0) ;
 signal Net_204 : std_logic_vector(15 downto 0) ;
 signal Net_205 : std_logic_vector(15 downto 0) ;
 signal Net_206 : std_logic_vector(15 downto 0) ;
 signal Net_207 : std_logic_vector(15 downto 0) ;
 signal Net_208 : std_logic_vector(15 downto 0) ;
 signal Net_209 : std_logic_vector(15 downto 0) ;
 signal Net_210 : std_logic_vector(15 downto 0) ;
 signal Net_211 : std_logic_vector(15 downto 0) ;
 signal Net_212 : std_logic_vector(15 downto 0) ;
 signal Net_213 : std_logic_vector(15 downto 0) ;
 signal Net_214 : std_logic_vector(15 downto 0) ;
 signal Net_215 : std_logic_vector(15 downto 0) ;
 signal Net_216 : std_logic_vector(15 downto 0) ;
 signal Net_217 : std_logic_vector(15 downto 0) ;
 signal Net_218 : std_logic_vector(15 downto 0) ;
 signal Net_219 : std_logic_vector(15 downto 0) ;
 signal Net_220 : std_logic_vector(15 downto 0) ;
 signal Net_221 : std_logic_vector(15 downto 0) ;
 signal Net_222 : std_logic_vector(15 downto 0) ;
 signal Net_223 : std_logic_vector(15 downto 0) ;
 signal Net_224 : std_logic_vector(15 downto 0) ;
 signal Net_225 : std_logic_vector(15 downto 0) ;
 signal Net_226 : std_logic_vector(15 downto 0) ;
 signal Net_227 : std_logic_vector(15 downto 0) ;
 signal Net_228 : std_logic_vector(15 downto 0) ;
 signal Net_229 : std_logic_vector(15 downto 0) ;
 signal Net_230 : std_logic_vector(15 downto 0) ;
 signal Net_231 : std_logic_vector(15 downto 0) ;
 signal Net_232 : std_logic_vector(15 downto 0) ;
 signal Net_233 : std_logic_vector(15 downto 0) ;
 signal Net_234 : std_logic_vector(15 downto 0) ;
 signal Net_235 : std_logic_vector(15 downto 0) ;
 signal Net_236 : std_logic_vector(15 downto 0) ;
 signal Net_237 : std_logic_vector(15 downto 0) ;
 signal Net_238 : std_logic_vector(15 downto 0) ;
 signal Net_239 : std_logic_vector(15 downto 0) ;
 signal Net_240 : std_logic_vector(15 downto 0) ;
 signal Net_241 : std_logic_vector(15 downto 0) ;
 signal Net_242 : std_logic_vector(15 downto 0) ;
 signal Net_243 : std_logic_vector(15 downto 0) ;
 signal Net_244 : std_logic_vector(15 downto 0) ;
 signal Net_245 : std_logic_vector(15 downto 0) ;
 signal Net_246 : std_logic_vector(15 downto 0) ;
 signal Net_247 : std_logic_vector(15 downto 0) ;
 signal Net_248 : std_logic_vector(15 downto 0) ;
 signal Net_249 : std_logic_vector(15 downto 0) ;
 signal Net_250 : std_logic_vector(15 downto 0) ;
 signal Net_251 : std_logic_vector(15 downto 0) ;
 signal Net_252 : std_logic_vector(15 downto 0) ;
 signal Net_253 : std_logic_vector(15 downto 0) ;
 signal Net_254 : std_logic_vector(15 downto 0) ;
 signal Net_255 : std_logic_vector(15 downto 0) ;
 signal Net_256 : std_logic_vector(15 downto 0) ;
 signal Net_257 : std_logic_vector(17 downto 0) ;
 signal Net_258 : std_logic_vector(11 downto 0) ;
 signal Net_259 : std_logic_vector(11 downto 0) ;
 signal Net_260 : std_logic_vector(11 downto 0) ;
 signal Net_261 : std_logic_vector(11 downto 0) ;
 signal Net_262 : std_logic_vector(11 downto 0) ;
 signal Net_263 : std_logic_vector(11 downto 0) ;
 signal Net_264 : std_logic_vector(11 downto 0) ;
 signal Net_265 : std_logic_vector(11 downto 0) ;
 signal Net_266 : std_logic_vector(11 downto 0) ;
 signal Net_267 : std_logic_vector(11 downto 0) ;
 signal Net_268 : std_logic_vector(11 downto 0) ;
 signal Net_269 : std_logic_vector(11 downto 0) ;
 signal Net_270 : std_logic_vector(11 downto 0) ;
 signal Net_271 : std_logic_vector(11 downto 0) ;
 signal Net_272 : std_logic_vector(11 downto 0) ;
 signal Net_273 : std_logic_vector(11 downto 0) ;
 signal Net_274 : std_logic_vector(11 downto 0) ;
 signal Net_275 : std_logic_vector(11 downto 0) ;
 signal Net_276 : std_logic_vector(11 downto 0) ;
 signal Net_277 : std_logic_vector(11 downto 0) ;
 signal Net_278 : std_logic_vector(11 downto 0) ;
 signal Net_279 : std_logic_vector(11 downto 0) ;
 signal Net_280 : std_logic_vector(11 downto 0) ;
 signal Net_281 : std_logic_vector(11 downto 0) ;
 signal Net_282 : std_logic_vector(11 downto 0) ;
 signal Net_283 : std_logic_vector(11 downto 0) ;
 signal Net_284 : std_logic_vector(11 downto 0) ;
 signal Net_285 : std_logic_vector(11 downto 0) ;
 signal Net_286 : std_logic_vector(11 downto 0) ;
 signal Net_287 : std_logic_vector(11 downto 0) ;
 signal Net_288 : std_logic_vector(11 downto 0) ;
 signal Net_289 : std_logic_vector(11 downto 0) ;
 signal Net_290 : std_logic_vector(11 downto 0) ;
 signal Net_291 : std_logic_vector(11 downto 0) ;
 signal Net_292 : std_logic_vector(11 downto 0) ;
 signal Net_293 : std_logic_vector(11 downto 0) ;
 signal Net_294 : std_logic_vector(11 downto 0) ;
 signal Net_295 : std_logic_vector(11 downto 0) ;
 signal Net_296 : std_logic_vector(11 downto 0) ;
 signal Net_297 : std_logic_vector(11 downto 0) ;
 signal Net_298 : std_logic_vector(11 downto 0) ;
 signal Net_299 : std_logic_vector(11 downto 0) ;
 signal Net_300 : std_logic_vector(11 downto 0) ;
 signal Net_301 : std_logic_vector(11 downto 0) ;
 signal Net_302 : std_logic_vector(11 downto 0) ;
 signal Net_303 : std_logic_vector(11 downto 0) ;
 signal Net_304 : std_logic_vector(11 downto 0) ;
 signal Net_305 : std_logic_vector(11 downto 0) ;
 signal Net_306 : std_logic_vector(11 downto 0) ;
 signal Net_307 : std_logic_vector(11 downto 0) ;
 signal Net_308 : std_logic_vector(11 downto 0) ;
 signal Net_309 : std_logic_vector(11 downto 0) ;
 signal Net_310 : std_logic_vector(11 downto 0) ;
 signal Net_311 : std_logic_vector(11 downto 0) ;
 signal Net_312 : std_logic_vector(11 downto 0) ;
 signal Net_313 : std_logic_vector(11 downto 0) ;
 signal Net_314 : std_logic_vector(11 downto 0) ;
 signal Net_315 : std_logic_vector(11 downto 0) ;
 signal Net_316 : std_logic_vector(11 downto 0) ;
 signal Net_317 : std_logic_vector(11 downto 0) ;
 signal Net_318 : std_logic_vector(11 downto 0) ;
 signal Net_319 : std_logic_vector(11 downto 0) ;
 signal Net_320 : std_logic_vector(11 downto 0) ;
 signal Net_321 : std_logic_vector(11 downto 0) ;
 signal Net_322 : std_logic_vector(11 downto 0) ;
 signal Net_323 : std_logic_vector(11 downto 0) ;
 signal Net_324 : std_logic_vector(11 downto 0) ;
 signal Net_325 : std_logic_vector(11 downto 0) ;
 signal Net_326 : std_logic_vector(11 downto 0) ;
 signal Net_327 : std_logic_vector(11 downto 0) ;
 signal Net_328 : std_logic_vector(11 downto 0) ;
 signal Net_329 : std_logic_vector(11 downto 0) ;
 signal Net_330 : std_logic_vector(11 downto 0) ;
 signal Net_331 : std_logic_vector(11 downto 0) ;
 signal Net_332 : std_logic_vector(11 downto 0) ;
 signal Net_333 : std_logic_vector(11 downto 0) ;
 signal Net_334 : std_logic_vector(11 downto 0) ;
 signal Net_335 : std_logic_vector(11 downto 0) ;
 signal Net_336 : std_logic_vector(11 downto 0) ;
 signal Net_337 : std_logic_vector(11 downto 0) ;
 signal Net_338 : std_logic_vector(11 downto 0) ;
 signal Net_339 : std_logic_vector(11 downto 0) ;
 signal Net_340 : std_logic_vector(11 downto 0) ;
 signal Net_341 : std_logic_vector(11 downto 0) ;
 signal Net_342 : std_logic_vector(11 downto 0) ;
 signal Net_343 : std_logic_vector(11 downto 0) ;
 signal Net_344 : std_logic_vector(11 downto 0) ;
 signal Net_345 : std_logic_vector(11 downto 0) ;
 signal Net_346 : std_logic_vector(11 downto 0) ;
 signal Net_347 : std_logic_vector(11 downto 0) ;
 signal Net_348 : std_logic_vector(11 downto 0) ;
 signal Net_349 : std_logic_vector(11 downto 0) ;
 signal Net_350 : std_logic_vector(11 downto 0) ;
 signal Net_351 : std_logic_vector(11 downto 0) ;
 signal Net_352 : std_logic_vector(11 downto 0) ;
 signal Net_353 : std_logic_vector(11 downto 0) ;
 signal Net_354 : std_logic_vector(11 downto 0) ;
 signal Net_355 : std_logic_vector(11 downto 0) ;
 signal Net_356 : std_logic_vector(11 downto 0) ;
 signal Net_357 : std_logic_vector(11 downto 0) ;
 signal Net_358 : std_logic_vector(11 downto 0) ;
 signal Net_359 : std_logic_vector(11 downto 0) ;
 signal Net_360 : std_logic_vector(11 downto 0) ;
 signal Net_361 : std_logic_vector(11 downto 0) ;
 signal Net_362 : std_logic_vector(11 downto 0) ;
 signal Net_363 : std_logic_vector(11 downto 0) ;
 signal Net_364 : std_logic_vector(11 downto 0) ;
 signal Net_365 : std_logic_vector(11 downto 0) ;
 signal Net_366 : std_logic_vector(11 downto 0) ;
 signal Net_367 : std_logic_vector(11 downto 0) ;
 signal Net_368 : std_logic_vector(11 downto 0) ;
 signal Net_369 : std_logic_vector(11 downto 0) ;
 signal Net_370 : std_logic_vector(11 downto 0) ;
 signal Net_371 : std_logic_vector(11 downto 0) ;
 signal Net_372 : std_logic_vector(11 downto 0) ;
 signal Net_373 : std_logic_vector(11 downto 0) ;
 signal Net_374 : std_logic_vector(11 downto 0) ;
 signal Net_375 : std_logic_vector(11 downto 0) ;
 signal Net_376 : std_logic_vector(11 downto 0) ;
 signal Net_377 : std_logic_vector(11 downto 0) ;
 signal Net_378 : std_logic_vector(11 downto 0) ;
 signal Net_379 : std_logic_vector(11 downto 0) ;
 signal Net_380 : std_logic_vector(11 downto 0) ;
 signal Net_381 : std_logic_vector(11 downto 0) ;
 signal Net_382 : std_logic_vector(11 downto 0) ;
 signal Net_383 : std_logic_vector(11 downto 0) ;
 signal Net_384 : std_logic_vector(11 downto 0) ;
 signal Net_385 : std_logic_vector(11 downto 0) ;
 signal Net_388 : std_logic_vector(17 downto 0) ;
 signal Net_389 : std_logic_vector(17 downto 0) ;
 signal Net_390 : std_logic_vector(17 downto 0) ;
 signal Net_391 : std_logic_vector(17 downto 0) ;
 signal Net_392 : std_logic_vector(17 downto 0) ;
 signal Net_393 : std_logic_vector(17 downto 0) ;
 signal Net_394 : std_logic_vector(17 downto 0) ;
 signal Net_395 : std_logic_vector(17 downto 0) ;
 signal Net_396 : std_logic_vector(17 downto 0) ;
 signal Net_397 : std_logic_vector(17 downto 0) ;
 signal Net_398 : std_logic_vector(17 downto 0) ;
 signal Net_399 : std_logic_vector(17 downto 0) ;
 signal Net_400 : std_logic_vector(17 downto 0) ;
 signal Net_401 : std_logic_vector(17 downto 0) ;
 signal Net_402 : std_logic_vector(17 downto 0) ;
 signal Net_403 : std_logic_vector(17 downto 0) ;
 signal Net_404 : std_logic_vector(17 downto 0) ;
 signal Net_405 : std_logic_vector(17 downto 0) ;
 signal Net_406 : std_logic_vector(17 downto 0) ;
 signal Net_407 : std_logic_vector(17 downto 0) ;
 signal Net_408 : std_logic_vector(17 downto 0) ;
 signal Net_409 : std_logic_vector(17 downto 0) ;
 signal Net_410 : std_logic_vector(17 downto 0) ;
 signal Net_411 : std_logic_vector(17 downto 0) ;
 signal Net_412 : std_logic_vector(17 downto 0) ;
 signal Net_413 : std_logic_vector(17 downto 0) ;
 signal Net_414 : std_logic_vector(17 downto 0) ;
 signal Net_415 : std_logic_vector(17 downto 0) ;
 signal Net_416 : std_logic_vector(17 downto 0) ;
 signal Net_417 : std_logic_vector(17 downto 0) ;
 signal Net_418 : std_logic_vector(17 downto 0) ;
 signal Net_419 : std_logic_vector(17 downto 0) ;
 signal Net_420 : std_logic_vector(17 downto 0) ;
 signal Net_421 : std_logic_vector(17 downto 0) ;
 signal Net_422 : std_logic_vector(17 downto 0) ;
 signal Net_423 : std_logic_vector(17 downto 0) ;
 signal Net_424 : std_logic_vector(17 downto 0) ;
 signal Net_425 : std_logic_vector(17 downto 0) ;
 signal Net_426 : std_logic_vector(17 downto 0) ;
 signal Net_427 : std_logic_vector(17 downto 0) ;
 signal Net_428 : std_logic_vector(17 downto 0) ;
 signal Net_429 : std_logic_vector(17 downto 0) ;
 signal Net_430 : std_logic_vector(17 downto 0) ;
 signal Net_431 : std_logic_vector(17 downto 0) ;
 signal Net_432 : std_logic_vector(17 downto 0) ;
 signal Net_433 : std_logic_vector(17 downto 0) ;
 signal Net_434 : std_logic_vector(17 downto 0) ;
 signal Net_435 : std_logic_vector(17 downto 0) ;
 signal Net_436 : std_logic_vector(17 downto 0) ;
 signal Net_437 : std_logic_vector(17 downto 0) ;
 signal Net_438 : std_logic_vector(17 downto 0) ;
 signal Net_439 : std_logic_vector(17 downto 0) ;
 signal Net_440 : std_logic_vector(17 downto 0) ;
 signal Net_441 : std_logic_vector(17 downto 0) ;
 signal Net_442 : std_logic_vector(17 downto 0) ;
 signal Net_443 : std_logic_vector(17 downto 0) ;
 signal Net_444 : std_logic_vector(17 downto 0) ;
 signal Net_445 : std_logic_vector(17 downto 0) ;
 signal Net_446 : std_logic_vector(17 downto 0) ;
 signal Net_447 : std_logic_vector(17 downto 0) ;
 signal Net_448 : std_logic_vector(17 downto 0) ;
 signal Net_449 : std_logic_vector(17 downto 0) ;
 signal Net_450 : std_logic_vector(17 downto 0) ;
 signal Net_451 : std_logic_vector(17 downto 0) ;
 signal Net_452 : std_logic_vector(17 downto 0) ;
 signal Net_453 : std_logic_vector(17 downto 0) ;
 signal Net_454 : std_logic_vector(17 downto 0) ;
 signal Net_455 : std_logic_vector(17 downto 0) ;
 signal Net_456 : std_logic_vector(17 downto 0) ;
 signal Net_457 : std_logic_vector(17 downto 0) ;
 signal Net_458 : std_logic_vector(17 downto 0) ;
 signal Net_459 : std_logic_vector(17 downto 0) ;
 signal Net_460 : std_logic_vector(17 downto 0) ;
 signal Net_461 : std_logic_vector(17 downto 0) ;
 signal Net_462 : std_logic_vector(17 downto 0) ;
 signal Net_463 : std_logic_vector(17 downto 0) ;
 signal Net_464 : std_logic_vector(17 downto 0) ;
 signal Net_465 : std_logic_vector(17 downto 0) ;
 signal Net_466 : std_logic_vector(17 downto 0) ;
 signal Net_467 : std_logic_vector(17 downto 0) ;
 signal Net_468 : std_logic_vector(17 downto 0) ;
 signal Net_469 : std_logic_vector(17 downto 0) ;
 signal Net_470 : std_logic_vector(17 downto 0) ;
 signal Net_471 : std_logic_vector(17 downto 0) ;
 signal Net_472 : std_logic_vector(17 downto 0) ;
 signal Net_473 : std_logic_vector(17 downto 0) ;
 signal Net_474 : std_logic_vector(17 downto 0) ;
 signal Net_475 : std_logic_vector(17 downto 0) ;
 signal Net_476 : std_logic_vector(17 downto 0) ;
 signal Net_477 : std_logic_vector(17 downto 0) ;
 signal Net_478 : std_logic_vector(17 downto 0) ;
 signal Net_479 : std_logic_vector(17 downto 0) ;
 signal Net_480 : std_logic_vector(17 downto 0) ;
 signal Net_481 : std_logic_vector(17 downto 0) ;
 signal Net_482 : std_logic_vector(17 downto 0) ;
 signal Net_483 : std_logic_vector(17 downto 0) ;
 signal Net_484 : std_logic_vector(17 downto 0) ;
 signal Net_485 : std_logic_vector(17 downto 0) ;
 signal Net_486 : std_logic_vector(17 downto 0) ;
 signal Net_487 : std_logic_vector(17 downto 0) ;
 signal Net_488 : std_logic_vector(17 downto 0) ;
 signal Net_489 : std_logic_vector(17 downto 0) ;
 signal Net_490 : std_logic_vector(17 downto 0) ;
 signal Net_491 : std_logic_vector(17 downto 0) ;
 signal Net_492 : std_logic_vector(17 downto 0) ;
 signal Net_493 : std_logic_vector(17 downto 0) ;
 signal Net_494 : std_logic_vector(17 downto 0) ;
 signal Net_495 : std_logic_vector(17 downto 0) ;
 signal Net_496 : std_logic_vector(17 downto 0) ;
 signal Net_497 : std_logic_vector(17 downto 0) ;
 signal Net_498 : std_logic_vector(17 downto 0) ;
 signal Net_499 : std_logic_vector(17 downto 0) ;
 signal Net_500 : std_logic_vector(17 downto 0) ;
 signal Net_501 : std_logic_vector(17 downto 0) ;
 signal Net_502 : std_logic_vector(17 downto 0) ;
 signal Net_503 : std_logic_vector(17 downto 0) ;
 signal Net_504 : std_logic_vector(17 downto 0) ;
 signal Net_505 : std_logic_vector(17 downto 0) ;
 signal Net_506 : std_logic_vector(17 downto 0) ;
 signal Net_507 : std_logic_vector(17 downto 0) ;
 signal Net_508 : std_logic_vector(17 downto 0) ;
 signal Net_509 : std_logic_vector(17 downto 0) ;
 signal Net_510 : std_logic_vector(17 downto 0) ;
 signal Net_511 : std_logic_vector(17 downto 0) ;
 signal Net_512 : std_logic_vector(17 downto 0) ;
 signal Net_513 : std_logic_vector(17 downto 0) ;
 signal Net_514 : std_logic_vector(17 downto 0) ;
 signal Net_518 : std_logic ;
 signal Net_520 : std_logic ;

begin

 u0: Stage01
    port map(
      S01_r_0 => Net_258,
      S01_r_1 => Net_259,
      S01_r_2 => Net_260,
      S01_r_3 => Net_261,
      S01_r_4 => Net_262,
      S01_r_5 => Net_263,
      S01_r_6 => Net_264,
      S01_r_7 => Net_265,
      S01_r_8 => Net_266,
      S01_r_9 => Net_267,
      S01_r_10 => Net_268,
      S01_r_11 => Net_269,
      S01_r_12 => Net_270,
      S01_r_13 => Net_271,
      S01_r_14 => Net_272,
      S01_r_15 => Net_273,
      S01_r_16 => Net_274,
      S01_r_17 => Net_275,
      S01_r_18 => Net_276,
      S01_r_19 => Net_277,
      S01_r_20 => Net_278,
      S01_r_21 => Net_279,
      S01_r_22 => Net_280,
      S01_r_23 => Net_281,
      S01_r_24 => Net_282,
      S01_r_25 => Net_283,
      S01_r_26 => Net_284,
      S01_r_27 => Net_285,
      S01_r_28 => Net_286,
      S01_r_29 => Net_287,
      S01_r_30 => Net_288,
      S01_r_31 => Net_289,
      S01_r_32 => Net_290,
      S01_r_33 => Net_291,
      S01_r_34 => Net_292,
      S01_r_35 => Net_293,
      S01_r_36 => Net_294,
      S01_r_37 => Net_295,
      S01_r_38 => Net_296,
      S01_r_39 => Net_297,
      S01_r_40 => Net_298,
      S01_r_41 => Net_299,
      S01_r_42 => Net_300,
      S01_r_43 => Net_301,
      S01_r_44 => Net_302,
      S01_r_45 => Net_303,
      S01_r_46 => Net_304,
      S01_r_47 => Net_305,
      S01_r_48 => Net_306,
      S01_r_49 => Net_307,
      S01_r_50 => Net_308,
      S01_r_51 => Net_309,
      S01_r_52 => Net_310,
      S01_r_53 => Net_311,
      S01_r_54 => Net_312,
      S01_r_55 => Net_313,
      S01_r_56 => Net_314,
      S01_r_57 => Net_315,
      S01_r_58 => Net_316,
      S01_r_59 => Net_317,
      S01_r_60 => Net_318,
      S01_r_61 => Net_319,
      S01_r_62 => Net_320,
      S01_r_63 => Net_321,
      S01_i_0 => Net_322,
      S01_i_1 => Net_323,
      S01_i_2 => Net_324,
      S01_i_3 => Net_325,
      S01_i_4 => Net_326,
      S01_i_5 => Net_327,
      S01_i_6 => Net_328,
      S01_i_7 => Net_329,
      S01_i_8 => Net_330,
      S01_i_9 => Net_331,
      S01_i_10 => Net_332,
      S01_i_11 => Net_333,
      S01_i_12 => Net_334,
      S01_i_13 => Net_335,
      S01_i_14 => Net_336,
      S01_i_15 => Net_337,
      S01_i_16 => Net_338,
      S01_i_17 => Net_339,
      S01_i_18 => Net_340,
      S01_i_19 => Net_341,
      S01_i_20 => Net_342,
      S01_i_21 => Net_343,
      S01_i_22 => Net_344,
      S01_i_23 => Net_345,
      S01_i_24 => Net_346,
      S01_i_25 => Net_347,
      S01_i_26 => Net_348,
      S01_i_27 => Net_349,
      S01_i_28 => Net_350,
      S01_i_29 => Net_351,
      S01_i_30 => Net_352,
      S01_i_31 => Net_353,
      S01_i_32 => Net_354,
      S01_i_33 => Net_355,
      S01_i_34 => Net_356,
      S01_i_35 => Net_357,
      S01_i_36 => Net_358,
      S01_i_37 => Net_359,
      S01_i_38 => Net_360,
      S01_i_39 => Net_361,
      S01_i_40 => Net_362,
      S01_i_41 => Net_363,
      S01_i_42 => Net_364,
      S01_i_43 => Net_365,
      S01_i_44 => Net_366,
      S01_i_45 => Net_367,
      S01_i_46 => Net_368,
      S01_i_47 => Net_369,
      S01_i_48 => Net_370,
      S01_i_49 => Net_371,
      S01_i_50 => Net_372,
      S01_i_51 => Net_373,
      S01_i_52 => Net_374,
      S01_i_53 => Net_375,
      S01_i_54 => Net_376,
      S01_i_55 => Net_377,
      S01_i_56 => Net_378,
      S01_i_57 => Net_379,
      S01_i_58 => Net_380,
      S01_i_59 => Net_381,
      S01_i_60 => Net_382,
      S01_i_61 => Net_383,
      S01_i_62 => Net_384,
      S01_i_63 => Net_385,
      S01_outr_0 => Net_127,
      S01_outr_1 => Net_126,
      S01_outr_2 => Net_125,
      S01_outr_3 => Net_124,
      S01_outr_4 => Net_123,
      S01_outr_5 => Net_122,
      S01_outr_6 => Net_121,
      S01_outr_7 => Net_120,
      S01_outr_8 => Net_119,
      S01_outr_9 => Net_118,
      S01_outr_10 => Net_117,
      S01_outr_11 => Net_116,
      S01_outr_12 => Net_115,
      S01_outr_13 => Net_114,
      S01_outr_14 => Net_113,
      S01_outr_15 => Net_112,
      S01_outr_16 => Net_111,
      S01_outr_17 => Net_110,
      S01_outr_18 => Net_109,
      S01_outr_19 => Net_108,
      S01_outr_20 => Net_107,
      S01_outr_21 => Net_106,
      S01_outr_22 => Net_105,
      S01_outr_23 => Net_104,
      S01_outr_24 => Net_103,
      S01_outr_25 => Net_101,
      S01_outr_26 => Net_102,
      S01_outr_27 => Net_100,
      S01_outr_28 => Net_99,
      S01_outr_29 => Net_98,
      S01_outr_30 => Net_97,
      S01_outr_31 => Net_96,
      S01_outr_32 => Net_95,
      S01_outr_33 => Net_94,
      S01_outr_34 => Net_93,
      S01_outr_35 => Net_92,
      S01_outr_36 => Net_91,
      S01_outr_37 => Net_90,
      S01_outr_38 => Net_89,
      S01_outr_39 => Net_88,
      S01_outr_40 => Net_87,
      S01_outr_41 => Net_86,
      S01_outr_42 => Net_85,
      S01_outr_43 => Net_84,
      S01_outr_44 => Net_83,
      S01_outr_45 => Net_82,
      S01_outr_46 => Net_81,
      S01_outr_47 => Net_80,
      S01_outr_48 => Net_79,
      S01_outr_49 => Net_78,
      S01_outr_50 => Net_77,
      S01_outr_51 => Net_76,
      S01_outr_52 => Net_75,
      S01_outr_53 => Net_74,
      S01_outr_54 => Net_73,
      S01_outr_55 => Net_72,
      S01_outr_56 => Net_71,
      S01_outr_57 => Net_70,
      S01_outr_58 => Net_69,
      S01_outr_59 => Net_68,
      S01_outr_60 => Net_67,
      S01_outr_61 => Net_66,
      S01_outr_62 => Net_65,
      S01_outr_63 => Net_64,
      S01_outi_0 => Net_63,
      S01_outi_1 => Net_62,
      S01_outi_2 => Net_61,
      S01_outi_3 => Net_60,
      S01_outi_4 => Net_59,
      S01_outi_5 => Net_58,
      S01_outi_6 => Net_57,
      S01_outi_7 => Net_56,
      S01_outi_8 => Net_55,
      S01_outi_9 => Net_54,
      S01_outi_10 => Net_53,
      S01_outi_11 => Net_52,
      S01_outi_12 => Net_51,
      S01_outi_13 => Net_50,
      S01_outi_14 => Net_49,
      S01_outi_15 => Net_48,
      S01_outi_16 => Net_47,
      S01_outi_17 => Net_46,
      S01_outi_18 => Net_45,
      S01_outi_19 => Net_44,
      S01_outi_20 => Net_43,
      S01_outi_21 => Net_41,
      S01_outi_22 => Net_42,
      S01_outi_23 => Net_40,
      S01_outi_24 => Net_39,
      S01_outi_25 => Net_38,
      S01_outi_26 => Net_37,
      S01_outi_27 => Net_36,
      S01_outi_28 => Net_35,
      S01_outi_29 => Net_34,
      S01_outi_30 => Net_33,
      S01_outi_31 => Net_32,
      S01_outi_32 => Net_31,
      S01_outi_33 => Net_30,
      S01_outi_34 => Net_29,
      S01_outi_35 => Net_28,
      S01_outi_36 => Net_27,
      S01_outi_37 => Net_26,
      S01_outi_38 => Net_25,
      S01_outi_39 => Net_24,
      S01_outi_40 => Net_23,
      S01_outi_41 => Net_22,
      S01_outi_42 => Net_21,
      S01_outi_43 => Net_20,
      S01_outi_44 => Net_19,
      S01_outi_45 => Net_18,
      S01_outi_46 => Net_17,
      S01_outi_47 => Net_16,
      S01_outi_48 => Net_15,
      S01_outi_49 => Net_14,
      S01_outi_50 => Net_13,
      S01_outi_51 => Net_12,
      S01_outi_52 => Net_11,
      S01_outi_53 => Net_10,
      S01_outi_54 => Net_9,
      S01_outi_55 => Net_8,
      S01_outi_56 => Net_7,
      S01_outi_58 => Net_5,
      S01_outi_59 => Net_4,
      S01_outi_57 => Net_6,
      S01_outi_60 => Net_3,
      S01_outi_61 => Net_2,
      S01_outi_62 => Net_1,
      S01_outi_63 => Net_0,
      clk => clk,
      reset => reset ) ;

 u1: Stage02
    port map(
      S02_outr_0 => Net_256,
      S02_outr_1 => Net_255,
      S02_outr_2 => Net_254,
      S02_outr_3 => Net_253,
      S02_outr_4 => Net_252,
      S02_outr_5 => Net_251,
      S02_outr_6 => Net_250,
      S02_outr_7 => Net_249,
      S02_outr_8 => Net_248,
      S02_outr_9 => Net_247,
      S02_outr_10 => Net_246,
      S02_outr_11 => Net_245,
      S02_outr_12 => Net_244,
      S02_outr_13 => Net_243,
      S02_outr_14 => Net_242,
      S02_outr_15 => Net_240,
      S02_outr_16 => Net_241,
      S02_outr_17 => Net_239,
      S02_outr_18 => Net_238,
      S02_outr_19 => Net_236,
      S02_outr_20 => Net_237,
      S02_outr_21 => Net_235,
      S02_outr_22 => Net_234,
      S02_outr_23 => Net_233,
      S02_outr_24 => Net_232,
      S02_outr_25 => Net_231,
      S02_outr_26 => Net_230,
      S02_outr_27 => Net_229,
      S02_outr_28 => Net_228,
      S02_outr_29 => Net_227,
      S02_outr_30 => Net_226,
      S02_outr_31 => Net_225,
      S02_outr_32 => Net_224,
      S02_outr_33 => Net_223,
      S02_outr_34 => Net_222,
      S02_outr_35 => Net_221,
      S02_outr_36 => Net_220,
      S02_outr_37 => Net_219,
      S02_outr_38 => Net_218,
      S02_outr_39 => Net_217,
      S02_outr_40 => Net_216,
      S02_outr_41 => Net_215,
      S02_outr_42 => Net_214,
      S02_outr_43 => Net_213,
      S02_outr_44 => Net_212,
      S02_outr_45 => Net_211,
      S02_outr_46 => Net_210,
      S02_outr_47 => Net_209,
      S02_outr_48 => Net_208,
      S02_outr_49 => Net_207,
      S02_outr_50 => Net_206,
      S02_outr_51 => Net_205,
      S02_outr_52 => Net_204,
      S02_outr_53 => Net_203,
      S02_outr_54 => Net_202,
      S02_outr_55 => Net_201,
      S02_outr_56 => Net_200,
      S02_outr_57 => Net_199,
      S02_outr_58 => Net_198,
      S02_outr_59 => Net_197,
      S02_outr_60 => Net_196,
      S02_outr_61 => Net_195,
      S02_outr_62 => Net_194,
      S02_outr_63 => Net_193,
      S02_outi_0 => Net_191,
      S02_outi_1 => Net_192,
      S02_outi_2 => Net_190,
      S02_outi_3 => Net_189,
      S02_outi_4 => Net_188,
      S02_outi_5 => Net_187,
      S02_outi_6 => Net_186,
      S02_outi_7 => Net_185,
      S02_outi_8 => Net_184,
      S02_outi_9 => Net_183,
      S02_outi_10 => Net_182,
      S02_outi_11 => Net_181,
      S02_outi_12 => Net_180,
      S02_outi_13 => Net_179,
      S02_outi_14 => Net_178,
      S02_outi_15 => Net_177,
      S02_outi_16 => Net_176,
      S02_outi_17 => Net_175,
      S02_outi_18 => Net_174,
      S02_outi_19 => Net_173,
      S02_outi_20 => Net_172,
      S02_outi_21 => Net_171,
      S02_outi_22 => Net_170,
      S02_outi_23 => Net_169,
      S02_outi_24 => Net_168,
      S02_outi_25 => Net_167,
      S02_outi_26 => Net_166,
      S02_outi_27 => Net_165,
      S02_outi_28 => Net_164,
      S02_outi_29 => Net_163,
      S02_outi_30 => Net_162,
      S02_outi_31 => Net_161,
      S02_outi_32 => Net_160,
      S02_outi_33 => Net_159,
      S02_outi_34 => Net_158,
      S02_outi_35 => Net_157,
      S02_outi_36 => Net_156,
      S02_outi_37 => Net_155,
      S02_outi_38 => Net_154,
      S02_outi_39 => Net_153,
      S02_outi_40 => Net_152,
      S02_outi_41 => Net_151,
      S02_outi_42 => Net_150,
      S02_outi_43 => Net_149,
      S02_outi_44 => Net_148,
      S02_outi_45 => Net_147,
      S02_outi_46 => Net_146,
      S02_outi_47 => Net_145,
      S02_outi_48 => Net_144,
      S02_outi_49 => Net_143,
      S02_outi_50 => Net_142,
      S02_outi_51 => Net_141,
      S02_outi_52 => Net_140,
      S02_outi_53 => Net_139,
      S02_outi_54 => Net_136,
      S02_outi_55 => Net_138,
      S02_outi_56 => Net_137,
      S02_outi_58 => Net_134,
      S02_outi_59 => Net_133,
      S02_outi_57 => Net_135,
      S02_outi_60 => Net_132,
      S02_outi_61 => Net_131,
      S02_outi_62 => Net_130,
      S02_outi_63 => Net_129,
      S02_r_0 => Net_127,
      S02_r_1 => Net_126,
      S02_r_2 => Net_125,
      S02_r_3 => Net_124,
      S02_r_4 => Net_123,
      S02_r_5 => Net_122,
      S02_r_6 => Net_121,
      S02_r_7 => Net_120,
      S02_r_8 => Net_119,
      S02_r_9 => Net_118,
      S02_r_10 => Net_117,
      S02_r_11 => Net_116,
      S02_r_12 => Net_115,
      S02_r_13 => Net_114,
      S02_r_14 => Net_113,
      S02_r_15 => Net_112,
      S02_r_16 => Net_111,
      S02_r_17 => Net_110,
      S02_r_18 => Net_109,
      S02_r_19 => Net_108,
      S02_r_20 => Net_107,
      S02_r_21 => Net_106,
      S02_r_22 => Net_105,
      S02_r_23 => Net_104,
      S02_r_24 => Net_103,
      S02_r_25 => Net_101,
      S02_r_26 => Net_102,
      S02_r_27 => Net_100,
      S02_r_28 => Net_99,
      S02_r_29 => Net_98,
      S02_r_30 => Net_97,
      S02_r_31 => Net_96,
      S02_r_32 => Net_95,
      S02_r_33 => Net_94,
      S02_r_34 => Net_93,
      S02_r_35 => Net_92,
      S02_r_36 => Net_91,
      S02_r_37 => Net_90,
      S02_r_38 => Net_89,
      S02_r_39 => Net_88,
      S02_r_40 => Net_87,
      S02_r_41 => Net_86,
      S02_r_42 => Net_85,
      S02_r_43 => Net_84,
      S02_r_44 => Net_83,
      S02_r_45 => Net_82,
      S02_r_46 => Net_81,
      S02_r_47 => Net_80,
      S02_r_48 => Net_79,
      S02_r_49 => Net_78,
      S02_r_50 => Net_77,
      S02_r_51 => Net_76,
      S02_r_52 => Net_75,
      S02_r_53 => Net_74,
      S02_r_54 => Net_73,
      S02_r_55 => Net_72,
      S02_r_56 => Net_71,
      S02_r_57 => Net_70,
      S02_r_58 => Net_69,
      S02_r_59 => Net_68,
      S02_r_60 => Net_67,
      S02_r_61 => Net_66,
      S02_r_62 => Net_65,
      S02_r_63 => Net_64,
      S02_i_0 => Net_63,
      S02_i_1 => Net_62,
      S02_i_2 => Net_61,
      S02_i_3 => Net_60,
      S02_i_4 => Net_59,
      S02_i_5 => Net_58,
      S02_i_6 => Net_57,
      S02_i_7 => Net_56,
      S02_i_8 => Net_55,
      S02_i_9 => Net_54,
      S02_i_10 => Net_53,
      S02_i_11 => Net_52,
      S02_i_12 => Net_51,
      S02_i_13 => Net_50,
      S02_i_14 => Net_49,
      S02_i_15 => Net_48,
      S02_i_16 => Net_47,
      S02_i_17 => Net_46,
      S02_i_18 => Net_45,
      S02_i_19 => Net_44,
      S02_i_20 => Net_43,
      S02_i_21 => Net_41,
      S02_i_22 => Net_42,
      S02_i_23 => Net_40,
      S02_i_24 => Net_39,
      S02_i_25 => Net_38,
      S02_i_26 => Net_37,
      S02_i_27 => Net_36,
      S02_i_28 => Net_35,
      S02_i_29 => Net_34,
      S02_i_30 => Net_33,
      S02_i_31 => Net_32,
      S02_i_32 => Net_31,
      S02_i_33 => Net_30,
      S02_i_34 => Net_29,
      S02_i_35 => Net_28,
      S02_i_36 => Net_27,
      S02_i_37 => Net_26,
      S02_i_38 => Net_25,
      S02_i_39 => Net_24,
      S02_i_40 => Net_23,
      S02_i_41 => Net_22,
      S02_i_42 => Net_21,
      S02_i_43 => Net_20,
      S02_i_44 => Net_19,
      S02_i_45 => Net_18,
      S02_i_46 => Net_17,
      S02_i_47 => Net_16,
      S02_i_48 => Net_15,
      S02_i_49 => Net_14,
      S02_i_50 => Net_13,
      S02_i_51 => Net_12,
      S02_i_52 => Net_11,
      S02_i_53 => Net_10,
      S02_i_54 => Net_9,
      S02_i_55 => Net_8,
      S02_i_56 => Net_7,
      S02_i_57 => Net_6,
      S02_i_58 => Net_5,
      S02_i_59 => Net_4,
      S02_i_60 => Net_3,
      S02_i_61 => Net_2,
      S02_i_62 => Net_1,
      S02_i_63 => Net_0,
      clk => clk,
      reset => reset ) ;

 u2: Stage03
    port map(
      S03_outr_0 => Net_514,
      S03_outr_1 => Net_513,
      S03_outr_2 => Net_512,
      S03_outr_3 => Net_511,
      S03_outr_4 => Net_510,
      S03_outr_5 => Net_509,
      S03_outr_6 => Net_508,
      S03_outr_7 => Net_507,
      S03_outr_8 => Net_506,
      S03_outr_9 => Net_505,
      S03_outr_10 => Net_504,
      S03_outr_11 => Net_503,
      S03_outr_12 => Net_502,
      S03_outr_13 => Net_501,
      S03_outr_14 => Net_500,
      S03_outr_15 => Net_499,
      S03_outr_16 => Net_498,
      S03_outr_17 => Net_497,
      S03_outr_18 => Net_496,
      S03_outr_19 => Net_495,
      S03_outr_20 => Net_494,
      S03_outr_21 => Net_493,
      S03_outr_22 => Net_492,
      S03_outr_23 => Net_491,
      S03_outr_24 => Net_490,
      S03_outr_25 => Net_489,
      S03_outr_26 => Net_488,
      S03_outr_27 => Net_487,
      S03_outr_28 => Net_486,
      S03_outr_29 => Net_485,
      S03_outr_30 => Net_484,
      S03_outr_31 => Net_483,
      S03_outr_32 => Net_481,
      S03_outr_33 => Net_482,
      S03_outr_34 => Net_480,
      S03_outr_35 => Net_479,
      S03_outr_36 => Net_477,
      S03_outr_37 => Net_478,
      S03_outr_38 => Net_476,
      S03_outr_39 => Net_475,
      S03_outr_40 => Net_474,
      S03_outr_41 => Net_473,
      S03_outr_42 => Net_472,
      S03_outr_43 => Net_471,
      S03_outr_44 => Net_470,
      S03_outr_45 => Net_468,
      S03_outr_46 => Net_469,
      S03_outr_47 => Net_467,
      S03_outr_48 => Net_466,
      S03_outr_49 => Net_465,
      S03_outr_50 => Net_464,
      S03_outr_51 => Net_463,
      S03_outr_52 => Net_462,
      S03_outr_53 => Net_461,
      S03_outr_54 => Net_460,
      S03_outr_55 => Net_459,
      S03_outr_56 => Net_458,
      S03_outr_57 => Net_457,
      S03_outr_58 => Net_455,
      S03_outr_59 => Net_456,
      S03_outr_60 => Net_454,
      S03_outr_61 => Net_453,
      S03_outr_62 => Net_452,
      S03_outr_63 => Net_451,
      S03_outi_0 => Net_450,
      S03_outi_1 => Net_449,
      S03_outi_2 => Net_448,
      S03_outi_3 => Net_447,
      S03_outi_4 => Net_446,
      S03_outi_5 => Net_445,
      S03_outi_6 => Net_444,
      S03_outi_7 => Net_443,
      S03_outi_8 => Net_441,
      S03_outi_9 => Net_442,
      S03_outi_10 => Net_440,
      S03_outi_11 => Net_438,
      S03_outi_12 => Net_439,
      S03_outi_13 => Net_437,
      S03_outi_14 => Net_436,
      S03_outi_15 => Net_434,
      S03_outi_16 => Net_435,
      S03_outi_17 => Net_433,
      S03_outi_18 => Net_432,
      S03_outi_19 => Net_431,
      S03_outi_20 => Net_430,
      S03_outi_21 => Net_429,
      S03_outi_22 => Net_428,
      S03_outi_23 => Net_427,
      S03_outi_24 => Net_426,
      S03_outi_25 => Net_425,
      S03_outi_26 => Net_424,
      S03_outi_27 => Net_423,
      S03_outi_28 => Net_422,
      S03_outi_29 => Net_421,
      S03_outi_30 => Net_420,
      S03_outi_31 => Net_419,
      S03_outi_32 => Net_418,
      S03_outi_33 => Net_417,
      S03_outi_34 => Net_416,
      S03_outi_35 => Net_415,
      S03_outi_36 => Net_414,
      S03_outi_37 => Net_413,
      S03_outi_38 => Net_412,
      S03_outi_39 => Net_410,
      S03_outi_40 => Net_411,
      S03_outi_41 => Net_409,
      S03_outi_42 => Net_408,
      S03_outi_43 => Net_407,
      S03_outi_44 => Net_406,
      S03_outi_45 => Net_405,
      S03_outi_46 => Net_404,
      S03_outi_47 => Net_403,
      S03_outi_48 => Net_402,
      S03_outi_49 => Net_401,
      S03_outi_50 => Net_400,
      S03_outi_51 => Net_399,
      S03_outi_52 => Net_398,
      S03_outi_53 => Net_397,
      S03_outi_54 => Net_396,
      S03_outi_55 => Net_395,
      S03_outi_56 => Net_394,
      S03_outi_58 => Net_391,
      S03_outi_59 => Net_392,
      S03_outi_57 => Net_393,
      S03_outi_60 => Net_390,
      S03_outi_61 => Net_389,
      S03_outi_62 => Net_388,
      S03_outi_63 => Net_257,
      S03_r_0 => Net_256,
      S03_r_1 => Net_255,
      S03_r_2 => Net_254,
      S03_r_3 => Net_253,
      S03_r_4 => Net_252,
      S03_r_5 => Net_251,
      S03_r_6 => Net_250,
      S03_r_7 => Net_249,
      S03_r_8 => Net_248,
      S03_r_9 => Net_247,
      S03_r_10 => Net_246,
      S03_r_11 => Net_245,
      S03_r_12 => Net_244,
      S03_r_13 => Net_243,
      S03_r_14 => Net_242,
      S03_r_15 => Net_240,
      S03_r_16 => Net_241,
      S03_r_17 => Net_239,
      S03_r_18 => Net_238,
      S03_r_19 => Net_236,
      S03_r_20 => Net_237,
      S03_r_21 => Net_235,
      S03_r_22 => Net_234,
      S03_r_23 => Net_233,
      S03_r_24 => Net_232,
      S03_r_25 => Net_231,
      S03_r_26 => Net_230,
      S03_r_27 => Net_229,
      S03_r_28 => Net_228,
      S03_r_29 => Net_227,
      S03_r_30 => Net_226,
      S03_r_31 => Net_225,
      S03_r_32 => Net_224,
      S03_r_33 => Net_223,
      S03_r_34 => Net_222,
      S03_r_35 => Net_221,
      S03_r_36 => Net_220,
      S03_r_37 => Net_219,
      S03_r_38 => Net_218,
      S03_r_39 => Net_217,
      S03_r_40 => Net_216,
      S03_r_41 => Net_215,
      S03_r_42 => Net_214,
      S03_r_43 => Net_213,
      S03_r_44 => Net_212,
      S03_r_45 => Net_211,
      S03_r_46 => Net_210,
      S03_r_47 => Net_209,
      S03_r_48 => Net_208,
      S03_r_49 => Net_207,
      S03_r_50 => Net_206,
      S03_r_51 => Net_205,
      S03_r_52 => Net_204,
      S03_r_53 => Net_203,
      S03_r_54 => Net_202,
      S03_r_55 => Net_201,
      S03_r_56 => Net_200,
      S03_r_57 => Net_199,
      S03_r_58 => Net_198,
      S03_r_59 => Net_197,
      S03_r_60 => Net_196,
      S03_r_61 => Net_195,
      S03_r_62 => Net_194,
      S03_r_63 => Net_193,
      S03_i_0 => Net_191,
      S03_i_1 => Net_192,
      S03_i_2 => Net_190,
      S03_i_3 => Net_189,
      S03_i_4 => Net_188,
      S03_i_5 => Net_187,
      S03_i_6 => Net_186,
      S03_i_7 => Net_185,
      S03_i_8 => Net_184,
      S03_i_9 => Net_183,
      S03_i_10 => Net_182,
      S03_i_11 => Net_181,
      S03_i_12 => Net_180,
      S03_i_13 => Net_179,
      S03_i_14 => Net_178,
      S03_i_15 => Net_177,
      S03_i_16 => Net_176,
      S03_i_17 => Net_175,
      S03_i_18 => Net_174,
      S03_i_19 => Net_173,
      S03_i_20 => Net_172,
      S03_i_21 => Net_171,
      S03_i_22 => Net_170,
      S03_i_23 => Net_169,
      S03_i_24 => Net_168,
      S03_i_25 => Net_167,
      S03_i_26 => Net_166,
      S03_i_27 => Net_165,
      S03_i_28 => Net_164,
      S03_i_29 => Net_163,
      S03_i_30 => Net_162,
      S03_i_31 => Net_161,
      S03_i_32 => Net_160,
      S03_i_33 => Net_159,
      S03_i_34 => Net_158,
      S03_i_35 => Net_157,
      S03_i_36 => Net_156,
      S03_i_37 => Net_155,
      S03_i_38 => Net_154,
      S03_i_39 => Net_153,
      S03_i_40 => Net_152,
      S03_i_41 => Net_151,
      S03_i_42 => Net_150,
      S03_i_43 => Net_149,
      S03_i_44 => Net_148,
      S03_i_45 => Net_147,
      S03_i_46 => Net_146,
      S03_i_47 => Net_145,
      S03_i_48 => Net_144,
      S03_i_49 => Net_143,
      S03_i_50 => Net_142,
      S03_i_51 => Net_141,
      S03_i_52 => Net_140,
      S03_i_53 => Net_139,
      S03_i_54 => Net_136,
      S03_i_55 => Net_138,
      S03_i_56 => Net_137,
      S03_i_57 => Net_135,
      S03_i_58 => Net_134,
      S03_i_59 => Net_133,
      S03_i_60 => Net_132,
      S03_i_61 => Net_131,
      S03_i_62 => Net_130,
      S03_i_63 => Net_129,
      clk => clk,
      reset => reset ) ;

 u3: Shift_in
    port map(
      Shift_in_r_0 => Net_258,
      Shift_in_r_1 => Net_259,
      Shift_in_r_2 => Net_260,
      Shift_in_r_3 => Net_261,
      Shift_in_r_4 => Net_262,
      Shift_in_r_5 => Net_263,
      Shift_in_r_6 => Net_264,
      Shift_in_r_7 => Net_265,
      Shift_in_r_8 => Net_266,
      Shift_in_r_9 => Net_267,
      Shift_in_r_10 => Net_268,
      Shift_in_r_11 => Net_269,
      Shift_in_r_12 => Net_270,
      Shift_in_r_13 => Net_271,
      Shift_in_r_14 => Net_272,
      Shift_in_r_15 => Net_273,
      Shift_in_r_16 => Net_274,
      Shift_in_r_17 => Net_275,
      Shift_in_r_18 => Net_276,
      Shift_in_r_19 => Net_277,
      Shift_in_r_20 => Net_278,
      Shift_in_r_21 => Net_279,
      Shift_in_r_22 => Net_280,
      Shift_in_r_23 => Net_281,
      Shift_in_r_24 => Net_282,
      Shift_in_r_25 => Net_283,
      Shift_in_r_26 => Net_284,
      Shift_in_r_27 => Net_285,
      Shift_in_r_28 => Net_286,
      Shift_in_r_29 => Net_287,
      Shift_in_r_30 => Net_288,
      Shift_in_r_31 => Net_289,
      Shift_in_r_32 => Net_290,
      Shift_in_r_33 => Net_291,
      Shift_in_r_34 => Net_292,
      Shift_in_r_35 => Net_293,
      Shift_in_r_36 => Net_294,
      Shift_in_r_37 => Net_295,
      Shift_in_r_38 => Net_296,
      Shift_in_r_39 => Net_297,
      Shift_in_r_40 => Net_298,
      Shift_in_r_41 => Net_299,
      Shift_in_r_42 => Net_300,
      Shift_in_r_43 => Net_301,
      Shift_in_r_44 => Net_302,
      Shift_in_r_45 => Net_303,
      Shift_in_r_46 => Net_304,
      Shift_in_r_47 => Net_305,
      Shift_in_r_48 => Net_306,
      Shift_in_r_49 => Net_307,
      Shift_in_r_50 => Net_308,
      Shift_in_r_51 => Net_309,
      Shift_in_r_52 => Net_310,
      Shift_in_r_53 => Net_311,
      Shift_in_r_54 => Net_312,
      Shift_in_r_55 => Net_313,
      Shift_in_r_56 => Net_314,
      Shift_in_r_57 => Net_315,
      Shift_in_r_58 => Net_316,
      Shift_in_r_59 => Net_317,
      Shift_in_r_60 => Net_318,
      Shift_in_r_61 => Net_319,
      Shift_in_r_62 => Net_320,
      Shift_in_r_63 => Net_321,
      Shift_in_i_0 => Net_322,
      Shift_in_i_1 => Net_323,
      Shift_in_i_2 => Net_324,
      Shift_in_i_3 => Net_325,
      Shift_in_i_4 => Net_326,
      Shift_in_i_5 => Net_327,
      Shift_in_i_6 => Net_328,
      Shift_in_i_7 => Net_329,
      Shift_in_i_8 => Net_330,
      Shift_in_i_9 => Net_331,
      Shift_in_i_10 => Net_332,
      Shift_in_i_11 => Net_333,
      Shift_in_i_12 => Net_334,
      Shift_in_i_13 => Net_335,
      Shift_in_i_14 => Net_336,
      Shift_in_i_15 => Net_337,
      Shift_in_i_16 => Net_338,
      Shift_in_i_17 => Net_339,
      Shift_in_i_18 => Net_340,
      Shift_in_i_19 => Net_341,
      Shift_in_i_20 => Net_342,
      Shift_in_i_21 => Net_343,
      Shift_in_i_22 => Net_344,
      Shift_in_i_23 => Net_345,
      Shift_in_i_24 => Net_346,
      Shift_in_i_25 => Net_347,
      Shift_in_i_26 => Net_348,
      Shift_in_i_27 => Net_349,
      Shift_in_i_28 => Net_350,
      Shift_in_i_29 => Net_351,
      Shift_in_i_30 => Net_352,
      Shift_in_i_31 => Net_353,
      Shift_in_i_32 => Net_354,
      Shift_in_i_33 => Net_355,
      Shift_in_i_34 => Net_356,
      Shift_in_i_35 => Net_357,
      Shift_in_i_36 => Net_358,
      Shift_in_i_37 => Net_359,
      Shift_in_i_38 => Net_360,
      Shift_in_i_39 => Net_361,
      Shift_in_i_40 => Net_362,
      Shift_in_i_41 => Net_363,
      Shift_in_i_42 => Net_364,
      Shift_in_i_43 => Net_365,
      Shift_in_i_44 => Net_366,
      Shift_in_i_45 => Net_367,
      Shift_in_i_46 => Net_368,
      Shift_in_i_47 => Net_369,
      Shift_in_i_48 => Net_370,
      Shift_in_i_49 => Net_371,
      Shift_in_i_50 => Net_372,
      Shift_in_i_51 => Net_373,
      Shift_in_i_52 => Net_374,
      Shift_in_i_53 => Net_375,
      Shift_in_i_54 => Net_376,
      Shift_in_i_55 => Net_377,
      Shift_in_i_56 => Net_378,
      Shift_in_i_58 => Net_380,
      Shift_in_i_59 => Net_381,
      Shift_in_i_57 => Net_379,
      Shift_in_i_60 => Net_382,
      Shift_in_i_61 => Net_383,
      Shift_in_i_62 => Net_384,
      Shift_in_i_63 => Net_385,
      Shift_in_r => Shift_in_r,
      Shift_in_i => Shift_in_i,
      clk => clk,
      reset => reset,
      done => Net_518,
      Data_valid => Data_valid_in ) ;

 u4: Shift_out
    port map(
      Shift_out_r_0 => Net_514,
      Shift_out_r_1 => Net_513,
      Shift_out_r_2 => Net_512,
      Shift_out_r_3 => Net_511,
      Shift_out_r_4 => Net_510,
      Shift_out_r_5 => Net_509,
      Shift_out_r_6 => Net_508,
      Shift_out_r_7 => Net_507,
      Shift_out_r_8 => Net_506,
      Shift_out_r_9 => Net_505,
      Shift_out_r_10 => Net_504,
      Shift_out_r_11 => Net_503,
      Shift_out_r_12 => Net_502,
      Shift_out_r_13 => Net_501,
      Shift_out_r_14 => Net_500,
      Shift_out_r_15 => Net_499,
      Shift_out_r_16 => Net_498,
      Shift_out_r_17 => Net_497,
      Shift_out_r_18 => Net_496,
      Shift_out_r_19 => Net_495,
      Shift_out_r_20 => Net_494,
      Shift_out_r_21 => Net_493,
      Shift_out_r_22 => Net_492,
      Shift_out_r_23 => Net_491,
      Shift_out_r_24 => Net_490,
      Shift_out_r_25 => Net_489,
      Shift_out_r_26 => Net_488,
      Shift_out_r_27 => Net_487,
      Shift_out_r_28 => Net_486,
      Shift_out_r_29 => Net_485,
      Shift_out_r_30 => Net_484,
      Shift_out_r_31 => Net_483,
      Shift_out_r_32 => Net_481,
      Shift_out_r_33 => Net_482,
      Shift_out_r_34 => Net_480,
      Shift_out_r_35 => Net_479,
      Shift_out_r_36 => Net_477,
      Shift_out_r_37 => Net_478,
      Shift_out_r_38 => Net_476,
      Shift_out_r_39 => Net_475,
      Shift_out_r_40 => Net_474,
      Shift_out_r_41 => Net_473,
      Shift_out_r_42 => Net_472,
      Shift_out_r_43 => Net_471,
      Shift_out_r_44 => Net_470,
      Shift_out_r_45 => Net_468,
      Shift_out_r_46 => Net_469,
      Shift_out_r_47 => Net_467,
      Shift_out_r_48 => Net_466,
      Shift_out_r_49 => Net_465,
      Shift_out_r_50 => Net_464,
      Shift_out_r_51 => Net_463,
      Shift_out_r_52 => Net_462,
      Shift_out_r_53 => Net_461,
      Shift_out_r_54 => Net_460,
      Shift_out_r_55 => Net_459,
      Shift_out_r_56 => Net_458,
      Shift_out_r_57 => Net_457,
      Shift_out_r_58 => Net_455,
      Shift_out_r_59 => Net_456,
      Shift_out_r_60 => Net_454,
      Shift_out_r_61 => Net_453,
      Shift_out_r_62 => Net_452,
      Shift_out_r_63 => Net_451,
      Shift_out_i_0 => Net_450,
      Shift_out_i_1 => Net_449,
      Shift_out_i_2 => Net_448,
      Shift_out_i_3 => Net_447,
      Shift_out_i_4 => Net_446,
      Shift_out_i_5 => Net_445,
      Shift_out_i_6 => Net_444,
      Shift_out_i_7 => Net_443,
      Shift_out_i_8 => Net_441,
      Shift_out_i_9 => Net_442,
      Shift_out_i_10 => Net_440,
      Shift_out_i_11 => Net_438,
      Shift_out_i_12 => Net_439,
      Shift_out_i_13 => Net_437,
      Shift_out_i_14 => Net_436,
      Shift_out_i_15 => Net_434,
      Shift_out_i_16 => Net_435,
      Shift_out_i_17 => Net_433,
      Shift_out_i_18 => Net_432,
      Shift_out_i_19 => Net_431,
      Shift_out_i_20 => Net_430,
      Shift_out_i_21 => Net_429,
      Shift_out_i_22 => Net_428,
      Shift_out_i_23 => Net_427,
      Shift_out_i_24 => Net_426,
      Shift_out_i_25 => Net_425,
      Shift_out_i_26 => Net_424,
      Shift_out_i_27 => Net_423,
      Shift_out_i_28 => Net_422,
      Shift_out_i_29 => Net_421,
      Shift_out_i_30 => Net_420,
      Shift_out_i_31 => Net_419,
      Shift_out_i_32 => Net_418,
      Shift_out_i_33 => Net_417,
      Shift_out_i_34 => Net_416,
      Shift_out_i_35 => Net_415,
      Shift_out_i_36 => Net_414,
      Shift_out_i_37 => Net_413,
      Shift_out_i_38 => Net_412,
      Shift_out_i_39 => Net_410,
      Shift_out_i_40 => Net_411,
      Shift_out_i_41 => Net_409,
      Shift_out_i_42 => Net_408,
      Shift_out_i_43 => Net_407,
      Shift_out_i_44 => Net_406,
      Shift_out_i_45 => Net_405,
      Shift_out_i_46 => Net_404,
      Shift_out_i_47 => Net_403,
      Shift_out_i_48 => Net_402,
      Shift_out_i_49 => Net_401,
      Shift_out_i_50 => Net_400,
      Shift_out_i_51 => Net_399,
      Shift_out_i_52 => Net_398,
      Shift_out_i_53 => Net_397,
      Shift_out_i_54 => Net_396,
      Shift_out_i_55 => Net_395,
      Shift_out_i_56 => Net_394,
      Shift_out_i_57 => Net_393,
      Shift_out_i_58 => Net_391,
      Shift_out_i_59 => Net_392,
      Shift_out_i_60 => Net_390,
      Shift_out_i_61 => Net_389,
      Shift_out_i_62 => Net_388,
      Shift_out_i_63 => Net_257,
      Shift_out_r => Shift_out_r,
      Shift_out_i => Shift_out_i,
      clk => clk,
      valid_data => Net_520,
      Data_valid_out => Data_valid_out,
      reset => reset ) ;

 u5: Valid
    port map(
      done_shift_in => Net_518,
      clk => clk,
      reset => reset,
      data_valid => Net_520 ) ;
end  a0 ; -- of Top_Level
